/*---------------------------------------------------------------------------*/
/*                              PMS                                          */
/*---------------------------------------------------------------------------*/
/*    Copyright (c) Becker & Hickl GmbH  1997-2002  All Rights Reserved.     */
/*---------------------------------------------------------------------------*/
/*                                                                           */
/* Title   : PMS_DEF.H                                                       */
/* Purpose : Include file for programs using PMS300(400)  32-bit DLL under   */
/*              Windows9x/ME/NT/2K/XP.                                       */
/* Note    :                                                                 */
/*                                                                           */
/*   Functions listed here are exported in   pmsdll32.dll                    */
/*                                                                           */
/*   Use proper Import Library File pmsdll32.lib depending on used compiler  */
/*    ( Borland C++ v.4 , Watcom C , Microsoft Visual C and  Symantec C      */
/*        compilers are supported )                                          */
/*                                                                           */
/*---------------------------------------------------------------------------*/

/*- Defines -----------------------------------------------------------------*/

  /* needed if linking DLL in external compiler; harmless otherwise */
#include "cvirte.h"

#ifndef CVICDECL
#define CVICDECL
#endif

#ifdef __cplusplus
  extern "C"{
#endif


#pragma pack(push)
#pragma pack(1)

#define  PMS_APP_REV  274
#define  PMS_DLL_REV  250

#define  APP_REV_TEXT  "2.74"
#define  DLL_REV_TEXT  "2.50"


#define MAX_NO_OF_PMS        8

/*- Error codes  ------------------------------------------------------------*/

#define OK                0    /* no error */

#define  PMSDLL_ERROR_KEYWORDS \
{ \
  keyword (PMS_NONE), \
  keyword (PMS_OPEN_FILE), \
  keyword (PMS_FILE_NVALID), \
  keyword (PMS_MEM_ALLOC), \
  keyword (PMS_READ_STR), \
  keyword (PMS_NO_ACT_MOD), \
  keyword (PMS_MOD_NO), \
  keyword (PMS_WRONG_ID), \
  keyword (PMS_NOT_ACTIVE), \
  keyword (PMS_EEPROM_READ), \
  keyword (PMS_EEPROM_WRITE), \
  keyword (PMS_EEP_CHKSUM), \
  keyword (PMS_EEP_WR_DIS), \
  keyword (PMS_DIF_VALUE), \
  keyword (PMS_BAD_SYNC), \
  keyword (PMS_COUNT_RD), \
  keyword (PMS_ERR_COUNT), \
  keyword (PMS_ERR_DAC), \
  keyword (PMS_SYS_ARMED), \
  keyword (PMS_NOT_ARMED), \
  keyword (PMS_NOT_STOP), \
  keyword (PMS_CANT_START), \
  keyword (PMS_EEP_NOT_USE), \
  keyword (PMS_BAD_PAR_ID), \
  keyword (PMS_PAR_NOT_SET), \
  keyword (PMS_WRONG_ADR), \
  keyword (PMS_CTIME_OVFL), \
  keyword (PMS_MTIME_OVFL), \
  keyword (PMS_WINDRV_ERR), \
  keyword (PMS_PCI_CARD_NO), \
  keyword (PMS_NOT_INIT), \
  keyword (PMS_ERR_ARM), \
  keyword (PMS_ERR_TRIG), \
  keyword (PMS_ERR_N_TRIG), \
  keyword (PMS_DMA_ERR), \
  keyword (PMS_WINDRV_VER), \
  keyword (PMS_LICENSE_ERR), \
  };

/*
   NONE            0       no error
   OPEN_FILE      -1       can't open configuration file
   FILE_NVALID    -2       not valid configuration file
   MEM_ALLOC      -3       can't allocate memory
   READ_STR       -4       can't read string from PMS configuration file
   NO_ACT_MOD     -5       no active modules
   MOD_NO         -6       module number out of range
   WRONG_ID       -7       wrong module ID
   NOT_ACTIVE     -8       can't execute function - module not active
   EEPROM_READ    -9       error during reading EEPROM
   EEPROM_WRITE   -10      error during writing to EEPROM
   EEP_CHKSUM     -11      wrong EEPROM checksum
   EEP_WR_DIS     -12      cannot execute - write access to EEPROM denied
   DIF_VALUE      -13      read value different than in data structure
   BAD_SYNC       -14      SYNC address cannot be the same as base address
   COUNT_RD       -15      can't read counter
   ERR_COUNT      -16      wrong value read from counter
   ERR_DAC        -17      wrong value in DAC's test
   SYS_ARMED      -18      cannot execute the function  - system is armed
   NOT_ARMED      -19      cannot stop the measurement - system is not armed
   NOT_STOP       -20      cannot stop the measurement in module
   CANT_START     -21      not able to find any module which can be started
   EEP_NOT_USE    -22      cannot read EEPROM - EEPROM is not used
   BAD_PAR_ID     -23      unknown PMS parameter id
   PAR_NOT_SET    -24      parameter can be set only during initialization
   WRONG_ADR      -25      to big PMS memory address
   CTIME_OVFL     -26      Collection time counter overflow - measurement aborted
   MTIME_OVFL     -27      Macro time counter overflow - measurement aborted
   WINDRV_ERR     -28      Error in WinDriver - test pms_err_string
   PCI_CARD_NO    -29      wrong PCI bus module number or module in use
   NOT_INIT       -30      PMS is not yet initialized or unknown module type
   ERR_ARM        -31      Trigger test - cannot arm the module
   ERR_TRIG       -32      Trigger test - triggered without the pulse
   ERR_N_TRIG     -33      Trigger test - not triggered after the trigger pulse
   DMA_ERR        -34      DMA Transfer Error
   WINDRV_VER     -35      Incorrect WinDriver version
   LICENSE_ERR    -36      Wrong or missing license key
*/


#define  PMS_PARAMETERS_KEYWORDS \
{ \
  keyword (BASE_ADR), \
  keyword (ACTIVE), \
  keyword (MEAS_MODE), \
  keyword (ENABLE_MEAS), \
  keyword (TRIGGER), \
  keyword (GATE_LEVEL_A), \
  keyword (GATE_LEVEL_B), \
  keyword (INP_THRESHOLD_A), \
  keyword (INP_THRESHOLD_B), \
  keyword (EVENT_THRESHOLD_A), \
  keyword (EVENT_THRESHOLD_B), \
  keyword (COLLECT_TIME), \
  keyword (START_PTR), \
  keyword (END_PTR), \
  keyword (ACCUMULATE), \
  keyword (MACRO_TIME), \
  keyword (PCI_CARD_NO), \
  keyword (TRIG_THRESHOLD), \
  keyword (PCI_BUS_NO), \
  keyword (SWEEPS), \
  keyword (INP_HOLDOFF), \
  keyword (OUT12V), \
};


#ifdef keyword
#undef keyword
#endif

#define keyword(key) key

enum pmsdll_error_enum     PMSDLL_ERROR_KEYWORDS
enum pms_parameters_enum   PMS_PARAMETERS_KEYWORDS


/*---------------------------------------------------------------------------*/
/*- Defines -----------------------------------------------------------------*/

#define PMS300_ID           0x9     /* correct PMS300 module id */


      /* possible modes of DLL operation - returned from PMS_set(get)_mode */
#define PMS_HARD             0     /* hardware mode */
#define PMS_SIMUL300         30    /* simulation mode of PMS300 */
#define PMS_SIMUL400         40    /* simulation mode of PMS400 */
#define PMS_SIMUL400A        41    /* simulation mode of PMS400A */

    /* supported module types  - returned value from PMS_test_id */
#define M_WRONG_TYPE  -1
#define M_PMS300    30
#define M_PMS400    40
#define M_PMS400A   41

    /* measurement mode defines - possible values of parameter MEAS_MODE */
#define MULTISCALER_MODE  0
#define EVENT_MODE  1



    /*  initialization error codes -
           - possible return values of function PMS_get_init_status */
#define INIT_OK                    0
#define INIT_NOT_DONE             -1 /* active = 0  - initialization not done */
#define INIT_WRONG_EEP_CHKSUM     -2 /* wrong EEPROM checksum */
#define INIT_WRONG_MOD_ID         -3 /* wrong module identification code */
#define INIT_WRONG_BASE_ADR       -4 /* not unique base address */
#define INIT_WRONG_SYNC_ADR       -5 /* sync address equal to base address */
#define INIT_WRONG_COUNTER        -6 /* counter test failed */
#define INIT_WRONG_DACS           -7 /* DAC's test failed */
#define INIT_CANT_OPEN_PCI_CARD   -8 /* cannot open PCI card */
#define INIT_MOD_IN_USE           -9 /* module already in use */
#define INIT_WINDRVR_VER         -10 /* incorrect WinDriver version */
#define INIT_WRONG_LICENSE       -11 /* Wrong or missing license key */




               /* bits definitions for STATUS register */

#define MT_OV             0x8000  /* Macro Time overflow */
#define MT_EXP            0x4000  /* Macro Time expired (=0) */
#define CT_OV             0x2000  /* Collection Timer overflow */
#define CT_EXP            0x1000  /* Collection Timer expired (=0) */
#define NEXT              0x800   /* Collect Timer expired when STOP_CT was set
                                      - next point will be measured */
#define B_OVFL            0x200   /* Overflow on counter B */
#define A_OVFL            0x100   /* Overflow on counter A */
#define DIS_ARM           0x80    /* external hardware signals,that is not ready */
                                  /*   for arming */
#define DIS_COL           0x40    /* start of next collection time interval is
                                       suppressed by external hardware */
#define MEASURE           0x2     /* module collects data (Armed & Triggered) */
#define ARMED             0x1     /* module is armed by software */
           // for PMS-400 with FPGA version > 0x301
#define WTRG              0x8     /* waits for trigger when HWARMED = 1*/
#define HWARMED           0x4     /* hardware is armed (ARMED = 1 & DIS_ARM = 0 */


/*---------------------------------------------------------------------------*/


typedef struct Pmsdata{          /* structure for PMS module data  */

  unsigned short base_adr;       /*  I/O base address */
  short init;                    /* set to initialization result code */
  short active;                  /* most of the library functions are executed
                                    only when module is active ( not 0 )*/
  short test_eep;                /* test EEPROM checksum on startup or not  */
  short meas_mode;               /* measurement mode */
  short enable_meas;             /* measurement enabled/disabled(1/0) */
  float gate_level_A;            /* gate  A disciminator level  */
  float gate_level_B;            /* gate  B disciminator level  */
  float inp_threshold_A;         /* input threshold level of channel A  */
  float inp_threshold_B;         /* input threshold level of channel A  */
  unsigned short event_threshold_A;  /* event's threshold A */
  unsigned short event_threshold_B;  /* event's threshold B */
  float collect_time;            /* collection time interval */
  unsigned short start_ptr;      /* memory start pointer */
  unsigned short end_ptr;        /* memory end pointer */
  short accumulate;              /* accumulate or overwrite values in memory */
  short trigger;                 /* external trigger condition -
                                     none(0),active low(1),active high(2) */
  float macro_time;              /* makro collection time in event mode */
  float trig_threshold;          /* trigger threshold level for PCI modules */
  unsigned long sweeps;          /* no of accumulation sweeps
                                  0 (PMS-300 and PMS-400 with FPGA version < 0x301),
                             0(default) ... 0xffffffff for
                             PMS-400 with FPGA version >= 0x301 and PMS-400A */
  short pci_card_no;             /* slot no for PCI module(0-3) or -1 for ISA module */
  unsigned short inp_holdoff;    /* inputs holdoff 1..15(default), only for PMS-400A
                                defines holdoff time = 10.0 / inp_holdoff [ns]  */
  short out12v;                  /* 12V output active (1) / not active (0, default ),
                                    only for PMS-400A */
  }PMSdata;


              /* structure for module info */
typedef struct _PMSModInfo{
  short    module_type;      /* module type */
  short    bus_number;       /* PCI bus number */
  short    slot_number;      /* slot number on PCI bus */
  short    in_use;           /* -1 used and locked by other application, 0 - not used
                                 1 - in use  */
  short    init;             /* set to initialisation result code */
  unsigned short base_adr;   /* base I/O address */
  }PMSModInfo;




     /* structure containing PMS adjust parameters stored in EEPROM */
typedef struct _PMS_Adjust_Para{       /* all offsets in DAC steps */
  char gate_offset_A;              /* gate offset of channel A */
  char gate_offset_B;              /* gate offset of channel B */
  char thr_offset_A;               /* input threshold offset of channel A */
  char thr_offset_B;               /* input threshold offset of channel B */
  char trig_offset;                /* trigger threshold offset */
  }PMS_Adjust_Para;



typedef struct _PMS_EEP_Data{     /* structure for PMS module EEPROM data  */
  char module_type[16];           /* PMS module type */
  char serial_no[16];             /* PMS module serial number */
  char date[16];                  /* PMS module production date */
  float gate_min;                 /* gate threshold low limit */
  float gate_max;                 /* gate threshold high limit */
  float trig_min;                 /* trigger threshold low limit */
  float trig_max;                 /* trigger threshold high limit */
  float thr_min;                  /* input threshold low limit */
  float thr_max;                  /* input threshold high limit */
  PMS_Adjust_Para adj_para;       /* structure with adjust parameters */
  }PMS_EEP_Data;



/*
 CVIDECL means C calling convention - default for C and C++ programs
 DLLSTDCALL means _stdcall calling convention which is required by Visual Basic

 The only difference between PMSstd_.. and PMS_.. functions is calling convention

*/


/*-------------------------------------------------------------------------*/
/*- Initialisation functions  ---------------------------------------------*/
/*-------------------------------------------------------------------------*/
short  CVICDECL    PMS_init (char *ini_file);
short  DLLSTDCALL  PMSstd_init (char *ini_file);

short  CVICDECL    PMS_test_if_active(short mod_no);
short  DLLSTDCALL  PMSstd_test_if_active(short mod_no);

short  CVICDECL    PMS_get_init_status(short mod_no,short * ini_status);
short  DLLSTDCALL  PMSstd_get_init_status(short mod_no,short * ini_status);

short  CVICDECL    PMS_get_sync_adr(short *adr);
short  DLLSTDCALL  PMSstd_get_sync_adr(short *adr);

short  CVICDECL    PMS_set_sync_adr(short adr);
short  DLLSTDCALL  PMSstd_set_sync_adr(short adr);

short  CVICDECL    PMS_get_mode (void);
short  DLLSTDCALL  PMSstd_get_mode (void);

short  CVICDECL    PMS_set_mode (short mode, short force_use, short *in_use);
short  DLLSTDCALL  PMSstd_set_mode (short mode, short force_use, short *in_use);

short  CVICDECL    PMS_get_version (short mod_no,short *version);
short  DLLSTDCALL  PMSstd_get_version (short mod_no,short *version);

short  CVICDECL    PMS_get_module_info(short mod_no, PMSModInfo * mod_info);
short  DLLSTDCALL  PMSstd_get_module_info(short mod_no, PMSModInfo * mod_info);



/*-------------------------------------------------------------------------*/
/*- Setup functions  ------------------------------------------------------*/
/*-------------------------------------------------------------------------*/

short  CVICDECL    PMS_get_parameter(short mod_no, short par_id, float * value);
short  DLLSTDCALL  PMSstd_get_parameter(short mod_no, short par_id, float * value);

short  CVICDECL    PMS_set_parameter(short mod_no, short par_id, float value);
short  DLLSTDCALL  PMSstd_set_parameter(short mod_no, short par_id, float value);

short  CVICDECL    PMS_get_parameters(short mod_no,PMSdata *pmsdata);
short  DLLSTDCALL  PMSstd_get_parameters(short mod_no,PMSdata *pmsdata);

short  CVICDECL    PMS_set_parameters(short mod_no,PMSdata *pmsdata);
short  DLLSTDCALL  PMSstd_set_parameters(short mod_no,PMSdata *pmsdata);

short  CVICDECL    PMS_get_eeprom_data (short mod_no,PMS_EEP_Data *eep_data);
short  DLLSTDCALL  PMSstd_get_eeprom_data (short mod_no,PMS_EEP_Data *eep_data);

short  CVICDECL    PMS_write_eeprom_data (short mod_no,unsigned short write_enable,
                                        PMS_EEP_Data *eep_data);
short  DLLSTDCALL  PMSstd_write_eeprom_data (short mod_no,unsigned short write_enable,
                                        PMS_EEP_Data *eep_data);

short  CVICDECL    PMS_get_adjust_parameters (short mod_no,
                                            PMS_Adjust_Para * adjpara);
short  DLLSTDCALL  PMSstd_get_adjust_parameters (short mod_no,
                                            PMS_Adjust_Para * adjpara);

short  CVICDECL    PMS_set_adjust_parameters (short mod_no,
                                            PMS_Adjust_Para * adjpara);
short  DLLSTDCALL  PMSstd_set_adjust_parameters (short mod_no,
                                            PMS_Adjust_Para * adjpara);



/*-------------------------------------------------------------------------*/
/*- Status  functions  ----------------------------------------------------*/
/*-------------------------------------------------------------------------*/

short  CVICDECL    PMS_test_if_busy(short *busy);
short  DLLSTDCALL  PMSstd_test_if_busy(short *busy);

short  CVICDECL    PMS_read_status(short mod_no,unsigned short *status);
short  DLLSTDCALL  PMSstd_read_status(short mod_no,unsigned short *status);

short  CVICDECL    PMS_get_macro_time(short mod_no, double * time);
short  DLLSTDCALL  PMSstd_get_macro_time(short mod_no, double * time);

short  CVICDECL    PMS_get_points_no(short mod_no, unsigned long * points_no);
short  DLLSTDCALL  PMSstd_get_points_no(short mod_no, unsigned long * points_no);

short  CVICDECL    PMS_get_sweep(short mod_no, unsigned long * sweep);
short  DLLSTDCALL  PMSstd_get_sweep(short mod_no, unsigned long * sweep);


/*-------------------------------------------------------------------------*/
/*- Measurement control functions  ----------------------------------------*/
/*-------------------------------------------------------------------------*/

short  CVICDECL    PMS_start_measure(void);
short  DLLSTDCALL  PMSstd_start_measure(void);

short  CVICDECL    PMS_stop_measure(void);
short  DLLSTDCALL  PMSstd_stop_measure(void);



/*-------------------------------------------------------------------------*/
/*- PMS memory transfer functions  ----------------------------------------*/
/*-------------------------------------------------------------------------*/

short  CVICDECL    PMS_fill_memory(short mod_no,short channel,
                            unsigned short from,unsigned short to,
                            unsigned long fill_value);
short  DLLSTDCALL  PMSstd_fill_memory(short mod_no,short channel,
                            unsigned short from,unsigned short to,
                            unsigned long fill_value);

short  CVICDECL    PMS_read_data(short mod_no,unsigned long * points_read,
                            unsigned short from,unsigned short to,
                            unsigned long * A_buf,unsigned long * B_buf);
short  DLLSTDCALL  PMSstd_read_data(short mod_no,unsigned long * points_read,
                            unsigned short from,unsigned short to,
                            unsigned long * A_buf,unsigned long * B_buf);

short  CVICDECL    PMS_read_events(short mod_no,unsigned long * events_read,
                            unsigned short from,unsigned short to,
                            unsigned long * A_data,unsigned long * B_data,
                            float * A_time,float * B_time);
short  DLLSTDCALL  PMSstd_read_events(short mod_no,unsigned long * events_read,
                            unsigned short from,unsigned short to,
                            unsigned long * A_data,unsigned long * B_data,
                            float * A_time,float * B_time);


/*-------------------------------------------------------------------------*/
/*- Test functions  -------------------------------------------------------*/
/*-------------------------------------------------------------------------*/

short  CVICDECL    PMS_test_id (short mod_no);
short  DLLSTDCALL  PMSstd_test_id (short mod_no);

short  CVICDECL    PMS_test_counter (short mod_no,short *A_active,
                                                short *B_active);
short  DLLSTDCALL  PMSstd_test_counter (short mod_no,short *A_active,
                                                short *B_active);

short  CVICDECL    PMS_get_test_error_string(char *error_string);
short  DLLSTDCALL  PMSstd_get_test_error_string(char *error_string);


/*- Miscellaneous -----------------------------------------------------------*/

short   CVICDECL     PMS_close (void);
short   DLLSTDCALL   PMSstd_close (void);



/*- The End -----------------------------------------------------------------*/

#pragma pack(pop)

#ifdef __cplusplus
  }
#endif
