#include "plot_MathProcessing.h"
#include "plot_SubtPlane.h"
#include "plot_MathFilters.h"

#include <QScriptEngine>
#include <QDebug>

namespace plot {
namespace math {

void TMathProcessing::SmartAddTask(const TTask &Task)
{
    Work[Task.Action] = Task;
}
void TMathProcessing::run()
{
    while (true) {
        bool set = false;
        for (int i = 1; i < FiltersSize; i++) {
            if (Work[i].Action == i) {
                ActiveTask = Work[i];
                Work[i].Action = TMathProcessing::Nothing;
                set = true;
                break;
            }
        }
        if (!set) break;

        try {
            //TODO func ptr table
            switch (ActiveTask.Action) {
            case TMathProcessing::Fit2d:
                _Fit2d();
                break;
            case TMathProcessing::Scale2d:
                _Scale2d();
                break;
            case TMathProcessing::Fit3dX:
                _Fit3d_XDir();
                break;
            case TMathProcessing::Fit3dY:
                _Fit3d_YDir();
                break;
            case TMathProcessing::Subtr:
                _Subtr();
                break;
            case TMathProcessing::LineMedianX:
            case TMathProcessing::LineMedianY:
                _LineCorrectByMediana();
                break;
            case TMathProcessing::MeanValue:
                _DoMeanValue();
                break;
            case TMathProcessing::MedianValue:
                _DoMedianValue();
                break;
            case TMathProcessing::Gaussian:
                _DoGaussian();
                break;
            case TMathProcessing::CustomFilter2d:
                _DoCustom2d();
                break;
            case TMathProcessing::CustomFilter3d:
                _DoCustom3d();
                break;
            };
        } catch (...) {
            qDebug() << "math exception";
        }
    }
}
void TMathProcessing::_Fit2d()
{
    for (int i = 0; i < Curves->size(); i++)
        _Fit2d(i);
}
void TMathProcessing::_Scale2d()
{
    if (ActiveTask.arg.toInt() < 0)
        return;

    bool scale = false;

    for (int i = 0; i < Curves->size(); i++)
        if (i == ActiveTask.arg.toInt() && (*Curves)[i] && (*Curves)[i]->size()) {
            scale = true;
            QPair<double, double> _minmax = minmax((*Curves)[i]->YSeq);
            mi = _minmax.first;
            mx = _minmax.second;

            break;
        }

    if (!scale)
        return;

    for (int i = 0; i < Curves->size(); i++)
        if (i != ActiveTask.arg.toInt() && (*Curves)[i])
            _Scale2d(i);
}

void TMathProcessing::DoManualFitting(int N, double *YArray, unsigned long Size)
{
    if (ManualCurves->QwtCurves.size() <= N)
        return;

    QwtPlotCurve &CurveForMF = ManualCurves->QwtCurves[N]->MCCurve;
    if (CurveForMF.dataSize() <= 1)
        return;

    quint32 Index = 0;
    double XMin = CurveForMF.sample(0).x();
    double YXMin = CurveForMF.sample(0).y();
    double XMax = CurveForMF.sample(CurveForMF.dataSize()-1).x();
    double YXMax = CurveForMF.sample(CurveForMF.dataSize()-1).y();
    bool dir = (XMin > XMax);
    if (dir) {
        qSwap(XMin, XMax);
        qSwap(YXMax, YXMin);
    }

    for (unsigned long i = 0; i < Size; i++) {
        double xpos = (*Curves)[N]->isCompressedXCurve() ? ((*Curves)[N]->XSeq.data()[0] + (*Curves)[N]->XSeq.data()[1] * i) : (*Curves)[N]->XSeq.data()[i];
        if (xpos <= XMin) {
            YArray[i] -= YXMin;
        } else if (xpos >= XMax) {
            YArray[i] -= YXMax;
        } else {
            while (Index <= CurveForMF.dataSize()-2 &&
                   (dir ?
                    xpos < CurveForMF.sample(Index+1).x() :
                    xpos > CurveForMF.sample(Index+1).x()) )
                Index++;

            double t = (xpos - CurveForMF.sample(Index).x()) / (CurveForMF.sample(Index + 1).x() - CurveForMF.sample(Index).x());
            YArray[i] -= QLineF(CurveForMF.sample(Index), CurveForMF.sample(Index + 1)).pointAt(t).y();
        }
    }
}

double ajust (double min, double max, double min2, double max2, double val)
{
    if (max2 == min2)
        return val;

    return min + (max - min)*(val - min2)/(max2 - min2);
}

void TMathProcessing::_Scale2d(int N)
{
    long Size = (*Curves)[N]->size();
    double *YArray = (*Curves)[N]->YSeq.data();

    QPair<double, double> _minmax = minmax((*Curves)[N]->YSeq);

    for (int i = 0; i < Size; i++)
        YArray[i] = ajust(mi, mx, _minmax.first, _minmax.second, YArray[i]);
}

void TMathProcessing::_Fit2d(int N)
{
    if (!(*Curves)[N] || ActiveTask.arg.toInt() == TPlotEnums::FO_None)
        return;

    long Size = (*Curves)[N]->size();
    double *YArray = (*Curves)[N]->YSeq.data();

    if (ActiveTask.arg.toInt() == TPlotEnums::FO_Manual) DoManualFitting(N, YArray, Size);
    else {
        double *Coeffs = dataLinePartFitPolynom(YArray, ActiveTask.arg.toInt(), 0, 0, Size);
        dataLinePartSubtractPolynom(YArray, ActiveTask.arg.toInt(), Coeffs, 0, Size);
        delete [] Coeffs;
    }
}
void TMathProcessing::_Fit3d_XDir()
{
    if (ActiveTask.arg.toInt() < 0)
        return;

    for (int i = 0; i < Data->size(); i++)
        _Fit3d_XDir(*Data->at(i));
}
void TMathProcessing::_Fit3d_XDir(TMin3dData &Min3dData)
{
    unsigned int MinX = 0, MaxX = 0, MinY = 0, MaxY = 0;
    SetupMinsMaxs(MinX, MaxX, MinY, MaxY, Min3dData);

    for (int i = 0; i < Min3dData.Array.size(); i++)
    {
        double *Coeffs = dataLinePartFitPolynom(&*Min3dData.Array[i].begin(), ActiveTask.arg.toInt(), 0, MinX, MaxX);
        dataLinePartSubtractPolynom(&*Min3dData.Array[i].begin(), ActiveTask.arg.toInt(), Coeffs, MinX, MaxX);
        delete [] Coeffs; Coeffs = 0;
    }
}
void TMathProcessing::_Fit3d_YDir()
{
    if (ActiveTask.arg.toInt() < 0)
        return;

    for (int i = 0; i < Data->size(); i++)
        _Fit3d_YDir(*Data->at(i));
}
void TMathProcessing::_Fit3d_YDir(TMin3dData &Min3dData)
{
    unsigned int MinX = 0, MaxX = 0, MinY = 0, MaxY = 0;
    SetupMinsMaxs(MinX, MaxX, MinY, MaxY, Min3dData);

    double *YArray = new double [MaxY - MinY];
    for (unsigned int XCounter = MinX; XCounter < MaxX; XCounter++)
    {
        for (unsigned int YCounter = MinY; YCounter < MaxY; YCounter++)
            YArray[YCounter - MinY] = Min3dData.Array[YCounter][XCounter];

        double *Coeffs = dataLinePartFitPolynom(YArray, ActiveTask.arg.toInt(), 0, 0, MaxY - MinY);
        dataLinePartSubtractPolynom(YArray, ActiveTask.arg.toInt(), Coeffs, 0, MaxY - MinY);
        delete [] Coeffs; Coeffs = 0;

        for (unsigned int YCounter = MinY; YCounter < MaxY; YCounter++)
            Min3dData.Array[YCounter][XCounter] = YArray[YCounter - MinY];
    }
    delete [] YArray;
}
void TMathProcessing::_Subtr()
{
    if (ActiveTask.arg.toInt() < 0)
        return;

    for (int i = 0; i < Data->size(); i++)
        _Subtr(*Data->at(i));
}
void TMathProcessing::_Subtr(TMin3dData &Min3dData)
{
    unsigned int MinX = 0, MaxX = 0, MinY = 0, MaxY = 0;
    SetupMinsMaxs(MinX, MaxX, MinY, MaxY, Min3dData);

    double *Coeffs = dataFieldAreaFitPolynom(Min3dData.Array, MinY, MinX, MaxY - MinY, MaxX - MinX, ActiveTask.arg.toInt(), ActiveTask.arg.toInt(), 0);
    dataFieldAreaSubtractPolynom(Min3dData.Array, MinY, MinX, MaxY - MinY, MaxX - MinX, ActiveTask.arg.toInt(), ActiveTask.arg.toInt(), Coeffs);
    delete [] Coeffs;
}
void TMathProcessing::_LineCorrectByMediana()
{
    TPlotEnums::TAxis Direction = TPlotEnums::A_XAxis;
    if (ActiveTask.Action == TMathProcessing::LineMedianY)
        Direction = TPlotEnums::A_YAxis;

    for (int i = 0; i < Data->size(); i++) {
        unsigned int MinX = 0, MaxX = 0, MinY = 0, MaxY = 0;
        SetupMinsMaxs(MinX, MaxX, MinY, MaxY, *Data->at(i));
        TCorrectLinesByMedian(&Data->at(i)->Array, Direction, MinX, MinY, MaxX, MaxY);
    }
}
void TMathProcessing::_DoMeanValue()
{
    for (int i = 0; i < Data->size(); i++) {
        unsigned int MinX = 0, MaxX = 0, MinY = 0, MaxY = 0;
        SetupMinsMaxs(MinX, MaxX, MinY, MaxY, *Data->at(i));
        TMeanValue(&Data->at(i)->Array, ActiveTask.arg.toInt(), MinX, MinY, MaxX, MaxY);
    }
}
void TMathProcessing::_DoMedianValue()
{
    for (int i = 0; i < Data->size(); i++) {
        unsigned int MinX = 0, MaxX = 0, MinY = 0, MaxY = 0;
        SetupMinsMaxs(MinX, MaxX, MinY, MaxY, *Data->at(i));
        TMedianValue(&Data->at(i)->Array, ActiveTask.arg.toInt(), MinX, MinY, MaxX, MaxY);
    }
}
void TMathProcessing::_DoGaussian()
{
    for (int i = 0; i < Data->size(); i++) {
        unsigned int MinX = 0, MaxX = 0, MinY = 0, MaxY = 0;
        SetupMinsMaxs(MinX, MaxX, MinY, MaxY, *Data->at(i));
        TGaussian(&Data->at(i)->Array, ActiveTask.arg.toInt(), MinX, MinY, MaxX, MaxY);
    }
}
void TMathProcessing::_DoCustom3d()
{
    QString flt = "function filter(x) { with(Math) { return ( %1 ); } }";
    flt = flt.arg(ActiveTask.arg.toString());

    QScriptEngine myEngine;
    myEngine.evaluate(flt);
    QScriptValue fun = myEngine.globalObject().property("filter");

    for (int i = 0; i < Data->size(); i++) {
        unsigned int MinX = 0, MaxX = 0, MinY = 0, MaxY = 0;
        SetupMinsMaxs(MinX, MaxX, MinY, MaxY, *Data->at(i));

        //TODO opencl, qtconcurrent, workerthreads
        for (int YCounter = MinY; YCounter < (int)MaxY; YCounter++)
            for (int XCounter = MinX; XCounter < (int)MaxX; XCounter++) {
                QScriptValue res = fun.call(QScriptValue(), QScriptValueList() << Data->at(i)->Array[YCounter][XCounter]);
                Data->at(i)->Array[YCounter][XCounter] = res.toNumber();
            }
    }
}
void TMathProcessing::_DoCustom2d()
{
    QString flt = "function filter(x) { with(Math) { return ( %1 ); } }";
    flt = flt.arg(ActiveTask.arg.toString());

    QScriptEngine myEngine;
    myEngine.evaluate(flt);
    QScriptValue fun = myEngine.globalObject().property("filter");

    for (int i = 0; i < Curves->size(); i++) {
        if (!(*Curves)[i])
            continue;

        long Size = (*Curves)[i]->size();
        double *YArray = (*Curves)[i]->YSeq.data();

        //TODO opencl, qtconcurrent, workerthreads
        for (int i = 0; i < Size; i++) {
            QScriptValue res = fun.call(QScriptValue(), QScriptValueList() << YArray[i]);
            YArray[i] = res.toNumber();
        }
    }
}
void TMathProcessing::SetupMinsMaxs(unsigned int &MinX, unsigned int &MaxX, unsigned int &MinY, unsigned int &MaxY, const TMin3dData &Min3dData)
{
    bool LimsCorrect = Min3dData.LimsForMathCorrect();
    MinX = LimsCorrect ? (unsigned int)Min3dData.LimsForMathX[0] : 0;
    MinY = LimsCorrect ? (unsigned int)Min3dData.LimsForMathY[0] : 0;
    MaxX = LimsCorrect ? (unsigned int)Min3dData.LimsForMathX[1] : Min3dData.NXElems;
    MaxY = LimsCorrect ? (unsigned int)Min3dData.LimsForMathY[1] : Min3dData.NYElems;
}
void TMathProcessing::DoFit(int N)
{
    TTask Task;
    Task.arg = N;
    Task.Action = TMathProcessing::Fit2d;
    SmartAddTask(Task);
}
void TMathProcessing::DoScale(int N)
{
    TTask Task;
    Task.arg = N;
    Task.Action = TMathProcessing::Scale2d;
    SmartAddTask(Task);
}
void TMathProcessing::DoFitX(int N)
{
    TTask Task;
    Task.arg = N;
    Task.Action = TMathProcessing::Fit3dX;
    SmartAddTask(Task);
}
void TMathProcessing::DoFitY(int N)
{
    TTask Task;
    Task.arg = N;
    Task.Action = TMathProcessing::Fit3dY;
    SmartAddTask(Task);
}
void TMathProcessing::DoSubtr(int N)
{
    TTask Task;
    Task.arg = N;
    Task.Action = TMathProcessing::Subtr;
    SmartAddTask(Task);
}
void TMathProcessing::DoLineCorrectByMedianaX()
{
    TTask Task;
    Task.Action = TMathProcessing::LineMedianX;
    SmartAddTask(Task);
}
void TMathProcessing::DoLineCorrectByMedianaY()
{
    TTask Task;
    Task.Action = TMathProcessing::LineMedianY;
    SmartAddTask(Task);
}
void TMathProcessing::DoMeanValue(int Sz)
{
    TTask Task;
    Task.Action = TMathProcessing::MeanValue;
    Task.arg = Sz;
    SmartAddTask(Task);
}
void TMathProcessing::DoMedianValue(int Sz)
{
    TTask Task;
    Task.Action = TMathProcessing::MedianValue;
    Task.arg = Sz;
    SmartAddTask(Task);
}
void TMathProcessing::DoGaussian(int Sz)
{
    TTask Task;
    Task.Action = TMathProcessing::Gaussian;
    Task.arg = Sz;
    SmartAddTask(Task);
}
void TMathProcessing::DoCustom2d(QString str)
{
    TTask Task;
    Task.Action = TMathProcessing::CustomFilter2d;
    Task.arg = str;
    SmartAddTask(Task);
}
void TMathProcessing::DoCustom3d(QString str)
{
    TTask Task;
    Task.Action = TMathProcessing::CustomFilter3d;
    Task.arg = str;
    SmartAddTask(Task);
}

}
}
