/* -*- mode: C++ ; c-file-style: "stroustrup" -*- *****************************
 * Qwt Widget Library
 * Copyright (C) 1997   Josef Wilgen
 * Copyright (C) 2002   Uwe Rathmann
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the Qwt License, Version 1.0
 *****************************************************************************/

#ifndef QWT_ABSTRACT_SCALE_H
#define QWT_ABSTRACT_SCALE_H



class QwtScaleEngine;
class QwtScaleDraw;
class QwtScaleDiv;
class QwtScaleMap;
class QwtInterval;

/*!
  \brief An abstract base class for classes containing a scale

  QwtAbstractScale is used to provide classes with a QwtScaleDraw,
  and a QwtScaleDiv. The QwtScaleDiv might be set explicitely
  or calculated by a QwtScaleEngine.
*/

class QwtAbstractScale
{
public:
    QwtAbstractScale();
    ~QwtAbstractScale();

    void setScale( double vmin, double vmax, double step = 0.0 );
    void setScale( const QwtInterval &, double step = 0.0 );
    void setScale( const QwtScaleDiv & );

    void setScaleEngine( QwtScaleEngine * );
    void setAbstractScaleDraw( QwtScaleDraw * );

    void updateScaleDraw();
    void rescale( double vmin, double vmax, double step = 0.0 );
    virtual void scaleChange();

    QwtScaleEngine *scaleEngine;
    QwtScaleDraw *scaleDraw;
    double stepSize;
};

#endif
