/* -*- mode: C++ ; c-file-style: "stroustrup" -*- *****************************
 * Qwt Widget Library
 * Copyright (C) 1997   Josef Wilgen
 * Copyright (C) 2002   Uwe Rathmann
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the Qwt License, Version 1.0
 *****************************************************************************/

#ifndef QWT_DOUBLE_RANGE_H
#define QWT_DOUBLE_RANGE_H



/*!
  \brief A class which controls a value within an interval

  This class is useful as a base class or a member for sliders.
  It represents an interval of type double within which a value can
  be moved. The value can be either an arbitrary point inside
  the interval (see QwtDoubleRange::setValue), or it can be fitted
  into a step raster (see QwtDoubleRange::fitValue and
  QwtDoubleRange::incValue).
*/

class QwtDoubleRange
{
public:
    QwtDoubleRange();

    void setRange( double vmin, double vmax, 
        double vstep = 0.0 );

    virtual void setValue( double );
    void setStep( double );

    double minValue;
    double maxValue;
    double step;
    double value;

protected:
    virtual void valueChange() { }
    virtual void rangeChange() { }

    void setNewValue( double value, bool align = false );

};

#endif
