#ifndef EG3000_H
#define EG3000_H

#include "../eg3000/defs.h" //graphed hackfix
#include "grapheditor/out/TResults.h"
#include <QThread>

class QSettings;
double settings_double(QSettings &sett, const QString &path, double def);
double input_calib(BYTE ch, double val, double zmax = 10.);
double input_gain(BYTE ch);
double input_shift(BYTE ch);
void input_gain_set(BYTE ch, double gain);
void input_gain_clear(BYTE ch);
void input_shift_set(BYTE ch, double shift);
void input_shift_clear(BYTE ch);

QString input_units(BYTE ch);
void input_unit_set(BYTE ch, QString unit);
void input_unit_clear(BYTE ch);


double setpoint_limit_top(double value, bool cl_on = false);

void go_relative_x(double x_diff);
void go_x(double x);
void go_relative_x2(double x_diff);
void go_x2(double x);
void go_relative_y(double y_diff);
void go_y(double y);
void go_relative_y2(double y_diff);
void go_y2(double y);

double get_objective_z();
double get_max_objective_z();
void go_relative_objective_z(double z_diff);
void go_objective_z(double z);


enum MotorStatus {
    MSOff = 0,
    MSUp,
    MSDown,
    MSUpLimit,
    MSDownLimit
};

//pid status?
//nlc status?

class USB_WRITE_THREAD: public QThread
{
    Q_OBJECT
public slots:
    void ScanCorrection(int st);
    void scanPreciseMode(bool enabled);
    void start_stop(bool);
    void thread_close();

    void setSensorOverflowMode(BYTE scanner, BYTE mode);
    void setLVM(BYTE head, BYTE stage);

    void setKelvinOn(bool);
    void setKelvinP(double);

    void setDFLXtalk(double);
    void setMAGXtalk(double);

    void setQcontrolGain(double);

    void setPiezoTip(bool);
    void setPiezoTip2(bool);
    void setCapMode(bool);
    void setCapMode2(bool);
    void setEFMplus(bool);
    void setLFLCurrent(bool);
    void setExtCurrent(bool);
    void setDrive2(double);
    void setFreq2(double);

    void Set_osc_Channel(int, int);
    void sl_csb_speed(int idx);
    void Set_headDAC(double);
    void Set_headDAC2(double);
    void ccd_readout_time(WORD time_ms);
    void scan_first_delay(double time);
    void scan_line_delay(double time);
    void scan_point_delay(double time);
    void spm_pid_precision(double delta);
    void spm_jm_lift(DWORD dz);
    void spm_jm_precision(WORD msb, WORD lsb);
    void spm_jm_div(int div);
    void scan_fast_return(bool);
    void scan_sync_mode(int);

    void Osc_Start(void);
    void Osc_Stop(void);

    void Usb_Load_Data(void);
    void Data_to_Cont();
    void Save_to_EEPROM(void);
    void Go_to_XYZ(double, double, double);
    void Go_to_XYZ2(double, double, double);
    void Go_to_XYZ_CCD(int, double, double);
    void Go_to_Z(double);
    void Go_to_Zobject(double);
    void Go_to_Z3(double);
    void Go_to_X(double);
    void Go_to_X2(double);
    void Go_to_Y(double);
    void Go_to_Y2(double);

    void FBZ_send(unsigned char);
    void set_scanner(int);
    void Set_Scan_Parameters(unsigned short int, unsigned short int, double, double, double, double, double, unsigned char, unsigned char, unsigned char, unsigned char, unsigned char, bool, bool, bool, bool, bool, double);
    void SCAN_Start(void);
    void SCAN_Stop(void);
    void Set_Speed(double);
    void scan_lm_height(double);

    void SET_SPM_P(double);
    void SET_SPM_I(double);
    void SET_SPM_Sp(double);
    void SET_SPM_FB_Channel(int);
    void Set_FB_sign(bool);
    void Set_FB_Scanner(int sc);

    void freq_scan_start(double, double, int);
    void set_freq(double);
    void set_cutoff_freq(double);
    void set_ampl_gains(int, int, int);
    void set_ampl_gains2(int, int, int);
    void set_phase(double);
    void set_piezodrive_volt(double);

    void set_Ex_dac1(double);
    void set_Ex_dac2(double);
    void set_Ex_dac3(double);
    void set_Ex_dac4(double);

    void set_raw_motors_inv_end(unsigned char, unsigned short);
    void set_raw_motors_control(unsigned char, unsigned short);
    void set_step_motors_leveling(int,unsigned char, double, double);
    void set_landing(BYTE, BYTE, double, double, double);
    void set_landing_m4(int);

    void set_spectroscopy(int st, double z1,double z2, double speed, int channel, int points);
    void SetccdScanMode(int, int, int, int, int, double, double, double, double, bool);
    void CCD_SCAN_Start(void);
    void CCD_SCAN_Stop(void);
    void CCD_SCAN_Ready(void);
    void sl_Head_Data(int, int, int);
    void HVXYZzero();

    void slLitoStartStop(bool sig);
    void set_Litho_Type(int);
    void set_Litho_Scanner(int);
    void set_Litho_continuous(bool);
    void set_Litho_VOut(int);
    void sl_LithoNextpacket();

    void sl_get_curr_xy();
    void sl_get_curr_cz();
    void sl_get_curr_data(int ch);

    void SendExtraCalib();

public:
    USB_WRITE_THREAD();
    void SetTestMode(bool);
    void run();

    int csb_speed_idx;
    bool csb_speed_extended;
    BYTE fast_scan_correction;
    bool scan_precise_mode;

    bool testMode;

    //head
    float *Scanner_data_pid_cal_range;
    DWORD *Scanner_data_scale;
    long *Scanner_data_Shift;

    bool *Nlc_status_cl_xy;
    bool *CL_status_xy;
    bool *Nlc_status_cl_z;
    bool *CL_status_z;
    bool *Nlc_status_ol_xy;
    bool *Nlc_status_ol_z;

    float *free_move_speed;

    //z3
    float *Scanner3_data_pid_cal_range;
    DWORD *Scanner3_data_scale;
    long *Scanner3_data_Shift;

    bool *CL3_status_z;
    float *free_move_speed3;

    //stage
    float *Scanner2_data_pid_cal_range;
    DWORD *Scanner2_data_scale;
    long *Scanner2_data_Shift;

    bool *Nlc_status_cl_xy2;
    bool *CL_status_xy2;
    bool *Nlc_status_cl_z2;
    bool *CL_status_z2;
    bool *Nlc_status_ol_xy2;
    bool *Nlc_status_ol_z2;

    float *free_move_speed2;

    void Set_Scanner_Data_in(float* scanner_data_pid_cal_range, DWORD *scanner_data_scale, long* scanner_data_Time,
                             bool* nlc_c_xy, bool* ncl_xy, bool* nlc_c_z, bool* ncl_z,
                             bool* nlc_o_xy, bool* nlc_o_z, float *_free_move_speed,
                             float* scanner2_data_pid_cal_range, DWORD *scanner2_data_scale, long* scanner2_data_Time,
                             bool* nlc2_c_xy, bool* ncl2_xy, bool* nlc2_c_z, bool* ncl2_z,
                             bool* nlc2_o_xy, bool* nlc2_o_z, float *_free_move_speed2);
    void SetExtraCalibOut(double*, double*);

    double Xsp, Ysp, Zsp;
    double Xsp2, Ysp2, Zsp2;
    double Zsp3;
    BYTE FBZ_config;

    WORD NX;
    WORD NY;
    double Vsc;
    float scan_lm_h;
    double Step_X;
    double Step_Y;
    double St_point_X;
    double St_point_Y;
    BYTE scan_dir_st_point;
    BYTE scan_a;
    BYTE scan_b;
    BYTE scan_c;
    BYTE scan_d;

    int scanner_scan;
    int ccdNX,ccdNY;
    double ccdStep_X, ccdStep_Y, ccdSt_point_X, ccdSt_point_Y;
    int ccdscan_dir_st_point;
    int ccdScanner;
    int ccdMode;
    bool ccdFastScan;

    double Spm_P, Spm_I, Spm_SetPoint;
    BYTE Spm_FB_Channel;
    int FB_sign;
    char FB_Scanner;

    double current_frequency, cutoff, fr_start, fr_end, phase, volt;
    int gain1, gain2, gain3;
    int gain12, gain22, gain32;
    int fr_channel;

    double Ex_dac1, Ex_dac2, Ex_dac3, Ex_dac4;

    WORD motor_pwm;
    long motor_step[8+1];
    long motor_speed[8+1];
    WORD motor_cont_byte[8+1];
    WORD motor_inv_byte[8];
    DWORD motor_bl_steps[8];
    WORD motor_bl_speed;
    DWORD motor_land_safe_calib;
    BYTE rawmotoraddr;
    WORD rawmotordata;

    BYTE thermo_data[6];

    BYTE Landing_mode;
    BYTE Landing_direction;
    double Landing_speed;
    double Landing_speed_close;
    double Landing_close_perc;

    volatile bool thread_started;
    volatile bool thread_closed;

    double Z1spectr;
    double Z2spectr;
    double SpeedSpectr;
    int SpectrChannel;
    int SpectrPoints;
    int spect_onn_off;

    int LaserOnOf, AC_DC, Preamp;

    double* adcCalib;
    double* dacCalib;

    bool LmEn, LmEnA, LmEnB, LmEnC, LmEnD;
    double LmHeight;

    const graphed::out::TResultsList *lithoResList;
    void setLithResOut(const graphed::out::TResultsList* res);
    EG_LithoParams2 LithoPar;
    graphed::out::TResultsList::const_iterator ResIt;
    graphed::STLPointFVector::const_iterator PointIt;
    int LithPacketInd;
    int LithoMode;
    int LithoScanner;
    bool LithoContinuous;
    int LithoDACIndex;
    bool newShape;

signals:
    void xUpdate(double);
    void x2Update(double);
    void yUpdate(double);
    void y2Update(double);

public:
    Q_SIGNAL void zUpdate(double);
    Q_SIGNAL void z2Update(double);
    Q_SIGNAL void z3Update(double);

    //-----for test mode-----------
signals:
    void sig_osc_test(bool);
    void sig_lock_test();
    void sig_scan_test(bool);
    void sig_Spectr_test();
    void sig_scan_test_ccd(bool);
    void sig_scan_test_ccd_ready();
    void sig_LithoTotalReady();
    void sig_Lit_test();

public:
    int Active_Ch[4];

    bool updLVM;
    BYTE lvm_bits;

    BYTE SensorOFon;
    BYTE updateSensorOF;

    bool kelvinOn;
    double kelvinP;
    bool updateKelvin;

    float dfl_xtalk;
    bool updateDFLXtalk;
    float mag_xtalk;
    bool updateMAGXtalk;

    float qcontrol_gain;
    bool updateQcontrolGain;

    bool piezoTip;
    bool updatePiezoTip;
    bool piezoTip2;
    bool updatePiezoTip2;

    bool lflCurrent;
    bool updatelflCurrent;
    bool extCurrent;
    bool updateextCurrent;

    bool capmode;
    bool updateCapMode;
    bool capmode2;
    bool updateCapMode2;
    bool efmplus;
    bool updateEFMplus;

    double drive2;
    bool updatedrive2;
    double freq2;
    bool updatefreq2;

    WORD head_dac;
    bool updateHeadDAC;
    WORD head_dac2;
    bool updateHeadDAC2;

    WORD m_ccd_readout_time;
    bool updateCCDTime;

    WORD m_scan_first_delay;
    bool updateFirstDelay;
    WORD m_scan_line_delay;
    bool updateLineDelay;
    WORD m_scan_point_delay;
    bool updatePointDelay;
    bool m_scan_fast_return;
    bool updateFastReturn;
    int m_scan_sync_mode;
    bool updateSyncMode;

    WORD m_spm_pid_precision;
    bool updateSPMPrecision;

    WORD m_jm_precision, m_jm_precision_lsb;
    bool updateJM_Precision;
    DWORD m_jm_lift;
    bool updateJM_Lift;
    int m_jm_div;
    bool updateJM_div;

    int landing_m4_mode;
    bool updateLandingM4;

    volatile bool Action_1, Action_2, Action_3, Action_5[3], Action_6,
    Action_61, Action_65, Action_7,
    Action_71, Action_72, Action_73, Action_76, Action_77,
    Action_78, Action_79, Action_8, Action_81, Action_82, Action_83,
    Action_84, Action_85, Action_852, Action_86, Action_87, Action_88, Action_89,
    Action_9, Action_10, Action_11,
    updMotor[8+1], updLanding, Action_17, Action_18, Action_19, Action_20,
    Action_21, Action_22, Action_23;
    bool sendLMheight;

    bool sendPIDnlc;
    double *nlcPID;
    BYTE *nlcPIDOn;

    BYTE csb_motor_addr;
    bool updateCsbMotorAddr;
    BYTE csb_motor_count;
    bool updateCsbMotorCount;
    BYTE remap_motor_to[8];
    bool updateRemapMotor[8];

    BYTE motorRst;
    bool RawMotor_control[8+1], RawMotor_inv_end[8], RawMotor_bl_steps[8], RawMotor_pwm, RawMotor_bl_speed, RawMotorLandSafeCalib, RawMotor;
    bool Raw_Thermo;
    bool doGetXY;
    bool doGetCZ;
    bool doGetData;
    int getDataCh;

};

class IProcessPacket
{
public:
    IProcessPacket() { master = 0; }
    ~IProcessPacket();

    //should be thread safe
    virtual bool processPacket(BYTE addr, BYTE sub_addr, void *data, DWORD size) = 0;

    void *master;
};

class USB_READ_THREAD: public QThread
{
    Q_OBJECT

public slots:
    void start_stop(bool);
    void thread_close();

    void SetScanPoints(unsigned short int, unsigned short int, double, double, double, double, double, unsigned char, unsigned char, unsigned char, unsigned char, unsigned char, bool, bool, bool, bool, bool, double);

    //---for test mode--------------------
    void sl_set_osc_test(bool);
    void sl_set_lockin_test();
    void sl_set_Spectr_test();
    void sl_set_scan_test(bool);

    //------------------------------------
    void sl_set_Litho_test();
    void set_scan_test_ccd(bool);
    void sl_set_scan_test_ccd_ready();
    void SetccdScanMode(int, int, int, int, int, double, double, double, double, bool);

signals:
    void capBits(int n, long bits);
    void status(int);
    void sigSPMStatus(double zpos, double dfl, double lfl, double mag, double int_);
    void sigScanError();
    void sigSetpointError(int);
    void thermoData(QByteArray);

    void getDataRecvX(double);
    void getDataRecvY(double);
    void getDataRecvX2(double);
    void getDataRecvY2(double);

    void getDataRecv(int ch, long val);

public:
    void SetTestMode(bool);
    bool testMode;
    bool makeTestData(BYTE *subaddr);

    USB_READ_THREAD();
    ~USB_READ_THREAD();
    void run();

    //call only while not running
    void addReader(IProcessPacket *iface);
    void removeReader(IProcessPacket *iface);
    QVector<IProcessPacket *> readers;

    EG_ScanData *ScanDataPart[8];
    EG_ScanData *ScanDataPart_lsb[8];

    //head
    float *Scanner_data_pid_cal_range;
    DWORD *Scanner_data_scale;
    long *Scanner_data_Shift;

    bool *Nlc_status_cl_xy;
    bool *CL_status_xy;
    bool *Nlc_status_cl_z;
    bool *CL_status_z;
    bool *Nlc_status_ol_xy;
    bool *Nlc_status_ol_z;

    float *free_move_speed;

    //stage
    float *Scanner2_data_pid_cal_range;
    DWORD *Scanner2_data_scale;
    long *Scanner2_data_Shift;

    bool *Nlc_status_cl_xy2;
    bool *CL_status_xy2;
    bool *Nlc_status_cl_z2;
    bool *CL_status_z2;
    bool *Nlc_status_ol_xy2;
    bool *Nlc_status_ol_z2;

    float *free_move_speed2;

    void Set_Scanner_Data_out(float* scanner_data_pid_cal_range, DWORD *scanner_data_scale, long* scanner_data_Time,
                              bool* nlc_c_xy, bool* ncl_xy, bool* nlc_c_z, bool* ncl_z,
                              bool* nlc_o_xy, bool* nlc_o_z, float *_free_move_speed,
                              float* scanner2_data_pid_cal_range, DWORD *scanner2_data_scale, long* scanner2_data_Time,
                              bool* nlc2_c_xy, bool* ncl2_xy, bool* nlc2_c_z, bool* ncl2_z,
                              bool* nlc2_o_xy, bool* nlc2_o_z, float *_free_move_speed2);

    WORD Nx;
    WORD Ny;
    BYTE direction;
    BYTE up_down_left_right;
    int scan_wait;

    int ccdNX,ccdNY;

    bool scanError;
    bool headConnected;
    bool stageConnected;
    volatile bool scannersConnectedUpdated;

    bool thread_started;
    bool thread_closed;

signals:
    void Update_EEP_Data(void);

    void ScanLineReady(QByteArray data);
    void FreqScanData(QByteArray data);
    void ForceCurveData(QByteArray data);
    void OscCurveData(QByteArray data);

    void CCD_Start();
    void LithoReady();

    void Motors_status(int,int);
    void Landing_status(int);

public:
    BYTE scan_dir_st_point;

    int Scan_ChA;
    int Scan_ChB;
    int Scan_ChC;
    int Scan_ChD;

    double St_point_Xn;
    double St_point_Yn;
    double step_x;
    double step_y;
    int sign;

private:
    //-for test mode----
    int deflec_ind;
    bool is_Osc_test;
    bool is_Lock_in_test;
    bool is_Spectr_test;
    bool Scan_test;
    int sc_ch;
    bool ccdswitch;
    bool Lithotest;
    WORD line_test;
    BYTE switch_osc_ch;

    double test_plane_func(double, double);
    bool ccdTest;
    bool ccdReady;
    int ccdCounter;
};

class USB_CONTROL: public QThread
{
    Q_OBJECT

public:
    USB_CONTROL();
    ~USB_CONTROL();
    void SetTestMode(bool);
    void run();

signals:
    void restart_boot_timer();
    void device_status(int code);
    void usbStatus(int);
    void updateUsbStatus(bool);

public slots:
    void thread_close(USB_WRITE_THREAD *wThread, USB_READ_THREAD *rThread);
    void drop_connect();

    void program(QString);
    void s_boot_timeout();

public:
    bool m_closeThread;
    bool m_waitMainInit;
    bool testMode;
    bool st;

    QTimer *boot_timeout_t;
    bool boot_timeout;

    bool need_program;
    QString path;
};

struct USB_THREAD_GROUP
{
    USB_CONTROL cThread;
    USB_WRITE_THREAD wThread;
    USB_READ_THREAD rThread;
};

#endif
