#include "capture.hpp"
#include <QProcess>
#include <QDir>
#include <QDebug>

void RecurseDirectory(const QString& sDir)
{
    QDir dir(sDir);
    QFileInfoList list = dir.entryInfoList();

    for (int iList=0; iList<list.count(); iList++) {
        QFileInfo info = list[iList];
        QString sFilePath = info.filePath();

        if (info.isDir()) {
            if (info.fileName()!=".." && info.fileName()!=".") {
                RecurseDirectory(sFilePath);
            }
        } else {
            QFile::remove(sFilePath);
        }
    }
}

void Capture::SetFps(int _fps)
{
    fps = _fps;
}
void Capture::SetSize(int w, int h)
{
    width = w;
    height = h;
}
void Capture::SetFileName(QString name)
{
    FileName = name;
}

void Capture::run()
{
    isStart = true;
    QList<QImage> toSave;
    unsigned long frame_num = 0;

    QDir().mkpath("./video_frames");
    RecurseDirectory("./video_frames");

    while (isStart || !toSave.isEmpty()) {
        if (!toSave.isEmpty()) {
            if (!toSave.takeFirst().save("./video_frames/" +
                                         QString::number(frame_num).rightJustified(10, '0') +
                                         ".jpg",
                                         "jpg", 75))
                qDebug() << "save error: imageformats/qjpeg4.dll may not exist";
            frame_num++;
        } msleep(5);
        if (added_lock.tryLock()) {
            if (!added.isEmpty()) {
                toSave.append(added);
                added.clear();
            }
            added_lock.unlock();
        }
    }

    QString command = "mencoder mf://*.jpg -mf w=" + QString::number(width) +
            ":h=" + QString::number(height) +
            ":fps=" + QString::number(fps) +
            ":type=jpg -ovc copy -o \"" + FileName + "\"";

    QProcess mencoder;
    mencoder.setWorkingDirectory("./video_frames");
    mencoder.start(command);
    if (mencoder.waitForStarted())
        mencoder.waitForFinished();

    RecurseDirectory("./video_frames");
    QDir().rmpath("./video_frames");
}

void Capture::AddImage(QImage img)
{
    img.detach();
    added_lock.lock();
    added.push_back(img);
    added_lock.unlock();
}

void Capture::Stop()
{
    isStart = false;
}
