#ifndef VIDEOPLAYER_H
#define VIDEOPLAYER_H

#include <QDialog>
#include <QWidget>
#include <QElapsedTimer>
#include <QLabel>
#include <QGridLayout>
#include <QtEvents>
#include <QFileDialog>
#include <QFutureWatcher>
class QCheckBox;
class QComboBox;
class QRubberBand;
#include "controls.h"
#include "edit.hpp"
#include "qwt_plot.h"
#include <2d/plot_2dtypes.h>
#include <3d/plot_3dtypes.h>

#include "i_video.hpp"
#include "capture.hpp"

class ZScanWindow : public QDialog
{
    Q_OBJECT
public:
    ZScanWindow(QWidget *parent) : QDialog(parent) {
        path = "index";

        LEndPos = new QLabel("10");
        EStartPos = new Edit(0, 100, 4);
        EStep = new Edit(-100, 100, 4);
        StepsCount = new Edit(1, 1000); StepsCount->setImmediateMode();

        BPath = new QPushButton(tr("Output directory"));
        BOk = new QPushButton(tr("Start"));
        BCancel = new QPushButton(tr("Cancel"));

        EStep->setValue(1);
        StepsCount->setValue(10);

        connect(EStartPos, SIGNAL(newValue(double)), SLOT(update_end()));
        connect(EStep, SIGNAL(newValue(double)), SLOT(update_end()));
        connect(StepsCount, SIGNAL(newValue()), SLOT(update_end()));

        connect(BPath, SIGNAL(clicked()), SLOT(path_select()));
        connect(BOk, SIGNAL(clicked()), SLOT(accept()));
        connect(BCancel, SIGNAL(clicked()), SLOT(reject()));

        QGridLayout *layout = new QGridLayout(this);
        layout->setSizeConstraint(QLayout::SetFixedSize);

        layout->addWidget(new QLabel(tr("Start pos: ")), 0, 0);
        layout->addWidget(new QLabel(tr("Step size: ")), 1, 0);
        layout->addWidget(new QLabel(tr("Step num: ")), 2, 0);
        layout->addWidget(new QLabel(tr("End pos: ")), 3, 0);

        layout->addWidget(EStartPos, 0, 1);
        layout->addWidget(EStep, 1, 1);
        layout->addWidget(StepsCount, 2, 1);
        layout->addWidget(LEndPos, 3, 1);

        layout->addWidget(BPath, 4, 0, 1, 2);
        layout->addWidget(BOk, 5, 0);
        layout->addWidget(BCancel, 5, 1);

        setWindowTitle(tr("Z Scan Options"));
    }

    QString path;
    Edit *EStartPos;
    Edit *EStep;
    Edit *StepsCount;
    QLabel *LEndPos;

    QPushButton *BPath;
    QPushButton *BOk;
    QPushButton *BCancel;

    void keyPressEvent(QKeyEvent *e) {
        if (e->key()!= Qt::Key_Escape && e->key()!= Qt::Key_Return && e->key()!= Qt::Key_Enter) QDialog::keyPressEvent(e);
    }

public slots:
    void path_select() {
        path = QFileDialog::getSaveFileName(this, tr("Output folder & index file"), path,
                                            QString("index files(*.txt)"),
                                            0, QFileDialog::DontUseNativeDialog);

        if (path.isEmpty()) path = "index";
        if (path.endsWith(".txt", Qt::CaseInsensitive)) path.chop(4);
    }

    void update_end() {
        LEndPos->setText(QString::number(EStartPos->value() + EStep->value()*StepsCount->value()));
    }
};

class StatusWidget1;
class ImageItem;
class QwtPlotMarker;
class VideoPlayer : public QWidget
{
    Q_OBJECT
public:
    VideoPlayer(QWidget *parent = 0);
    ~VideoPlayer();
    static int times_constructed;
    int inst_idx;

    TwoStateButton *PBToNewWindow;
    StatusWidget1 *deviceStatus;
    TwoStateButton *m_BStart;
    TwoStateButton *saveAviB;

    QPointF posx;
    bool active;
    QwtPlotMarker *marker;

    unsigned char fb_state;
    DoubleChanger *zPos;
    bool isInterrupted;

    QString configPath;

    QFutureWatcher<void> resetWatch;
    void resetCamerasImpl(); //private

public slots:
    void resetCameras();
    void resetCamerasCommit(); //private

    void cancelOperation();
    void on_Z_Scan();
    void on_Z_Reconstruct();
    void on_Camera_ImgReceived(QImage Img);
    void calcOscContrast(QByteArray data);
    void on_BAutoFocus_clicked();
    void on_BAutoFocusOsc_clicked();
    void on_BToData_clicked();
    void StartStop(bool value);
    void slSetExp(double value);
    void slSetUserExp(bool val);
    void slAWB();
    void sl_AE();
    void sl_saveToFile();
    void sl_saveAvi(bool value);
    void ae_awb_AutoControl(bool value);
    void on_cwh_ch();
    void sl_center_set(bool value);
    void sl_center_set2(bool value);
    void sl_axes(bool value);
    void sl_grid(bool value);
    void sl_line(bool value);
    void sl_add_config();
    void sl_del_config();
    void sl_select_config(int val);
    void sl_range();
    void sl_save();
    void sl_set_XYZ_range(double x, double y, double z);
    void sl_line_measure(bool value);

    void goZjoy(int speed);
    void goXYjoy(QPointF speedxy);

private:
    void setDeviceStatus(int);
    bool eventFilter(QObject *, QEvent *);
    void closeEvent(QCloseEvent *);

    int plot_mode;
    QPointF prev_xy_joy_output;
    QRubberBand *head_cant;
    QRubberBand *stage_cant;
    QRubberBand *stage_laser;
    QwtPlotMarker *markerA;
    QwtPlotMarker *markerB;
    QwtPlotMarker *markerC;
    plot::TLineSegment *lineAB;

    QList<QList<QString> *> settings;
    int prev_idx;

    qreal mkm_px_x;
    qreal mkm_px_y;
    Edit *range_x;
    Edit *range_y;
    Edit *ewcw;
    Edit *ewch;
    Edit *ewsz;
    QLineEdit *ewname;
    TwoStateButton *BCenterAndSet;
    TwoStateButton *BCenterAndSet2;
    TwoStateButton *BLine;
    TwoStateButton *BLineMeasure;
    QComboBox *CBConfig;
    QPushButton *BAddConfig;
    QPushButton *BDelConfig;
    QCheckBox *flipx;
    QCheckBox *flipy;
    QCheckBox *rotxy;
    QComboBox *xyMsel;

    bool DoAvi;
    double cw;
    double ch;

    QVector<IVideo *> CameraList;
    IVideo *activeCamera;

    Capture VideoSave;
    QElapsedTimer fps_time;
    int frames;

    double osc_contrast;
    bool new_osc_contrast;

    DoubleChanger *DCExposure;
    QCheckBox *CBUserExp;

    QPushButton *BZScan;
    QPushButton *BZ3dReconstruct;
    TwoStateButton *Auto;
    QPushButton *AEB;
    QPushButton *AviOptions;
    QPushButton *awbB;
    QPushButton *saveB;
    QPushButton *BAutoFocus;
    QPushButton *BAutoFocusOsc;
    QPushButton *BToData;
    QPushButton *BReset;
    TwoStateButton *BCantileverRange;
    TwoStateButton *BLaserRange;
    TwoStateButton *BGrid;
    TwoStateButton *BAxes;

    QwtPlot *QPlot;
    QwtPlotGrid *QGrid;
    ImageItem *ritem;
    ZScanWindow *m_ZScanWindow;
    QElapsedTimer refresh_limit_timer;
    qint64 refresh_limit_last_elapsed;
};

class ImageItem : public QwtPlotRasterItem
{
public:
    QImage image;
    QImage renderImage (const QwtScaleMap &/*xMap*/,
                        const QwtScaleMap &/*yMap*/, const QRectF &/*area*/, const QSize &/*imageSize*/) const
    { return image; }
};

#endif
