#ifndef AUTOCALIB_H_
#define AUTOCALIB_H_

#include <QWidget>
#include "plot.h"
#include "defs.h"
#include "controls.h"
class QCheckBox;
class QTableWidget;

class DelayedInitWidget : public QWidget
{
public:
    DelayedInitWidget(QWidget *parent = 0) : QWidget(parent), needGuiInit(true) { }

    bool needGuiInit;
    virtual void GuiCreate() = 0;
    void showEvent(QShowEvent *) {
        if (needGuiInit) {
            needGuiInit = false;
            GuiCreate();
        }
    }
};

class DataPlot;
class SPMTunning;
class Oscilloscope;

class AutoCalibPlot : public DelayedInitWidget
{
    Q_OBJECT

public:
    AutoCalibPlot(QWidget *parent = 0);
    void GuiCreate();

    plot::TPure2dPlot *scanplot;
    DataPlot *dataplot;
    SPMTunning *spm;
    Oscilloscope *osc;

    double calib_coeff;
    double calib_min;
    QVector<double> adcvalues;
    QVector<double> axisvalues;
    QwtPlotMarker *levelMarker;
    QVector<QwtPlotMarker *> markers;
    QwtPlotMarker *activemarker;
    int activepoint;

    QTableWidget *TWvalues;
    plot::T2dPlot *Source;
    plot::T2dPlot *CalibCurve;
    plot::TCurvesDataOwner curveData;
    plot::TCurvesDataOwner curveDataCalib;

    QComboBox *CBScanner;
    QComboBox *CBInput;
    QComboBox *CBValue;
    QComboBox *CBAxis;

    QCheckBox *CBSource;
    QCheckBox *CBLevel;
    QCheckBox *CBDerivative;
    Edit *LEPeriod;
    Edit *LELevel;
    QPushButton *BReturn;
    QPushButton *BSend;
    QPushButton *coeffOk;
    QPushButton *BSensorsAutoAdj;
    QPushButton *BPIDCalib;

    QLineEdit *lineC[6];
    double c0;

    bool eventFilter(QObject *, QEvent *);

public:
    bool sensorNoZ;
    bool isInterrupted;
protected:
    void SetOscChannel(int ch);
    double ReadOscFirst(int timeout = 1000);
    double ReadOscAverage(int timeout = 1000);
    double ReadOscLine(WORD *data_line, int timeout = 1000);
    double LineCalcAverage(WORD *data_line);
    double LineCalcRms(WORD *data_line);
    long LineCalcMaximums(WORD *data_line);

    void go_osc_xyz(double xpos, double ypos, double zpos);
    void read_osc(double *v, int ch);
    void read_osc_xyz(double *x, double *y, double *z);
    void read_osc_xyz_v(double *x, double *y, double *z);
    bool read_osc_xyz_no_outer_drift(double *x, double *y, double *z, int x_sign, int y_sign, int z_sign);
    bool read_osc_xyz_v_no_outer_drift(double *x, double *y, double *z, int x_sign, int y_sign, int z_sign);
    void read_osc_xyz(double xpos, double ypos, double zpos,
                      double *x, double *y, double *z);
    void read_osc_xyz_no_outer_drift(double xpos, double ypos, double zpos,
                                     double *x, double *y, double *z, int x_sign, int y_sign, int z_sign);

    bool use_direct_get_data;
    bool new_osc_data;
    WORD osc_data[OSC_LINE_SIZE];
    int want_osc_ch;
public:
    bool is_optical_scanner[3];

public slots:
    void cbSrcMode();
    void channelChanged();
    void UpdateLayout();
    void customFilter(bool checked);
    void BGetClicked();
    void BSendClicked();
    void ReCalcFromCalibCurve();
    void ReCalcFromSourceMan();
    void ReCalcFromSource();
    void ReCalcFromTable();
    void ReCalcCoefficent();
    void ValuesUpdate();

    void recvOscData(QByteArray data);
    void recv_get_data(int ch, long val);
    void capBits(int, long);
    void cancelOperation();

    void SensorReCalc(bool silent = false);
    void PICalcAuto(bool silent = false);

private:
    void restoreSensorReCalc(double x_range, double y_range, double z_range,
                             BYTE fb_state, bool old_fb, bool old_fbz, bool old_nl,
                             DWORD Scanner_data_scale[], long Scanner_data_Shift[], int old_ch[],
                             QWidgetList cwl);
    bool fixScaleAtBounds(double x, double y, double z, double *scale);
    void restorePICalc(double x_range, double y_range, double z_range,
                             BYTE fb_state, bool old_fb, bool old_fbz, bool old_nl,
                             int old_ch[],
                             QWidgetList cwl);

};

#endif
