#include "temp_control.hpp"
#include "mainwindow.h"
#include "EG3000.h"
#include <QVBoxLayout>
#include <QFormLayout>
#include <QSettings>

extern USB_WRITE_THREAD *_wThread; //from modules.cpp

temp_control::~temp_control()
{
    loop->shutdown = true;
    loop->wait();

    delete loop;
}

temp_control::temp_control(QWidget *parent) : QWidget(parent)
{
    T = 0;
    Tmax = 10;

    deviceStatus = new StatusWidget1(tr("Thermo"));
    setDeviceStatus(EDno);

    loop = new temp_control_loop;
    loop->start();

    QVBoxLayout *layout = new QVBoxLayout(this);
    layout->setContentsMargins(0, 6, 6, 6);

    plot::ResizeOwner(curveData, 1);
    curveData[0]->setXCalib(0, 1/5.);

    plot2d = new plot::T2dPlot;
    plot2d->SetAutoScalingEnabled(false);
    plot2d->Pure2dPlot->SetTitle(tr("Temperature curve"));
    plot2d->Pure2dPlot->SetAllXScales(0, Tmax);
    plot2d->Pure2dPlot->SetAllYScales(-50, 150);
    plot2d->Pure2dPlot->SetAutoLegendEnabled(false);
    plot2d->Pure2dPlot->SetXLegend(tr("time, s"));
    plot2d->Pure2dPlot->SetYLegend(tr("T, C"));
    layout->addWidget(plot2d, 1);

    plot2d->Pure2dPlot->SourceData = plot::getUnLock(curveData);

    QGroupBox *grb = new QGroupBox(tr("Temperature control"));
    layout->addWidget(grb);

    QHBoxLayout *l1 = new QHBoxLayout(grb);

    QFormLayout *l3 = new QFormLayout();
    QFormLayout *l4 = new QFormLayout();
    QFormLayout *l5 = new QFormLayout();

    l1->addLayout(l3);
    l1->addStretch(1);
    l1->addLayout(l4);
    l1->addStretch(1);
    l1->addLayout(l5);
    l1->addStretch(7);


    edP = new Edit(-10, 10, 3);
    edI = new Edit(-10, 10, 3);
    edD = new Edit(-10, 10, 3);
    edTemp = new Edit(-50, 150, 1);
    lTemp = new QLabel("?");
    Boff = new QPushButton(makeHugeIcon(":/CCD/off.png"), tr("Stop PID"));
    Bon = new QPushButton(makeHugeIcon(":/CCD/onn.png"), tr("Start PID"));

    edT = new Edit(1, 600);
    edT->setValue(10);
    connect(edT, SIGNAL(newValue()), this, SLOT(setT()));

    QSettings settings("Param/gui.conf", QSettings::IniFormat);
    edP->setValue(settings.value("temp/p", 0).toDouble());
    edI->setValue(settings.value("temp/i", 0).toDouble());
    edD->setValue(settings.value("temp/d", 0).toDouble());
    edTemp->setValue(20);

    connect(edP, SIGNAL(newValue(double)), this, SLOT(setPid()));
    connect(edI, SIGNAL(newValue(double)), this, SLOT(setPid()));
    connect(edD, SIGNAL(newValue(double)), this, SLOT(setPid()));
    connect(edTemp, SIGNAL(newValue(double)), loop, SLOT(setTemp(double)));
    connect(Boff, SIGNAL(clicked()), loop, SLOT(stopPid()));
    connect(Bon, SIGNAL(clicked()), loop, SLOT(startPid()));
    connect(this, SIGNAL(newPid(double,double,double)), loop, SLOT(setPid(double,double,double)));
    connect(loop, SIGNAL(currentTemp(double)), this, SLOT(addTemp(double)));
    connect(loop, SIGNAL(status(int)), this, SLOT(setDeviceStatus(int)), Qt::QueuedConnection);

    l3->addRow(Bon);
    l3->addRow(Boff);

    l4->addRow(tr("Current temperature"), lTemp);
    l4->addRow(tr("Set temperature"), edTemp);
    l4->addRow(tr("Time, s"), edT);

    l5->addRow(tr("P"), edP);
    l5->addRow(tr("I"), edI);
    l5->addRow(tr("D"), edD);
}


void temp_control::setDeviceStatus(int idx)
{
    if (!deviceStatus)
        return;

    setEnabled(idx >= EDok);
    switch (idx) {
    case EDno:
        deviceStatus->status->set(StatusWidget::Cross, Qt::red);
        deviceStatus->status->setToolTip(tr("No device"));
        break;
    case EDconn:
        deviceStatus->status->set(StatusWidget::Circle, QColor(255, 128, 0));
        deviceStatus->status->setToolTip(tr("Connecting"));
        break;
    case EDok:
        deviceStatus->status->set(StatusWidget::Circle, Qt::green);
        deviceStatus->status->setToolTip(tr("Connected"));
        break;
    case EDplay:
        deviceStatus->status->set(StatusWidget::Play, Qt::green);
        deviceStatus->status->setToolTip(tr("PID Enabled"));
        break;
    }
}

void temp_control::nativeThermoData(QByteArray data)
{
    union {
        char b[4];
        float f;
    } b_f;

    b_f.b[0] = data[0]; b_f.b[1] = data[1]; b_f.b[2] = data[2]; b_f.b[3] = data[3];

    switch (data[5]) {
    case 0:
        b_f.f = b_f.f * 2.5 / pow(2., 16);
        b_f.f = b_f.f / 2 / 2.5 / 1000;
        b_f.f = 1 / b_f.f - 1000;
    default:
        qDebug() << "temp" << b_f.f;
        addTemp(b_f.f);
        break;
    }
}

void temp_control::setT()
{
    if (Tmax > edT->value()) {
        curveData[0]->YSeq.clear();
        T = 0;
    }

    Tmax = edT->value();

    plot2d->Pure2dPlot->SetAllXScales(0, Tmax);
    plot2d->update();
}

void temp_control::setPid()
{
    emit newPid(edP->value(), edI->value(), edD->value());
}

void temp_control::addTemp(double temp)
{
    lTemp->setText(QString::number(temp));
    T++;
    if (T > Tmax*5) {
        curveData[0]->YSeq.clear();
        T = 0;
    }

    curveData[0]->AddY(temp);

    plot2d->Refresh();
}


#include <windows.h>
#ifndef _WIN32
#define __cdecl
#endif
typedef HANDLE (__cdecl *fnOpenSerialPort)(char* PortName);
typedef HANDLE (__cdecl *fnCloseSerialPort)();
typedef bool (__cdecl *fnStartPID)(FLOAT fSetPoint);
typedef bool (__cdecl *fnStartTReg)(FLOAT fSetPoint, FLOAT fVoltage);
typedef bool (__cdecl *fnStopPID)();
typedef bool (__cdecl *fnCheckConnection)(int iTimeOut);
typedef double (__cdecl *fnGetTValue)();
typedef bool (__cdecl *fnSetPIDPRef)(FLOAT kp, FLOAT ki, FLOAT kd);

#define DEF_FN(name) static fn##name pfn##name = 0

static HMODULE hdll = 0;
DEF_FN(OpenSerialPort);
DEF_FN(CloseSerialPort);
DEF_FN(StartPID);
DEF_FN(StartTReg);
DEF_FN(StopPID);
DEF_FN(CheckConnection);
DEF_FN(GetTValue);
DEF_FN(SetPIDPRef);

#ifdef _WIN32
#define LD_FN(name) pfn##name = (fn##name)GetProcAddress(hdll, "_" #name);
#define CALL_FN(name, res) if (pfn##name) res = pfn##name
#else
void _nullf(...) { }
#define LD_FN(name) pfn##name = 0;
#define CALL_FN(name, res) _nullf
#endif

temp_control_loop::temp_control_loop() :
    set_pid(false), set_temp(false),
    stop_pid(false), shutdown(false)
{
    temp = 20;

    QSettings settings("Param/gui.conf", QSettings::IniFormat);
    p = settings.value("temp/p", 0).toDouble();
    i = settings.value("temp/i", 0).toDouble();
    d = settings.value("temp/d", 0).toDouble();

    load();
}

void temp_control_loop::load()
{
    if (hdll)
        return;
#ifdef _WIN32
    hdll = LoadLibraryA("DX5100Lib.dll");
    if (!hdll)
        return;

    LD_FN(OpenSerialPort);
    LD_FN(CloseSerialPort);
    LD_FN(StartPID);
    LD_FN(StartTReg);
    LD_FN(StopPID);
    LD_FN(CheckConnection);
    LD_FN(GetTValue);
    LD_FN(SetPIDPRef);
#endif
}

void temp_control_loop::setTemp(double _temp)
{
    temp = _temp;
    set_temp = true;
}

void temp_control_loop::setPid(double _p, double _i, double _d)
{
    p = _p;
    i = _i;
    d = _d;

    QSettings settings("Param/gui.conf", QSettings::IniFormat);
    settings.setValue("temp/p", p);
    settings.setValue("temp/i", i);
    settings.setValue("temp/d", d);

    set_pid = true;
}

void temp_control_loop::startPid()
{
    set_temp = true;
}

void temp_control_loop::stopPid()
{
    stop_pid = true;
}

struct ThermoControl {
    enum _ {
        SetTemp = 0,
        SetP,
        SetI,
        SetD,
        StartPID,
        StopPID,
        StartMonitor,
        StopMonitor
    };
};

void temp_control_loop::run()
{
    QSettings settings("Param/controller.conf", QSettings::IniFormat);
    if (settings.value("dx5100").toString().startsWith("EG3000")) {
        stop_pid = true;
        set_pid = true;

        bool start_monitor = true;

        while (!shutdown) {
            union {
                BYTE b[4];
                float f;
            } b_f;

            if (!_wThread->isRunning()) {
                stop_pid = true;
                set_pid = true;
                start_monitor = true;
                emit status(EDno);
                for (int i = 0; i < 20 && !shutdown; i++)
                    msleep(50);
                continue;
            }
            if (stop_pid) {
                stop_pid = false;

                _wThread->thermo_data[0] = ThermoControl::StopPID;

                _wThread->Raw_Thermo = true;
                while (_wThread->Raw_Thermo && _wThread->isRunning()) msleep(10);

                emit status(EDok);
            }
            if (start_monitor) {
                start_monitor = false;

                _wThread->thermo_data[0] = ThermoControl::StartMonitor;

                _wThread->Raw_Thermo = true;
                while (_wThread->Raw_Thermo && _wThread->isRunning()) msleep(10);

                //emit status(EDconn);
            }
            if (set_pid) {
                set_pid = false;

                _wThread->thermo_data[0] = ThermoControl::SetP;
                b_f.f = p;
                _wThread->thermo_data[1] = b_f.b[0];
                _wThread->thermo_data[2] = b_f.b[1];
                _wThread->thermo_data[3] = b_f.b[2];
                _wThread->thermo_data[4] = b_f.b[3];

                _wThread->Raw_Thermo = true;
                while (_wThread->Raw_Thermo && _wThread->isRunning()) msleep(10);

                _wThread->thermo_data[0] = ThermoControl::SetI;
                b_f.f = i;
                _wThread->thermo_data[1] = b_f.b[0];
                _wThread->thermo_data[2] = b_f.b[1];
                _wThread->thermo_data[3] = b_f.b[2];
                _wThread->thermo_data[4] = b_f.b[3];

                _wThread->Raw_Thermo = true;
                while (_wThread->Raw_Thermo && _wThread->isRunning()) msleep(10);

                _wThread->thermo_data[0] = ThermoControl::SetD;
                b_f.f = d;
                _wThread->thermo_data[1] = b_f.b[0];
                _wThread->thermo_data[2] = b_f.b[1];
                _wThread->thermo_data[3] = b_f.b[2];
                _wThread->thermo_data[4] = b_f.b[3];

                _wThread->Raw_Thermo = true;
                while (_wThread->Raw_Thermo && _wThread->isRunning()) msleep(10);
            }
            if (set_temp) {
                set_temp = false;

                _wThread->thermo_data[0] = ThermoControl::SetTemp;
                b_f.f = temp;
                _wThread->thermo_data[1] = b_f.b[0];
                _wThread->thermo_data[2] = b_f.b[1];
                _wThread->thermo_data[3] = b_f.b[2];
                _wThread->thermo_data[4] = b_f.b[3];

                _wThread->Raw_Thermo = true;
                while (_wThread->Raw_Thermo && _wThread->isRunning()) msleep(10);

                _wThread->thermo_data[0] = ThermoControl::StartPID;

                _wThread->Raw_Thermo = true;
                while (_wThread->Raw_Thermo && _wThread->isRunning()) msleep(10);

                emit status(EDplay);
            }

            msleep(10);
        }

        /* //in controller reset/shutdown
        _wThread->thermo_data[0] = ThermoControl::StopMonitor;
        _wThread->Raw_Thermo = true;
        while (_wThread->Raw_Thermo && _wThread->isRunning()) msleep(10);
        */

        emit status(EDno);
        return;
    }

    bool res;
    HANDLE handle = 0;

    while (!shutdown) {
        if (!handle) {
            if (settings.value("dx5100").toString().startsWith("COM"))
                CALL_FN(OpenSerialPort, handle)(settings.value("dx5100").toString().toAscii().data());
            if (handle == (HANDLE)INVALID_HANDLE_VALUE) {
                handle = 0;
            } else if (handle != 0) {
                emit status(EDconn);
                CALL_FN(StopPID, res)();
                if (!res) {
                    CALL_FN(CloseSerialPort, handle)();
                    qDebug() << "dx51stoppid0 fail";
                    handle = 0;
                    emit status(EDno);
                    continue;
                }

                CALL_FN(SetPIDPRef, res)(p, i, d);
                if (!res) {
                    CALL_FN(CloseSerialPort, handle)();
                    qDebug() << "dx51setpid0 fail";
                    handle = 0;
                    emit status(EDno);
                    continue;
                }
            }
            emit status(handle != 0 ? EDok : EDno);
        }
        if (!handle) {
            emit status(EDno);
            for (int i = 0; i < 40 && !shutdown; i++)
                msleep(50);
            continue;
        }
        if (set_temp) {
            set_temp = false;
            CALL_FN(StartPID, res)(temp);
            if (!res) {
                CALL_FN(CloseSerialPort, handle)();
                qDebug() << "dx51startpid fail";
                handle = 0;
                emit status(EDno);
                continue;
            }
            emit status(EDplay);
        }
        if (set_pid) {
            set_pid = false;
            CALL_FN(SetPIDPRef, res)(p, i, d);
            if (!res) {
                CALL_FN(CloseSerialPort, handle)();
                qDebug() << "dx51setpid fail";
                handle = 0;
                emit status(EDno);
                continue;
            }
        }
        if (stop_pid) {
            stop_pid = false;
            CALL_FN(StopPID, res)();
            if (!res) {
                CALL_FN(CloseSerialPort, handle)();
                qDebug() << "dx51stoppid fail";
                handle = 0;
                emit status(EDno);
                continue;
            }
            emit status(EDok);
        }

        CALL_FN(CheckConnection, res)(500);
        if (!res) {
            CALL_FN(CloseSerialPort, handle)();
            qDebug() << "dx51check fail";
            handle = 0;
            emit status(EDno);
            continue;
        }

        if (pfnGetTValue)
            emit currentTemp(pfnGetTValue());

        for (int i = 0; i < 10 && !shutdown; i++)
            msleep(50);
    }

    if (handle) {
        CALL_FN(CloseSerialPort, handle)();
    }
    emit status(EDno);
}
