#include "modules.hpp"
#include "mainwindow.h"
#include "EG3000.h"
#include "hardware/relay.hpp"
#include "hardware/control3030.hpp"
#include "ccd/ccdplot.h"
#include "hardware/pmsplot.h"
#include "USBVideo/videoplayer.h"
#include <QApplication>
#include <QSettings>
#include <QTranslator>
#include <QDesktopWidget>
#include <QSplashScreen>
#include <QMessageBox>
#include <QTimer>


QMutex *ftdi_mutex = 0;


#ifdef Q_OS_UNIX
#include <sys/types.h>
#include <sys/stat.h>
#endif

#define NOMINMAX
#include "windows.h"

#ifdef NST_DELAYLOAD
#include <delayimp.h>
FARPROC WINAPI __pfnDliFailureHook2__(unsigned dliNotify, PDelayLoadInfo pdli)
{
    if (dliNotify == dliFailLoadLib)
        qDebug() << "dll delayload failed" << pdli->szDll;
    if (dliNotify == dliFailGetProc) {
        qDebug() << "dll function delayload failed" << pdli->szDll;
        if (pdli->dlp.fImportByName)
            qDebug() << "name" << pdli->dlp.szProcName;
        else
            qDebug() << "ordinal" << pdli->dlp.dwOrdinal;
    }
    throw 0;
    return 0;
}
#endif


QSplashScreen *makeSplash()
{
#ifndef _ADROID
    QPixmap spl(":/Splash_window/spr_sh.png");

    {//update splash with text
        QPainter pp;
        if (!pp.begin(&spl))
            qDebug() << "no splash img";

        pp.setPen(Qt::white);
        pp.drawText(spl.rect().bottomLeft() + QPointF(5, -7), TAboutWindow::tr("__SOFTWARE_VERSION_STRING"));
        pp.setPen(Qt::gray);
        pp.drawText(spl.rect().bottomRight() + QPointF(-400, -70), TAboutWindow::tr("Developers :"));
        pp.drawText(spl.rect().bottomRight() + QPointF(-280, -70), TAboutWindow::tr("Anton Ikonnikov"));
        pp.drawText(spl.rect().bottomRight() + QPointF(-280, -50), TAboutWindow::tr("Igor Dushkin"));
        pp.drawText(spl.rect().bottomLeft() + QPointF(200, -300), TAboutWindow::tr("Nano Scan Technologies Ltd."));
        pp.drawText(spl.rect().bottomLeft() + QPointF(200, -270), TAboutWindow::tr("Powered by QT and QWT"));
        pp.end();
    }
#else
    QPixmap spl(qApp->desktop()->size());
    spl.fill(Qt::black);
#endif

    QSplashScreen *splash = new QSplashScreen;
    splash->setWindowFlags(splash->windowFlags() | Qt::WindowStaysOnTopHint);
    splash->setPixmap(spl);
    spl = QPixmap(); //clear memory
    splash->setEnabled(false);
    splash->show();
    int sp_align = Qt::AlignBottom | Qt::AlignRight;
#ifdef _ADROID
    sp_align = Qt::AlignCenter;
#endif
    splash->showMessage(QObject::tr("Creating application interface..."), sp_align, Qt::white);
#ifdef _ADROID
    qApp->processEvents();
#endif

    return splash;
}


#ifndef _ADROID
#include <QSharedMemory>
#define UNIQUE_KEY "nspec_signleapp_key"

class SingleApp : public I_MODULE
{
public:
    SingleApp() {
        singleapp_sharedmem.setKey(UNIQUE_KEY);
        if (!(is_running = singleapp_sharedmem.attach())) {
            if (!singleapp_sharedmem.create(1024)) {
                qDebug() << "Unable to create single instance.";
                is_running = true;
                return;
            }

            QByteArray msg("0\0"); // default value to note that no message is available.
            setMessage(msg);
        }
    }
    ~SingleApp() {
        if (singleapp_sharedmem.isAttached())
            singleapp_sharedmem.detach();
    }
    bool setMessage(const QByteArray ba) {
        if (!singleapp_sharedmem.lock())
            return false;

        memcpy((char *)singleapp_sharedmem.data(), ba.constData(),
               qMin(singleapp_sharedmem.size(), ba.size()));

        return singleapp_sharedmem.unlock();
    }
    QByteArray getMessage() {
        if (!singleapp_sharedmem.lock())
            return QByteArray();

        QByteArray byteArray = QByteArray((char*)singleapp_sharedmem.constData(), singleapp_sharedmem.size());

        singleapp_sharedmem.unlock();
        return byteArray;
    }

    static const QLatin1String name() { return QLatin1String("singleapp"); }
    void *basecast(const QLatin1String name) { return name.latin1() != 0 ? 0 : static_cast<SingleApp *>(this); }
    void setRunning(bool on) {
        if (!on && singleapp_sharedmem.isAttached())
            singleapp_sharedmem.detach();
    }

    QSharedMemory singleapp_sharedmem;
    bool is_running;
};
QByteArray singleappGet()
{
    SingleApp *sa = (SingleApp *)ModuleManager::instance->find1("singleapp")->basecast();
    return sa->getMessage();
}
bool singleappSet(const QByteArray ba)
{
    SingleApp *sa = (SingleApp *)ModuleManager::instance->find1("singleapp")->basecast();
    return sa->setMessage(ba);
}
#endif

class RunFile : public I_MODULE
{
public:
    RunFile() {
        QFile lockfile("nspec.run");
        lockfile.open(QIODevice::WriteOnly);
    }
    static const QLatin1String name() { return QLatin1String("runfile"); }
    void setRunning(bool on) {
        if (!on)
            QFile::remove("nspec.run");
    }
};


int main(int argc, char *argv[])
{
#ifdef NST_DELAYLOAD
    __pfnDliFailureHook2 = __pfnDliFailureHook2__;
#endif
#ifdef Q_OS_UNIX
    umask(0);
#endif

    QApplication::setEffectEnabled(Qt::UI_AnimateMenu, false);
    QApplication::setEffectEnabled(Qt::UI_FadeMenu, false);
    QApplication::setEffectEnabled(Qt::UI_AnimateCombo, false);
    QApplication::setEffectEnabled(Qt::UI_AnimateTooltip, false);
    QApplication::setEffectEnabled(Qt::UI_FadeTooltip, false);
    QApplication::setEffectEnabled(Qt::UI_AnimateToolBox, false);

    QLocale::setDefault(QLocale::system());

    QApplication app(argc, argv);
    app.setQuitOnLastWindowClosed(false);
#ifndef _WIN32
    QApplication::setWindowIcon(QIcon(":/MainToolbar/About_nst.png"));
#endif

    InitPlotIO();

#if (QT_VERSION >= QT_VERSION_CHECK(5, 0, 0))
    QTimer *precision_timer = new QTimer(qApp);
    precision_timer->setTimerType(Qt::PreciseTimer);
    precision_timer->start(666);
#elif defined(_WIN32)
    if (timeBeginPeriod(1) != TIMERR_NOERROR)
        qDebug() << "no high-res timer";
#endif

    QElapsedTimer startup;
    startup.start();

    //change working directory to app dir, else data won't be found
#ifndef _ADROID
    QDir::setCurrent(app.applicationDirPath());
#endif
#ifdef Q_OS_MAC
    //only if bundle in CONFIG, nobundle not requires this
    QDir dir = QDir::current(); dir.cdUp(); dir.cdUp(); dir.cdUp();
    QDir::setCurrent(dir.path());
#endif
#ifndef _WIN32
    QString cfgpath = QDir::homePath() + "/.config/nspec";
    QDir::current().mkpath(cfgpath);
    QDir::setCurrent(cfgpath);
#endif

    //parses command line for file open
    bool open_file_on_load = false;
    QString filename;
    for (int i = 1; i < app.arguments().size(); i++) {
        QString arg = app.arguments().at(i);

        if (arg.startsWith("--")) {
            continue;
        }

        if (arg.endsWith(".nstdat") &&
                QFileInfo(arg).exists())
        {
            open_file_on_load = true;
            filename = arg;
            qDebug() << "command line open file" << filename;
            break;
        }
    }

#ifndef _ADROID
    SingleApp *singleapp = new SingleApp;
    if (singleapp->is_running) {
        if (open_file_on_load) {
            QByteArray msg("2");
            msg.append(filename.toUtf8());
            msg.append('\0');
            singleapp->setMessage(msg);
        } else {
            QByteArray msg("1\0");
            singleapp->setMessage(msg);
        }

        qDebug() << "already running, my pid =" << qApp->applicationPid();
        delete singleapp;
        return 0;
    }
#endif


    QDir::current().mkpath("Param");
    QDir::current().mkpath(TContainer::getDataDir());

    if (!QDir::current().exists("Param/gui.conf") &&
            !QDir::current().exists("Param/controller.conf")) { //first start
#ifdef _ADROID
        QSettings settings00("Param/gui.conf", QSettings::IniFormat);
        settings00.setValue("display/spectroscopy", false);
        settings00.setValue("display/lithography", false);
#endif
#ifndef _NO_FTDI
        QSettings settings01("Param/controller.conf", QSettings::IniFormat);
        settings01.setValue("use_eg4000", true);
#endif
    }

    bool useZ3_zobj = false;

    QTranslator qtTranslator;
    QTranslator translator;
    {//gui settings
        QSettings settings("Param/gui.conf", QSettings::IniFormat);

        QFont font;
        font.fromString(settings.value("font", "Arial,10,-1,5,50,0,0,0,0,0").toString());
        qApp->setFont(font);

        QString translation = settings.value("translation", QString(":/translation_base")).toString();
        translator.load(translation);
        app.installTranslator(&translator);
        app.setProperty("translation", translation);

        qtTranslator.load(QString("qt_") + (translation.contains("russian") ? "ru" : "en"));
        app.installTranslator(&qtTranslator);

#if (QT_VERSION >= QT_VERSION_CHECK(5, 0, 0)) && !defined(_ADROID)
        double scale = qApp->desktop()->logicalDpiX() / 101.;
#else
        double scale = qApp->desktop()->physicalDpiX() / 101.;
#endif

        int Bsize = settings.value("buttonsize", 25).toInt();
        QString style = "QAbstractButton, QComboBox, QTabWidget, QTabBar, QToolBar { qproperty-iconSize: " //, QAbstractItemView
                + QString::number(Bsize)
                + "px "
                + QString::number(Bsize)
                + "px; }"
                + "QGroupBox { border: 1px solid gray; border-radius: 5px; margin-top: 5px; margin-bottom: 0px; padding-top: 5px; padding-bottom: 1px; }"
                + "QGroupBox::title {top:" + QString::number(int(-7*scale)) + " px;left: 10px; subcontrol-origin: border; }"
                + "QComboBox { border: 1px solid #0000DF; border-radius: 3px; padding: 1px " + QString::number(int(5*scale)) + "px 1px 3px; }"
                + "QComboBox:focus { border: 2px solid #5050DF; border-radius: 3px; padding: 1px " + QString::number(int(5*scale)) + "px 1px 3px; }"
                + "QLineEdit { border: 1px solid #0000DF; border-radius: 3px; }" //, QDoubleSpinBox
                + "QTabBar::tab:only-one[documentMode=\"true\"] { height: 0 px; width: 0 px; }";

        app.setStyleSheet(style);

        app.setProperty("buttonsize", Bsize); //TODO remove

        useZ3_zobj = settings.value("display/oscilloscope/z3", false).toBool();
    }


    QSplashScreen *splash = makeSplash();

    ModuleManager modules;
    modules.addToFactory<ManagedThreadedCleaner>();

    modules.add(singleapp);

    bool crashed = QFile::exists("nspec.run");
    modules.add(new RunFile);

    Control3030 *eg3030 = 0;
    {
        QSettings settings("Param/controller.conf", QSettings::IniFormat);
        if (settings.value("use_dual_3030", false).toBool()) {
            ftdi_mutex = new QMutex();
            eg3030 = (Control3030 *)modules.load("eg3030")->basecast();
        }
    }
    USB_THREAD_GROUP *eg3000 = (USB_THREAD_GROUP *)modules.load("eg3000")->basecast();

#ifdef USE_RELAY
    modules.add(new RelayModule);
#endif

    MainWindow window;
    modules.add(&window);

    CCDPlot *ccdplot = 0;
    QWidget *parentw = window.ContentStackedWidget;
    {
        QSettings settings("Param/gui.conf", QSettings::IniFormat);

        if (settings.value("display/data", true).toBool())
            modules.load("data", parentw);

        if (settings.value("display/spectroscopy", true).toBool())
            ccdplot = (CCDPlot *)modules.load("ccdplot", parentw)->basecast();

        if (settings.value("display/scan", true).toBool())
            modules.load("scan", parentw);

        bool spm = false;
        if (spm = settings.value("display/spm", true).toBool())
            modules.load("spm", parentw);

        if (settings.value("display/motors", false).toBool() || spm)
            modules.load("motors");

        I_MODULE *oscm = modules.load("oscilloscope", parentw);
        if (!settings.value("display/oscilloscope", true).toBool()) {
            if (QPushButton *pb = window.guiModuleButtons.value(oscm))
                pb->hide();
            window.SWusb1->setMaximumWidth(spm || settings.value("display/scan", true).toBool() ? 1000 : 0); //instead of setVisible(), as it not works in toolbars :/ fuck qt.
        }
        {
            QSettings settings2("Param/controller.conf", QSettings::IniFormat);
            if (settings2.value("use_dual_3030", false).toBool())
                modules.load("oscilloscope", parentw);
        }

        if (settings.value("display/video", true).toBool())
            modules.load("video", parentw);
        if (settings.value("display/video2", false).toBool())
            modules.load("video", parentw);

        if (settings.value("display/lithography", true).toBool())
            modules.load("litho", parentw);

        if (settings.value("net/srv_on", false).toBool())
            modules.add(new MobileServer);
    }

    modules.add(new TOptionsModule);

    modules.add(new TAboutModule);



    { //zobjective range
        QList<I_MODULE *> l = modules.find(QLatin1String("oscilloscope"));
        Oscilloscope *osc = l.isEmpty() ? 0 : (Oscilloscope *)(eg3030 && !useZ3_zobj ? l.first() : l.last())->basecast();
        //(eg3030 && !useZ3_zobj ? window.osc2 : window.osc)
        XYZCalibrations *calib = osc->CalibWidget;
        const char *signal = !useZ3_zobj ? SIGNAL(sig_XYZ_range2(double,double,double)) : SIGNAL(sig_XYZ_range3(double,double,double));

        if (ccdplot)
            app.connect(calib, signal, ccdplot, SLOT(sl_set_XYZ_range(double, double, double)));
        foreach (I_MODULE *m, modules.find(QLatin1String("video")))
            if (VideoPlayer *v = (VideoPlayer *)m->basecast())
                app.connect(calib, signal, v, SLOT(sl_set_XYZ_range(double, double, double)));
    }
    {
        QObject *controller = eg3030 && !useZ3_zobj ? (QObject *)eg3030 : (QObject *)&eg3000->wThread;
        //zobjective sync
        const char *signal = !useZ3_zobj ? SIGNAL(z2Update(double)) : SIGNAL(z3Update(double));
        //zobjective emit
        const char *slot = !useZ3_zobj ? SLOT(Go_to_Zobject(double)) : SLOT(Go_to_Z3(double));

        if (ccdplot) {
            app.connect(controller, signal, ccdplot->zPos, SLOT(setValue(double)));
            app.connect(ccdplot->zPos, SIGNAL(newValue(double)), controller, slot);
            if (ccdplot->pms_plot) {
                app.connect(controller, signal, ccdplot->pms_plot->zPos, SLOT(setValue(double)));
                app.connect(ccdplot->pms_plot->zPos, SIGNAL(newValue(double)), controller, slot);
            }
        }
        foreach (I_MODULE *m, modules.find(QLatin1String("video")))
            if (VideoPlayer *v = (VideoPlayer *)m->basecast()) {
                app.connect(controller, signal, v->zPos, SLOT(setValue(double)));
                app.connect(v->zPos, SIGNAL(newValue(double)), controller, slot);
            }
        QList<I_MODULE *> l = modules.find(QLatin1String("oscilloscope"));
        Oscilloscope *osc = l.isEmpty() ? 0 : (Oscilloscope *)l.last()->basecast();
        if (osc) {
            app.connect(controller, signal, osc->DacsWidget->Zobj, SLOT(setValue(double)));
            app.connect(osc->DacsWidget->Zobj, SIGNAL(newValue(double)), controller, slot);
        }
    }


    if (sizeof(BYTE) != 1 || sizeof(WORD) != 2 || sizeof(DWORD) != 4 || sizeof(float) != 4)
        qDebug() << "wrong data size" <<
                    sizeof(BYTE) << sizeof(WORD) << sizeof(DWORD) << sizeof(float);
    if (sizeof(EG_CalData) != 228 || sizeof(EG_ForceCurve) != 20 ||
        sizeof(EG_ScanParams) != 108 || sizeof(EG_StepMotorData) != 16)
        qDebug() << "wrong eg_data size" <<
                    sizeof(EG_CalData) << sizeof(EG_ForceCurve) <<
                    sizeof(EG_ScanParams) << sizeof(EG_StepMotorData);

    splash->close();
    splash->deleteLater();
    window.show();

    qDebug() << "startup time" << startup.elapsed();


    bool skip_autocalib = false;
    bool crash_test = false;
    {
        QSettings settings("Param/gui.conf", QSettings::IniFormat);
        if (ccdplot && settings.value("autoinit_ccd", false).toBool()) crash_test = true;
        else if (settings.value("autocalib_sensors", false).toBool()) crash_test = true;
        else if (settings.value("autocalib_pid", false).toBool()) crash_test = true;
    }
    if (crashed && crash_test) {
        skip_autocalib = (QMessageBox::Yes ==
                          QMessageBox::warning(&window, app.tr("NSpec"),
                                               app.tr("Program possibly was not shutdown properly.") + "\n" +
                                               app.tr("Do you want to skip automatic calibrations?"),
                                               QMessageBox::Yes, QMessageBox::No));

        foreach (I_MODULE *m, modules.find(QLatin1String("oscilloscope"))) {
            Oscilloscope *osc = (Oscilloscope *)m->basecast();
            osc->CalibWidget->skip_autocalib = skip_autocalib;
        }
    }


    if (open_file_on_load)
        if (I_MODULE *datam = ModuleManager::instance->find1("data")) {
            DataPlot *data = (DataPlot *)datam->basecast();
            data->Container->OpenFile(filename);
        }


    modules.setRunning();
    if (ccdplot && !skip_autocalib)
        ccdplot->opticsReset();


    int retv = app.exec();

#if (QT_VERSION >= QT_VERSION_CHECK(5, 0, 0))
#elif defined(_WIN32)
    timeEndPeriod(1);
#endif

    return retv;
}
