#include "oscilloscope.h"

#include <QSettings>
#include <QMessageBox>
#include <QCloseEvent>
#include <QHBoxLayout>
#include <QStackedLayout>
#include <QCheckBox>
#include "mainwindow.h"
#include "autocalib.hpp"
#include "xyzcalibrations.hpp"

#include "EG3000.h"
extern USB_WRITE_THREAD *_wThread; //from modules.cpp


QVector<QPointer<QComboBox> > cb_channels_list;
void changeCBname(int srcidx, QString name)
{
    for (int i = 0; i < cb_channels_list.size(); i++)
        if (QComboBox *box = cb_channels_list.at(i))
            for (int i = 0; i < box->count(); i++)
                if (box->itemData(i) == srcidx) {
                    box->setItemText(i, name);
                    break;
                }
}

void Oscilloscope::fillCBchannels(QComboBox *box) const
{
    if (second_3030) {
        cb_channels_list << box;

        QString scanner;
        //if (head || z3)
            scanner = " " + QObject::tr("stage");

        box->addItem(QObject::tr("OFF"), srcNone);
        box->addItem(QObject::tr("Ux") + scanner, srcDACX2);
        box->addItem(QObject::tr("Uy") + scanner, srcDACY2);
        box->addItem(QObject::tr("Uz") + scanner, srcDACZ2);

        box->addItem(QObject::tr("X Sensor") + scanner, srcCX2);
        box->addItem(QObject::tr("Y Sensor") + scanner, srcCY2);
        box->addItem(QObject::tr("Z Sensor") + scanner, srcCZ2);

        box->addItem(QObject::tr("ADC 4/PMT"), srcADC4);
        box->addItem(QObject::tr("ADC 5"), srcADC5);

        return;
    }
    fillCBchannels_(box);
}

void fillCBchannels_(QComboBox *box, bool want_xyz)
{
    cb_channels_list << box;

    QSettings settings("Param/gui.conf", QSettings::IniFormat);
    bool spm = /*!second_3030 && */settings.value("display/spm", true).toBool();
    bool head = settings.value("display/oscilloscope/head", true).toBool();
    bool stage = settings.value("display/oscilloscope/stage", true).toBool();
    bool z3 = settings.value("display/oscilloscope/z3", false).toBool();
    bool add = show_additional_gui();
    bool adv = show_advanced_gui();
    bool hasADDACS1234 = false;
    {
#ifndef _NO_FTDI
        QSettings settings("Param/controller.conf", QSettings::IniFormat);
        hasADDACS1234 = !settings.value("use_eg4000", false).toBool();
#endif
    }

    box->addItem(QObject::tr("OFF"), srcNone);
    if (spm) {
        box->addItem(QObject::tr("Height"), srcHeight);
        box->addItem(QObject::tr("DFL"), srcDFL);
        box->addItem(QObject::tr("LFL")/*+"/"+QObject::tr("ITip")*/, srcLFL);
        box->addItem(QObject::tr("MAG"), srcMag1);
        box->addItem(QObject::tr("Phase"), srcPhase1);
    }

    if (want_xyz || (add && adv)) {
        if (head) {
            QString scanner;
            if (stage || z3)
                scanner = " " + QObject::tr("head");
            box->addItem(QObject::tr("Ux") + scanner, srcDACX);
            box->addItem(QObject::tr("Uy") + scanner, srcDACY);
            box->addItem(QObject::tr("Uz") + scanner, srcDACZ);
        }
        if (stage) {
            QString scanner;
            if (head || z3)
                scanner = " " + QObject::tr("stage");
            box->addItem(QObject::tr("Ux") + scanner, srcDACX2);
            box->addItem(QObject::tr("Uy") + scanner, srcDACY2);
            box->addItem(QObject::tr("Uz") + scanner, srcDACZ2);
        }
        if (z3) {
            QString scanner;
            if (head || stage)
                scanner = " " + QObject::tr("z3");
            box->addItem(QObject::tr("Uz") + scanner, srcDACZ3);
        }

        if (head) {
            QString scanner;
            if (stage || z3)
                scanner = " " + QObject::tr("head");
            box->addItem(QObject::tr("X Sensor") + scanner, srcCX);
            box->addItem(QObject::tr("Y Sensor") + scanner, srcCY);
            box->addItem(QObject::tr("Z Sensor") + scanner, srcCZ);
        }
        if (stage) {
            QString scanner;
            if (head || z3)
                scanner = " " + QObject::tr("stage");
            box->addItem(QObject::tr("X Sensor") + scanner, srcCX2);
            box->addItem(QObject::tr("Y Sensor") + scanner, srcCY2);
            box->addItem(QObject::tr("Z Sensor") + scanner, srcCZ2);
        }
        if (z3) {
            QString scanner;
            if (head || stage)
                scanner = " " + QObject::tr("z3");
            box->addItem(QObject::tr("Z Sensor") + scanner, srcCZ3);
        }
    } else {
        if (head) {
            QString scanner;
            if (stage)
                scanner = " " + QObject::tr("head");
            box->addItem(QObject::tr("Z Sensor") + scanner, srcCZ);
        }
        if (stage) {
            QString scanner;
            if (head)
                scanner = " " + QObject::tr("stage");
            box->addItem(QObject::tr("Z Sensor") + scanner, srcCZ2);
        }
    }

    if (add && (adv || !spm)) {
        box->addItem(QObject::tr("ADC 1"), srcADC1);
        box->addItem(QObject::tr("ADC 2"), srcADC2);
        if (spm) box->addItem(QObject::tr("ADC 3"), srcADC3);
    }
    if (hasADDACS1234) {
        box->addItem(QObject::tr("ADC 4/PMT"), srcADC4);
        box->addItem(QObject::tr("ADC 5"), srcADC5);
    }
    box->addItem(QObject::tr("ADC AUX 1"), srcADCHR1);
    box->addItem(QObject::tr("ADC AUX 2"), srcADCHR2);

    if (spm && add) {
        box->addItem(QObject::tr("Tip V"), srcVTip);
        box->addItem(QObject::tr("MAG(LFL)"), srcMag2);
    }
    if (spm && add && adv) {
        box->addItem(QObject::tr("DInt"), srcIntensity);
        box->addItem(QObject::tr("MAG Sin"), srcMagSin1);
        box->addItem(QObject::tr("MAG Cos"), srcMagCos1);
    }
}

void Oscilloscope::FixSecond3030Gui()
{
    DacsWidget->DacsChannel->clear();
    DacsWidget->DacsChannel->addItem(tr("X "), 8);
    DacsWidget->DacsChannel->addItem(tr("Y "), 9);
    DacsWidget->DacsChannel->addItem(tr("Z "), 10);
    DacsWidget->DacsChannel->setCurrentIndex(0);

    acp->CBScanner->setCurrentIndex(1);
    acp->CBScanner->setEnabled(false);
}

void Oscilloscope::device_status(int code)
{
    QString text;
    int type = Notification::Warn;

    switch (code) {
    default:
        return;
    case 0:
        text = tr("Controller has no firmware.") + " " + tr("Program it from oscilloscope window.");
        type = Notification::Bad;
        break;
    case 1:
        text = tr("Controller load timeout.") + " " + tr("Reboot and program it.");
        type = Notification::Bad;
        break;
    case 2:
        text = tr("Waiting for device to program.");
        break;
    case 3:
        text = tr("Programming. Please wait. Do not remove power from the device.");
        break;
    case 4:
        text = tr("Programming succesfully completed.");
        type = Notification::Good;
        break;
    };

    destructStatus(device_status_notice);
    device_status_notice = new Notification(text, (Notification::Tint)type, 60);
    doBeep();
}

int Oscilloscope::times_constructed = 0;
Oscilloscope::Oscilloscope(QWidget *parent) : QWidget(parent)
{
    second_3030 = times_constructed != 0;
    times_constructed++;

    for (int i = 0; i < 4; i++)
        k_m_nm[i] = 1;
    last_osc_ch = 0;
    value_debug = 0;

    refreshRateLimit.invalidate();

    oscRate = 4000;
    oscMaxLength = 10 * oscRate;
    current_t_count = 0;
    last_channel_idx = -1;
    OscLineLength = 256;

    plot::ResizeOwner(curveData, 4);
    curveData[0]->Pen.setColor(Qt::red);
    curveData[1]->Pen.setColor(QColor(0, 170, 0)); //Qt::green
    curveData[2]->Pen.setColor(Qt::blue);
    curveData[3]->Pen.setColor(Qt::black);
    for (int i = 0; i < curveData.size(); i++)
        curveData[i]->setXCalib(0, double(1) / oscRate);

    setWindowTitle(tr("Oscilloscope - NSpec"));
    GUICreate();
    connect(acp->BReturn, SIGNAL(clicked()), PBCalibMode, SIGNAL(clicked()));

    if (second_3030)
        FixSecond3030Gui();

    connect(MyPl, SIGNAL(CurveColourChanged(QColor, int)), this, SLOT(on_MyPl_CurveColourChanged(QColor, int)));
    for (int i = 0; i < 4; i++)
        connect(CBChannel[i], SIGNAL(currentIndexChanged(int)), this, SLOT(change_ch()));
    connect(MyPl, SIGNAL(AddDataToContainer(plot::TCurvesDataOwner,plot::TMetaStorage)), SLOT(AddDataToContainer_slot(plot::TCurvesDataOwner,plot::TMetaStorage)));

    MyPl->Pure2dPlot->SourceData = plot::getUnLock(curveData);
    MyPl->Refresh();
}

void Oscilloscope::AddDataToContainer_slot(plot::TCurvesDataOwner src, plot::TMetaStorage attributes)
{
    attributes.insert("Name", MyPl->Pure2dPlot->Title);
    attributes.insert("Source", "Oscilloscope");

    for (int i = 0; i < 4; i++) {
        if (CBChannel[i]->currentIndex() <= 0)
            continue;
        attributes.insert("Channel " + QString::number(i), CBChannel[i]->currentText());
        attributes.insert("Average", LEdAverage[i]->text());
        attributes.insert("RMS", LEdStDev[i]->text());
    }

    emit AddDataToContainer(src, attributes);
}


void Oscilloscope::closeEvent(QCloseEvent *event)
{
    PBToNewWindow->click();
    event->ignore();
}


void Oscilloscope::GUICreate()
{
    QHBoxLayout *MainLayout = new QHBoxLayout(this);

    OscStackedWidget = new QStackedWidget(this);
    OscStackedWidget->setContentsMargins(0, 0, 0, 0);

    CalibWidget = new XYZCalibrations(this, this, second_3030);
    connect(CalibWidget, SIGNAL(sig_XYZ_range(double,double,double)), this, SLOT(sl_xyz(double,double,double)));
    connect(CalibWidget, SIGNAL(sig_XYZ_range2(double,double,double)), this, SLOT(sl_xyz2(double,double,double)));
    connect(CalibWidget, SIGNAL(sig_XYZ_range3(double,double,double)), this, SLOT(sl_xyz3(double,double,double)));
    connect(CalibWidget, SIGNAL(input_calib_updated()), this, SLOT(change_ch()));
    connect(CalibWidget->PrintDebug, SIGNAL(clicked(bool)), this, SLOT(showDebug()));

    MainLayout->addWidget(OscStackedWidget, 1);
    MainLayout->addWidget(CalibWidget);


    QWidget *leftwid = new QWidget(OscStackedWidget);
    QHBoxLayout *leftl = new QHBoxLayout(leftwid);
    leftl->setContentsMargins(0, 0, 0, 0);

    acp = new AutoCalibPlot(OscStackedWidget);
    acp->osc = this;

    OscStackedWidget->addWidget(leftwid);
    OscStackedWidget->addWidget(acp);

    QVBoxLayout *VBLLeft = new QVBoxLayout;
    VBLLeft->setAlignment(Qt::AlignTop);
    VBLLeft->setContentsMargins(0, 0, 0, 0);

    QVBoxLayout *VBLMid = new QVBoxLayout;
    VBLMid->setSpacing(3);
    VBLMid->setContentsMargins(0, 0, 6, 0);

    leftl->addLayout(VBLLeft);
    leftl->addLayout(VBLMid, 1);


    DacsWidget = new DacsControl(leftwid);

    MyPl = new plot::T2dPlot;
    MyPl->SetGridVisible(true);
    MyPl->SetAutoScalingEnabled(false);
    MyPl->Pure2dPlot->XAxisUnit.SetDescAndSIUnit(tr("time"), tr("s"));
    MyPl->Pure2dPlot->YAxisUnit.SetDescription(tr("None"));
    MyPl->Pure2dPlot->SetAllXScales(0, 10);
    MyPl->Pure2dPlot->SetAllYScales(-11, 11);
    MyPl->Pure2dPlot->SetTitle(tr("Osc curve"));
    MyPl->layout()->setContentsMargins(0, 0, 0, 0);

    VBLMid->addWidget(MyPl, 1);
    VBLMid->addWidget(DacsWidget);


    QHBoxLayout *BLayout = new QHBoxLayout;
    BLayout->setSpacing(6);
    BLayout->setAlignment(Qt::AlignLeft);

    QHBoxLayout *HBLT = new QHBoxLayout;

    VBLLeft->addLayout(BLayout);
    VBLLeft->addLayout(HBLT);


    //----------------
    PBToNewWindow = new TwoStateButton(":/Osc_window/Detach.png",":/Osc_window/Attach.png");
    PBToNewWindow->setToolTip(QObject::tr("Individual window on/off"));

    start = new TwoStateButton(":start.png",":stop.png");
    connect(start, SIGNAL(state_changed(bool)), this, SLOT(slot_Osc_Start_Stop(bool)));

    SBT = new Edit(1, 60);
    SBT->setImmediateMode();
    SBT->setValue(10);
    connect(SBT, SIGNAL(newValue(int)), this, SLOT(s_setT(int)));

    PBCalibMode = new TwoStateButton(":/Osc_window/CalibOf.png",":/Osc_window/CalibOn.png");
    PBCalibMode->setToolTip(tr("Show/hide calibration window"));
    PBCalibMode->hide();
    connect(PBCalibMode, SIGNAL(state_changed(bool)), SLOT(calib_plot_change(bool)));

    BToCalib = new QPushButton(makeHugeIcon(":/Osc_window/CalibOf.png"), QString());
    BToCalib->setToolTip(tr("Switch to calibrations"));
    connect(BToCalib, SIGNAL(clicked()), PBCalibMode, SIGNAL(clicked()));

    HBLT->addWidget(new QLabel(tr("T, sec")));
    HBLT->addWidget(SBT);
    BLayout->addWidget(BToCalib);
    BLayout->addWidget(PBToNewWindow);
    BLayout->addWidget(start, 1);

    CBShowRelative = new QCheckBox(tr("Relative scale"));
    VBLLeft->addWidget(CBShowRelative);
    connect(CBShowRelative, SIGNAL(toggled(bool)), SLOT(change_ch()));
    bshowrelative = false;


    QGroupBox *GrBChannel[4];
    GrBChannel[0] = new QGroupBox(tr("Channel A"));
    GrBChannel[1] = new QGroupBox(tr("Channel B"));
    GrBChannel[2] = new QGroupBox(tr("Channel C"));
    GrBChannel[3] = new QGroupBox(tr("Channel D"));
    LEdColour[0] = new TRectWidget(Qt::red);
    LEdColour[1] = new TRectWidget(QColor(0, 170, 0));
    LEdColour[2] = new TRectWidget(Qt::blue);
    LEdColour[3] = new TRectWidget(Qt::black);

    for (int i = 0; i < 4; i++) {
        CBChannel[i] = new QComboBox(GrBChannel[i]);
        fillCBchannels(CBChannel[i]);

        LAverage[i] = new QLabel(tr("Average"));
        LStDev[i] = new QLabel(tr("RMS"));
        LAverage[i]->hide();
        LStDev[i]->hide();

        LEdAverage[i] = new QLineEdit;
        LEdAverage[i]->hide();
        LEdAverage[i]->setReadOnly(true);
        LEdAverage[i]->setFocusPolicy(Qt::NoFocus);

        LEdStDev[i] = new QLineEdit;
        LEdStDev[i]->hide();
        LEdStDev[i]->setReadOnly(true);
        LEdStDev[i]->setFocusPolicy(Qt::NoFocus);

        GrBChannel[i]->setFocusPolicy(Qt::StrongFocus);
        GrBChannel[i]->setFocusProxy(CBChannel[i]);
        CBChannel[i]->installEventFilter(this);


        connect(LEdColour[i], SIGNAL(clicked()), this, SLOT(colorClick()));

        VBLLeft->addSpacerItem(new QSpacerItem(12, 12, QSizePolicy::Maximum, QSizePolicy::Maximum));
        VBLLeft->addWidget(GrBChannel[i]);

        QVBoxLayout *VBLMainForGrB = new QVBoxLayout(GrBChannel[i]);
        VBLMainForGrB->setContentsMargins(6, 3, 6, 3);

        QVBoxLayout *VBLAverage = new QVBoxLayout;
        VBLAverage->setContentsMargins(0, 0, 0, 0);

        QVBoxLayout *VBLStDev = new QVBoxLayout;
        VBLStDev->setContentsMargins(0, 0, 0, 0);

        VBLAverage->addWidget(LAverage[i]);
        VBLAverage->addWidget(LEdAverage[i]);

        VBLStDev->addWidget(LStDev[i]);
        VBLStDev->addWidget(LEdStDev[i]);

        VBLMainForGrB->addLayout(VBLAverage);
        VBLMainForGrB->addLayout(VBLStDev);
        VBLMainForGrB->addWidget(CBChannel[i]);
        VBLMainForGrB->addWidget(LEdColour[i]);
    }

    BShowHideSett = new QPushButton(">>");
    BShowHideSett->setToolTip(tr("Show/hide calibration panel"));
    connect(BShowHideSett, SIGNAL(clicked(bool)), this, SLOT(sett_showhide()));
    VBLLeft->addWidget(BShowHideSett);
}

void Oscilloscope::sett_showhide()
{
    CalibWidget->setVisible(!CalibWidget->isVisibleTo(this));
    BShowHideSett->setText(CalibWidget->isVisibleTo(this) ? ">>" : "<<");
}
void Oscilloscope::calib_plot_change(bool val)
{
    OscStackedWidget->setCurrentIndex(val ? 1 : 0);
}


void Oscilloscope::s_setT(int t)
{
    oscMaxLength = t * oscRate;
    MyPl->Pure2dPlot->SetAllXScales(0, t);
    MyPl->Refresh(); //TODO delay refresh
}

QString toBinary(DWORD num, int bits = 16)
{
    QString s = "b";
    for (int i = bits-1; i >= 0; i--)
        s += (num & (1 << i)) ? "1" : "0";

    return s;
}

void Oscilloscope::newOscData(QByteArray data)
{
    const EG_OscData *sdata = (const EG_OscData *)data.constData();

    if (sdata->Channel >= 4 && sdata->Channel < 8)
        lsbData = data;
    if (sdata->Channel >= 4 || !CBChannel[sdata->Channel % 4]->currentIndex())
        return;

    if (sdata->Channel <= last_channel_idx) {
        current_t_count += OscLineLength;
        if (current_t_count + OscLineLength > oscMaxLength)
            current_t_count = 0;
    }
    last_channel_idx = sdata->Channel;
    if (current_t_count == 0)
        curveData[sdata->Channel]->YSeq.clear();

    if (!curveData[sdata->Channel]->YSeq.size())
        curveData[sdata->Channel]->setXCalib(double(current_t_count) / oscRate, double(1) / oscRate);

    const EG_OscData *sdata2 = (const EG_OscData *)lsbData.constData();
    if (lsbData.isEmpty() || sdata2->Channel % 4 != sdata->Channel)
        sdata2 = 0;

    if (value_debug) {
        value_debug--;
        qDebug() << sdata->Channel << (void *)sdata->Data[0] << toBinary(sdata->Data[0]);
        if (sdata2)
            qDebug() << "lsb" << (void *)sdata2->Data[0] << toBinary(sdata2->Data[0]);
    }
    BYTE source = sdata->Source;
    if (bshowrelative)
        source = srcNone; //default common calib

    for (int i = 0; i < OscLineLength; i++)
        curveData[sdata->Channel]->AddY(k_m_nm[sdata->Channel] * input_calib(source, sdata->Data[i] + (sdata2 ? ((double)sdata2->Data[i]) / 0x10000 : 0),
                                                                             (_wThread->FB_Scanner ? DacsWidget->Z2->edit->maximum() : DacsWidget->Z->edit->maximum())));

    lsbData.clear();

    GUIRefresh();
}

void Oscilloscope::GUIRefresh()
{
    if (refreshRateLimit.isValid() && refreshRateLimit.elapsed() < 60 + 4 * oscMaxLength / oscRate)
        return;

    refreshRateLimit.start();

    MyPl->Pure2dPlot->RefreshInc();

    for (int i = 0; i < 4 && i < curveData.size(); i++) {
        if (!osc_ch[i]) continue;

        plot::TSIUnit tunit;
        tunit.SetSIUnit(input_units(bshowrelative ? srcNone : osc_ch[i]));

        double vavg = avg(curveData[i]->YSeq) / k_m_nm[i];
        tunit.UpdateLegend(vavg);
        LEdAverage[i]->setText(QString::number(vavg / tunit.Multiplier, 'f', 3));
        LAverage[i]->setText(tr("Average, ") + tunit.Legend);

        double vrms = rms(curveData[i]->YSeq) / k_m_nm[i];
        tunit.UpdateLegend(vrms);
        LEdStDev[i]->setText(QString::number(vrms / tunit.Multiplier, 'f', 3));
        LStDev[i]->setText(tr("RMS, ") + tunit.Legend);
    }

    emit oscRefreshed();
}
void Oscilloscope::sl_osc_start()
{
    if (!start->on_off)
        start->change_state_man_with_sig(true);
}
void Oscilloscope::sl_osc_stop()
{
    start->change_state_man_with_sig(false);
}
void Oscilloscope::slot_Osc_Start_Stop(bool st)
{
    if (st) {
        if (MyPl->PlotControl->BMarkers->isChecked())
            MyPl->PlotControl->BMarkers->click();

        current_t_count = 0;
        last_channel_idx = -1;
        for (int i = 0; i < curveData.size(); i++)
            curveData[i]->YSeq.clear();

        lsbData.clear();

        emit osc_start();
    } else {
        emit osc_stop();
    }
}
void Oscilloscope::on_MyPl_CurveColourChanged(QColor c, int N)
{
    if (N < 0)
        for (int i = 0; i < 4; i++)
            on_MyPl_CurveColourChanged(c, i);
    else
        LEdColour[N]->SetColour(c);
}
bool Oscilloscope::eventFilter(QObject *Object, QEvent *Event)
{
    if (Event->type() == QEvent::FocusIn)
        for (int i = 0; i < 4; i++)
            if (Object == CBChannel[i]) {
                last_osc_ch = i;
                blockSignals(true);
                change_ch();
                blockSignals(false);
                break;
            }

    return QWidget::eventFilter(Object, Event);
}
void Oscilloscope::change_ch()
{
    bshowrelative = CBShowRelative->isChecked();
    for (int i = 0; i < 4; i++) {
        int ch = osc_ch[i] = CBChannel[i]->itemData(CBChannel[i]->currentIndex()).toInt();
        emit sig_ch_changed(i, ch);

        LEdAverage[i]->setVisible(ch != srcNone);
        LAverage[i]->setVisible(ch != srcNone);
        LEdStDev[i]->setVisible(ch != srcNone);
        LStDev[i]->setVisible(ch != srcNone);

        if (ch == srcNone) {
            curveData[i]->YSeq.clear();
            continue;
        }
        if (bshowrelative)
            ch = srcNone; //default common calib

        double zmax = qMax(DacsWidget->Z->edit->maximum(), DacsWidget->Z2->edit->maximum());
        double min = input_calib(ch, 0, zmax),
               max = input_calib(ch, 65535, zmax);

        plot::TSIUnit tunit;
        tunit.SetSIUnit(input_units(ch));
        tunit.UpdateLegend(min, max);
        k_m_nm[i] = 1. / tunit.Multiplier;
        min = min / tunit.Multiplier;
        max = max / tunit.Multiplier;

        if (i != last_osc_ch)
            continue;

        QString name = CBChannel[i]->currentText();
        MyPl->Pure2dPlot->YAxisUnit.SetDescAndSIUnit(name, tunit.Legend);
        MyPl->Pure2dPlot->SetAllYScales(1.1 * qMin(-10., qMin(min, max)), 1.1 * qMax(10., qMax(min, max)));

        MyPl->Refresh();
    }
}
void Oscilloscope::setChA(int src)
{
    CBChannel[0]->setCurrentIndex(CBChannel[0]->findData(src));
}
void Oscilloscope::colorClick()
{
    TRectWidget *w = qobject_cast<TRectWidget *>(sender());
    if (!w) return;

    int active = MyPl->Pure2dPlot->ActiveCurve;
    int idx = -1;
    for (int i = 0; i < 4; i++)
        if (w == LEdColour[i]) {
            idx = i;
            break;
        }
    if (idx == -1)
        return;

    MyPl->Pure2dPlot->ActiveCurve = idx;
    MyPl->PlotControl->AGCurveColours->actions().last()->trigger();
    //MyPl->on_CurveColour_changed();

    MyPl->Pure2dPlot->ActiveCurve = active;
}


void Oscilloscope::sl_xyz(double x, double y, double z)
{
    DacsWidget->X->setMaximum(x);
    DacsWidget->Y->setMaximum(y);
    DacsWidget->Z->setMaximum(z);
}
void Oscilloscope::sl_xyz2(double x, double y, double z)
{
    DacsWidget->X2->setMaximum(x);
    DacsWidget->Y2->setMaximum(y);
    DacsWidget->Z2->setMaximum(z);
}
void Oscilloscope::sl_xyz3(double, double, double z)
{
    DacsWidget->Z3->setMaximum(z);
}
void Oscilloscope::sl_z_osc2(double, double, double z)
{
    DacsWidget->Zobj->setMaximum(z);
}

void Oscilloscope::reset()
{
    value_debug = 0;

    int idx;
    idx = CBChannel[0]->currentIndex();
    CBChannel[0]->setCurrentIndex(0);CBChannel[0]->setCurrentIndex(idx);
    idx = CBChannel[1]->currentIndex();
    CBChannel[1]->setCurrentIndex(0);CBChannel[1]->setCurrentIndex(idx);
    idx = CBChannel[2]->currentIndex();
    CBChannel[2]->setCurrentIndex(0);CBChannel[2]->setCurrentIndex(idx);
    idx = CBChannel[3]->currentIndex();
    CBChannel[3]->setCurrentIndex(0);CBChannel[3]->setCurrentIndex(idx);

    start->change_state_man_with_sig(start->on_off);
}

void Oscilloscope::showDebug()
{
    qDebug() << "optical scanner flags:";
    for (int i = 0; i < 3; i++)
        qDebug() << acp->is_optical_scanner[i];

    value_debug = 0;
    for (int i = 0; i < 4; i++)
        if (CBChannel[i]->currentIndex())
            value_debug++;
}


DacsControl::DacsControl(QWidget *parent) : QWidget(parent)
{
    QSettings settings("Param/analog.conf", QSettings::IniFormat);
    double a0 = settings_double(settings, "dac/vtip_min", -10.);
    double b0 = settings_double(settings, "dac/vtip_max", 10.);
    double a1 = settings_double(settings, "dac/v1_min", -10.);
    double b1 = settings_double(settings, "dac/v1_max", 10.);
    double a2 = settings_double(settings, "dac/v2_min", -10.);
    double b2 = settings_double(settings, "dac/v2_max", 10.);
    double a3 = settings_double(settings, "dac/v3hr_min", -10.);
    double b3 = settings_double(settings, "dac/v3hr_max", 10.);
    double a4 = settings_double(settings, "dac/v4hr_min", -10.);
    double b4 = settings_double(settings, "dac/v4hr_max", 10.);

    Head_Dac = new DoubleChanger(a0, b0, 3);
    Extra_Dac_1 = new DoubleChanger(a1, b1, 3);
    Extra_Dac_2 = new DoubleChanger(a2, b2, 3);
    Extra_Dac_3 = new DoubleChanger(a3, b3, 3);
    Extra_Dac_4 = new DoubleChanger(a4, b4, 3);
    Z = new DoubleChanger(0, 50, 4);
    X = new DoubleChanger(0, 150, 4);
    Y = new DoubleChanger(0, 150, 4);
    Z2 = new DoubleChanger(0, 50, 4);
    X2 = new DoubleChanger(0, 150, 4);
    Y2 = new DoubleChanger(0, 150, 4);
    Z3 = new DoubleChanger(0, 50, 4);
    Zobj = new DoubleChanger(0, 50, 4);

    Head_Dac->setProperty("id", DataID::SPM_Vtip);
    Extra_Dac_1->setProperty("id", DataID::SPM_DAC1);
    Extra_Dac_2->setProperty("id", DataID::SPM_DAC2);
    Extra_Dac_3->setProperty("id", DataID::SPM_DAC3);
    Extra_Dac_4->setProperty("id", DataID::SPM_DAC4);
    X->setProperty("id", DataID::HV_DAC_X);
    Y->setProperty("id", DataID::HV_DAC_Y);
    Z->setProperty("id", DataID::HV_DAC_Z);
    X2->setProperty("id", DataID::HV_DAC_X2);
    Y2->setProperty("id", DataID::HV_DAC_Y2);
    Z2->setProperty("id", DataID::HV_DAC_Z2);
    Z3->setProperty("id", DataID::HV_DAC_Z3);
    Zobj->setProperty("id", DataID::HV_DAC_Zobj);

    DacsChannel = new QComboBox;
    {
        QSettings settings("Param/gui.conf", QSettings::IniFormat);
        if (settings.value("display/spm", true).toBool()) {
            DacsChannel->addItem(Oscilloscope::tr("Tip voltage"), 0);
        }
        bool hasADDACS1234 = false;
        {
#ifndef _NO_FTDI
            QSettings settings("Param/controller.conf", QSettings::IniFormat);
            hasADDACS1234 = !settings.value("use_eg4000", false).toBool();
#endif
        }
        if (hasADDACS1234) {
            DacsChannel->addItem(Oscilloscope::tr("DAC 1/PMT"), 1);
            DacsChannel->addItem(Oscilloscope::tr("DAC 2"), 2);
        }
        DacsChannel->addItem(Oscilloscope::tr("DAC AUX 1"), 3);
        DacsChannel->addItem(Oscilloscope::tr("DAC AUX 2"), 4);
        if (settings.value("display/oscilloscope/head", true).toBool()) {
            DacsChannel->addItem(Oscilloscope::tr("X head"), 5);
            DacsChannel->addItem(Oscilloscope::tr("Y head"), 6);
            DacsChannel->addItem(Oscilloscope::tr("Z head"), 7);
        }
        if (settings.value("display/oscilloscope/stage", true).toBool()) {
            DacsChannel->addItem(Oscilloscope::tr("X stage"), 8);
            DacsChannel->addItem(Oscilloscope::tr("Y stage"), 9);
            DacsChannel->addItem(Oscilloscope::tr("Z stage"), 10);
        }
        if (settings.value("display/oscilloscope/z3", false).toBool())
            DacsChannel->addItem(Oscilloscope::tr("Z3"), 11);
        //DacsChannel->addItem(Oscilloscope::tr("Z Objective"), 12);
    }
    connect(DacsChannel, SIGNAL(currentIndexChanged(int)), this, SLOT(s_dacs_channel(int)));


    QHBoxLayout *layout_Dacs = new QHBoxLayout(this);
    layout_Dacs->setSpacing(6);
    layout_Dacs->setContentsMargins(0, 0, 0, 0);

    box_Ex_Dac1 = new QGroupBox(Oscilloscope::tr("Tip voltage, V"), this);
    QLayout *layout_Ex_Dac = new QStackedLayout(box_Ex_Dac1);

    QGroupBox* box_Ex_Dac2 = new QGroupBox(Oscilloscope::tr("DACS Channel"), this);
    QHBoxLayout *layout_Ex_Dac22 = new QHBoxLayout(box_Ex_Dac2);
    layout_Ex_Dac22->setContentsMargins(6, 3, 6, 3);

    layout_Dacs->addWidget(box_Ex_Dac1);
    layout_Dacs->addWidget(box_Ex_Dac2);

    layout_Ex_Dac22->addWidget(DacsChannel);

    QList<QWidget *> wlst;
    wlst << Head_Dac << Extra_Dac_1 << Extra_Dac_2 << Extra_Dac_3 << Extra_Dac_4 << X << Y << Z << X2 << Y2 << Z2 << Z3 << Zobj;
    foreach (QWidget *w, wlst) {
        w->setContentsMargins(6, 3, 6, 3);
        layout_Ex_Dac->addWidget(w);
    }
}

DoubleChanger *DacsControl::get_dacs_channel(QComboBox *CB)
{
    int ch = CB->itemData(CB->currentIndex()).toInt();
    return (DoubleChanger *)(((QStackedLayout *)box_Ex_Dac1->layout())->itemAt(ch)->widget());
}
void DacsControl::fill_dacs_channel_CB(QComboBox *CB)
{
    for (int i = 0; i < DacsChannel->count(); i++)
        CB->addItem(DacsChannel->itemText(i), DacsChannel->itemData(i));
}
void DacsControl::s_dacs_channel(int ch)
{
    ch = DacsChannel->itemData(ch).toInt();

    ((QStackedLayout *)box_Ex_Dac1->layout())->setCurrentIndex(ch);
    QString title;
    switch (ch) {
    case 0:
        title = Oscilloscope::tr("Tip voltage") + ", " + QObject::tr("V");
        break;
    case 1:
        title = Oscilloscope::tr("DAC 1/PMT") + ", " + QObject::tr("V");
        break;
    case 2:
        title = Oscilloscope::tr("DAC 2") + ", " + QObject::tr("V");
        break;
    case 3:
        title = Oscilloscope::tr("DAC AUX 1") + ", " + QObject::tr("V");
        break;
    case 4:
        title = Oscilloscope::tr("DAC AUX 2") + ", " + QObject::tr("V");
        break;
    case 5:
        title = Oscilloscope::tr("X Position, ")+Oscilloscope::tr("micro")+QObject::tr("m");
        break;
    case 6:
        title = Oscilloscope::tr("Y Position, ")+Oscilloscope::tr("micro")+QObject::tr("m");
        break;
    case 7:
        title = Oscilloscope::tr("Z Position, ")+Oscilloscope::tr("micro")+QObject::tr("m");
        break;
    case 8:
        title = Oscilloscope::tr("X2 Position, ")+Oscilloscope::tr("micro")+QObject::tr("m");
        break;
    case 9:
        title = Oscilloscope::tr("Y2 Position, ")+Oscilloscope::tr("micro")+QObject::tr("m");
        break;
    case 10:
        title = Oscilloscope::tr("Z2 Position, ")+Oscilloscope::tr("micro")+QObject::tr("m");
        break;
    case 11:
        title = Oscilloscope::tr("Z3 Position, ")+Oscilloscope::tr("micro")+QObject::tr("m");
        break;
    case 12:
        title = Oscilloscope::tr("Zobj Position, ")+Oscilloscope::tr("micro")+QObject::tr("m");
        break;
    }
    box_Ex_Dac1->setTitle(title);
}
