#ifndef OSCILLOSCOPE_H
#define OSCILLOSCOPE_H

#include <QWidget>
#include <QElapsedTimer>
#include <QPointer>
#include "controls.h"
#include "edit.hpp"
#include "plot.h"
#include "defs.h"
#include "xyzcalibrations.hpp"

class TRectWidget: public QWidget
{
    Q_OBJECT
signals:
    void clicked();

public:
    QColor m_Colour;
    void SetColour(const QColor &c) { m_Colour = c; update(); }

    TRectWidget(const QColor &c = Qt::black): m_Colour(c) { setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Fixed); setMinimumSize(5, 7); }

    void paintEvent(QPaintEvent *) { QPainter p(this); p.fillRect(rect(), m_Colour); }
    void mouseDoubleClickEvent(QMouseEvent *) { emit clicked(); }
};

void fillCBchannels_(QComboBox *box, bool want_xyz = true);
void changeCBname(int srcidx, QString name);


class XYZCalibrations;
class DacsControl;
class AutoCalibPlot;
class Notification;
class Oscilloscope: public QWidget
{
    Q_OBJECT

public:
    Oscilloscope(QWidget *parent = 0);
    void fillCBchannels(QComboBox *box) const;

public slots:
    void s_setT(int t);
    void sl_osc_start();
    void sl_osc_stop();
    void slot_Osc_Start_Stop(bool);
    void change_ch();
    void setChA(int);
    void newOscData(QByteArray data);
    void GUIRefresh();
    void on_MyPl_CurveColourChanged(QColor, int);
    void colorClick();
    void sett_showhide();

    void showDebug();

    void reset();
    void sl_xyz(double, double, double);
    void sl_xyz2(double, double, double);
    void sl_xyz3(double, double, double);
    void sl_z_osc2(double, double, double);
    void calib_plot_change(bool);

    void device_status(int code);

    void AddDataToContainer_slot(plot::TCurvesDataOwner, plot::TMetaStorage);

signals:
    void osc_start();
    void osc_stop();
    void sig_ch_changed(int, int);
    void forwardedOscData(QByteArray);
    void oscRefreshed();

    void AddDataToContainer(plot::TCurvesDataOwner, plot::TMetaStorage);

public:
    void GUICreate();
    void FixSecond3030Gui();
    void closeEvent(QCloseEvent *event);
    bool second_3030;
    static int times_constructed;
    QPointer<Notification> device_status_notice;


    TwoStateButton *start;
    Edit *SBT;
    QPushButton *BToCalib;
    TwoStateButton *PBCalibMode;
    TwoStateButton *PBToNewWindow;
    QPushButton *BShowHideSett;
    QCheckBox *CBShowRelative;
    bool bshowrelative;

    plot::T2dPlot *MyPl;
    XYZCalibrations *CalibWidget;
    DacsControl *DacsWidget;
    AutoCalibPlot *acp;
    QStackedWidget *OscStackedWidget;

    QComboBox *CBChannel[4];
    TRectWidget *LEdColour[4];
    QLabel *LAverage[4];
    QLabel *LStDev[4];

    QLineEdit *LEdAverage[4];
    QLineEdit *LEdStDev[4];

    double k_m_nm[4];
    BYTE osc_ch[4];
    int last_osc_ch;
    int value_debug;

    plot::TCurvesDataOwner curveData;
    long current_t_count;
    int last_channel_idx;
    int oscMaxLength;
    int OscLineLength;
    int oscRate;
    QElapsedTimer refreshRateLimit;
    QByteArray lsbData;

    bool eventFilter(QObject *Object, QEvent *Event);
};


class DacsControl: public QWidget
{
    Q_OBJECT

public slots:
    DoubleChanger *get_dacs_channel(QComboBox *CB);
    void fill_dacs_channel_CB(QComboBox *CB);
    void s_dacs_channel(int ch);

public:
    DacsControl(QWidget *parent = 0);

    QComboBox *DacsChannel;
    QGroupBox *box_Ex_Dac1;

    DoubleChanger *Head_Dac;
    DoubleChanger *Extra_Dac_1;
    DoubleChanger *Extra_Dac_2;
    DoubleChanger *Extra_Dac_3;
    DoubleChanger *Extra_Dac_4;
    DoubleChanger *X;
    DoubleChanger *Y;
    DoubleChanger *Z;
    DoubleChanger *X2;
    DoubleChanger *Y2;
    DoubleChanger *Z2;
    DoubleChanger *Z3;
    DoubleChanger *Zobj;
};

#endif //OSCILLOSCOPE_H
