#ifndef SCANPLOT_H
#define SCANPLOT_H

#include <plot.h>
#include "controls.h"
#include "edit.hpp"
#include <defs.h>
#include <QPointer>
#include <QElapsedTimer>
class QCheckBox;

class IExternalScan : public QObject
{
    Q_OBJECT

public:
    IExternalScan() : displayPlot(0) {
        started = false;
        plot::ResizeOwner(Output3d, 1);
        plot::ResizeOwner(Section, 1);
        Section[0]->Pen.setColor(Qt::red);
    }
    virtual ~IExternalScan() {}
    void SetDirStPoint(int _dir, int _stpoint) { dir = _dir; start_point = _stpoint; }
    void SetStPointDirection(int value) {
        switch(value) {
        case spTopLeftAxisX: SetDirStPoint(Horiz, TopLeft); break;
        case spTopRightAxisX: SetDirStPoint(Horiz, TopRight); break;
        case spBottomRightAxisX: SetDirStPoint(Horiz, BottomRight); break;
        case spBottomLeftAxisX: SetDirStPoint(Horiz, BottomLeft); break;
        case spTopLeftAxisY: SetDirStPoint(Vert, TopLeft); break;
        case spTopRightAxisY: SetDirStPoint(Vert, TopRight); break;
        case spBottomRightAxisY: SetDirStPoint(Vert, BottomRight); break;
        case spBottomLeftAxisY: SetDirStPoint(Vert, BottomLeft); break;
        }
    }
    enum TDirections { Vert, Horiz };
    enum TStartPoints { BottomLeft, BottomRight, TopLeft, TopRight };

    virtual bool init(int _nx, int _ny, int _dir_stpoint, Range _sx, Range _sy) {
        nx = _nx;
        ny = _ny;
        dir_stpoint = _dir_stpoint;
        SetStPointDirection(dir_stpoint);
        sx = _sx;
        sy = _sy;
        started = true;

        clear();

        Output3d.at(0)->XLimits = sx;
        Output3d.at(0)->YLimits = sy;
        Output3d.at(0)->setSize(nx, ny);

        foreach (QWidget *wid, disableWidgets)
            wid->setEnabled(false);

        return true;
    }
    virtual void finish() {
        started = false;
        foreach (QWidget *wid, disableWidgets)
            wid->setEnabled(true);
    }
    virtual void save() { }
    virtual void clear() {
        lastError.clear();
        Output3d.at(0)->clear();
        Section[0]->clear();
    }
    virtual int getDataChannel() { return srcCCD; }
    virtual void updateData() { emit updated(); }

    virtual QWidget *createInterfaceWidget() { return 0; }

    virtual bool acquireNextPoint() = 0;
signals:
    void acquired();
    void updated();
    void setSectionEnabled(bool);

public:
    int nx, ny, dir, start_point, dir_stpoint;
    Range sx, sy;
    bool started;
    plot::T3dPlot *displayPlot;

    //data interface
    QString lastError;
    plot::TCurvesDataOwner Section;
    plot::T3dDataOwner Output3d;
    plot::TMetaStorage attributes;
    QVector<QWidget *> disableWidgets;
};

class SPMTunning;
IExternalScan *getNewJMIExtScan(SPMTunning *);

class QwtThermo;
class StatusBar;
class TTabWidget;
class ScanPlot: public QWidget
{
    Q_OBJECT

signals:
    void sig_ScanCorrection(int);
    void sig_XYZPosition(double x, double y, double z);
    void sig_XYZPosition2(double x, double y, double z);

    void scan_speed(double);
    void sig_sendScanParam(unsigned short int NX, unsigned short int NY, double Velocity,
                           double stepX, double stepY, double stPointX, double stPointY,
                           unsigned char scanDirStPoint, unsigned char sigScanA,
                           unsigned char sigScanB, unsigned char sigScanC, unsigned char sigScanD,
                           bool LmEn,bool LmEnA,bool LmEnB,bool LmEnC,bool LmEnD,double LmHeight);
    void ccdScanParam(int, int, int, int, int, double, double, double, double, bool);

    void scan_start();
    void scan_stop();
    void sig_ccd_scan_start();
    void sig_ccd_scan_stop();
    void sig_CCDready();

    void AddDataToContainer(plot::T3dDataOwner, plot::TMetaStorage);
    void AddDataToContainer(plot::TCurvesDataOwner, plot::TMetaStorage);

    //litho
    void SendBackground(const QImage &Img);
    void SendRect(const QRectF &Rect);

public:
    Q_SIGNAL void sync_scan_x(double);
    Q_SIGNAL void sync_scan_y(double);
    Q_SIGNAL void sync_scan_x2(double);
    Q_SIGNAL void sync_scan_y2(double);
    void sendXYsync(double x, double y, int sc);

public slots:
    void reset();
    void capBits(int, long);
    void clearScanDataBuff();
    void sl_fast_scan_correction(int ch);
    void sl_ScanCorrection();
    void sl_motor_level();

    void Send2dPlotData();
    void Send3dPlotData1();
    void Send3dPlotData2();
    void Send3dPlotData3();
    void Send3dPlotData4();
    void BSendToGraphEditor_clicked();

    void AddDataToContainer_slot(plot::TCurvesDataOwner, plot::TMetaStorage);
    void AddDataToContainer_slot(plot::T3dDataOwner, plot::TMetaStorage);
    void FillDataAttributes();
    void FillDataAttributes2();
    void slAutosaveData();

    void sl_setCurrentX(double v);
    void sl_setCurrentY(double v);
    void sl_setCurrentX2(double v);
    void sl_setCurrentY2(double v);
    void sl_syncXY();
    void sl_Update_XYZ_pos();
    void sendXYZPos();
    void onLEdDeltaX_editingFinished();
    void onLEdDeltaY_editingFinished();
    void sl_set_XYZ_range(double, double, double);
    void sl_set_XYZ_range2(double, double, double);
    void sl_set_XYZ_range_sync();
    void Zpos_dis_en(bool);
    void setZFB_status(int value);
    void slUpdZscanner(double zpos);

    void check_nx_ny();
    void check_speed();
    void onBApply_clicked();

    void scan_start_stop(bool);
    void s_stopScan();
    void sl_stop_scan();
    void sl_update_progress(QByteArray data);
    void sl_scan_refresh_ch(int n);

    void acquireNextPoint();
    void sl_ext_scan_acquired();
    void sl_ext_scan_update();
    void sl_ext_scan_refresh();
    void sl_ext_mode();
    void sl_dacscan();
    void sl_dacscan_advance();
    bool is_dacscan();

    void setScanner(int sc);
    void LmOnOff(bool value);
    void sl_CB_update();
    void sl_CB_filters_update();

    void sl_section_legend(int);
    void sl_set_section_enabled(bool);
    void SetSectionUnits();

public:
    ScanPlot(QWidget *parent = 0);
    ~ScanPlot();
    void GUICreate();
    void closeEvent(QCloseEvent *event);
    bool eventFilter(QObject *Object, QEvent *Event);

    QVector<QWidget *> scanControlWidgets; //disabled on scan
    QPointer<StatusBar> scan_bar;
    QPointer<StatusBar> ccd_scan_bar;
    bool scanFailed;

    bool Autosend;
    QTimer *AutosaveTimer;
    QTimer *RefreshTimer4d;
    QElapsedTimer refresh_3d_limit_timer[4];
    qint64 refresh_limit_last_elapsed[4];
    bool refresh_limit_need_math_update[4];
    int channelsActive;
    unsigned int Ch_activated;

    QGroupBox *GrBDACscan;
    QComboBox *CBDACscanCh;
    Edit *EDACcount;
    Edit *EDACstep;
    double DACinitialValue;
    int DACcurrentNum;


    QComboBox *CBChannel[4];
    QCheckBox *CBSec[4];
    bool sectionEnabled;

    bool l2k_controller;
    bool is_optical_scanner[3];

    double requested_scan_speed;
    QLabel *LScanRateWarn;
    DoubleChanger *DCScanRate;
    DoubleChanger *DCZPos;

    Edit *LEdX;
    Edit *LEdY;
    Edit *LEdDeltaX;
    Edit *LEdDeltaY;
    double x, y, x2, y2;
    double Xmax, Ymax, Zmax;
    double Xmax2, Ymax2, Zmax2;
    int ZFB_status;
    bool FB_status;
    QwtThermo *Zposition;
    QLabel *ZposValue;
    QWidget *GrBZPos;

    double xLeft, xRight, yTop, yBottom;
    unsigned short int Nx_points, Ny_points;
    int line_number[4];
    QwtPlotMarker *scanLineInd[4];
    bool ready[4];
    int nlcupd[4];
    bool scan_direction;
    unsigned char scanDirStPoint;
    double stPointX, stPointY;

    TTabWidget *tabgroup1;
    QWidget *BoxExternal;
    QComboBox *CBExternalMode;
    TwoStateButton *BCCDXYLock;
    QComboBox *CBPointsOYext;
    QComboBox *CBPointsOXext;
    QVector<IExternalScan *> externalScans;
    QVector<QWidget *> externalScanWidgets;
    IExternalScan *currExtScan;
    void addExternalScan(QString name, IExternalScan *iface);

    int fast_scan_correction;
    bool fast_scan_CLZ_disabled;


    DoubleChanger *DCfbP;
    DoubleChanger *DCfbI;
    DoubleChanger *DCfbSetPoint;

    DoubleChanger *spmPrecision;

    DoubleChanger *firstDelay;
    DoubleChanger *lineDelay;
    DoubleChanger *pointDelay;

    QCheckBox *fastReturn;
    QComboBox *pointSync;

    QComboBox *CBScanner;
    QComboBox *CBPointsOY;
    QComboBox *CBPointsOX;
    void Set_CBPoints(QComboBox *box);

    QComboBox *CBStartPos;


    plot::T3dPlot *Plot;

    void Fill_Data_Array(plot::T2dArray& Array, EG_ScanData *Scan_Data, EG_ScanData *lsb_Scan_Data = 0, plot::TCurveData *Section = 0, bool raw = false);
    QByteArray lsbData;

    plot::TMetaStorage scan_attributes;
    plot::T3dDataOwner *dataScan;
    plot::T3dDataOwner *dataScan_src;
    plot::T2dArray *dataScan_X;
    plot::T2dArray *dataScan_Xb;
    plot::T2dArray *dataScan_Y;
    plot::T2dArray *dataScan_Yb;

    plot::T3dDataOwner scanData[2][4];
    plot::TCurvesDataOwner Section_data;


    QScrollArea *LeftScroll;

    QLabel *lb_ccd_scan;
    QLabel *lb_ccd_scan_sm;

    TwoStateButton *LiftMode;
    QCheckBox *LmSwitch[4];
    QLabel *LblLiftMode;
    Edit *LEd_LFmodeHeight;

    TwoStateButton *PBToNewWindow;
    TwoStateButton *m_BStart;
    QToolButton *FastScanCorrection;
    TwoStateButton *BPreciseScan;
    TwoStateButton *BContinuousScan;
    TwoStateButton *BXYLock;
    QPushButton *BMotorLevel;
    QPushButton *HVxyzToZero;

};

#endif //SCANPLOT_H
