#include "scanplot.h"
#include "defs.h"
#include "EG3000.h"
#include "container/datalist.hpp"
#include "mainwindow.h"
#include "spmtunning.h"
#include "spm_motors.hpp"
#include "motors.hpp"
#include "nstslider.h"

#include "3d/plot_3dplot.h"
#include "math/plot_SubtPlane.h"
#include "math/plot_MathProcessing.h"
#include <qwt_plot_marker.h>
#include <functional>
#include <numeric>
#include <algorithm>
#include <limits>
#include <QApplication>
#include <QSplitter>
#include <QTimer>
#include <QMessageBox>
#include <QSettings>
#include <QCheckBox>

extern USB_WRITE_THREAD *_wThread; //from modules.cpp


bool ScanPlot::eventFilter(QObject *Object, QEvent *Event)
{
    if (currExtScan && currExtScan->eventFilter(Object, Event))
        return true;

    QPointF top_lef_point;
    QPointF bot_right_point;
    if (Event->type() == plot::TSelectEvent::id)
    {
        static Range x, y;
        if (((plot::TSelectEvent *)Event)->State == plot::TSelectEvent::St_SelectionStarted)
        {
            x = Plot->ThreeDPlots.at(Plot->ActivePlot)->PlotsXLimits;
            y = Plot->ThreeDPlots.at(Plot->ActivePlot)->PlotsYLimits;

            Plot->on_BRestoreScale_clicked();

            double w, h;
            Plot->ThreeDPlots.at(Plot->ActivePlot)->FindSizeWithAspectRatio(x[1] - x[0], y[1] - y[0], &w, &h);
            double s = qMax((x[1] - x[0]) / w, (y[1] - y[0]) / h);
            w = (w * s - (x[1] - x[0])) / 2; h = (h * s - (y[1] - y[0])) / 2;

            double w2 = w, h2 = h;
            s = qMax(0., -(x[0] - w));
            w -= s; w2 += s;
            s = qMax(0., -(y[0] - h));
            h -= s; h2 += s;
            s = qMax(0., (x[1] + w2) - LEdX->maximum()*1e-6);
            w += s; w2 -= s;
            s = qMax(0., (y[1] + h2) - LEdY->maximum()*1e-6);
            h += s; h2 -= s;

            for (int i = 0; i < 4; i++) {
                Plot->ThreeDPlots[i]->SetCurrentXScales(x[0] - w, x[1] + w2);
                Plot->ThreeDPlots[i]->SetCurrentYScales(y[0] - h, y[1] + h2);
                Plot->ThreeDPlots[i]->ResizeQPlot();
                Plot->ThreeDPlots[i]->PlotWasChanged();
            }

            Plot->ThreeDPlots[Plot->ActivePlot]->UpdateCanvasRect();
            Plot->ThreeDPlots[Plot->ActivePlot]->Updatekxky();
            Plot->ThreeDPlots[Plot->ActivePlot]->ReInitSelectRectGeometry(2.);
            Plot->ThreeDPlots[Plot->ActivePlot]->SetSelectWidth((x[1] - x[0]) / 2.);
            Plot->ThreeDPlots[Plot->ActivePlot]->SetSelectHeight((y[1] - y[0]) / 2.);

            qApp->sendPostedEvents(Plot, plot::TScalesChangedEvent::id);
            qApp->sendPostedEvents(this, plot::TScalesChangedEvent::id);

            LEdDeltaX->setEnabled(true);
            LEdDeltaY->setEnabled(true);
            m_BStart->setEnabled(false);
        }
        else if (((plot::TSelectEvent *)Event)->State == plot::TSelectEvent::St_SelectionCancled)
        {
            LEdDeltaX->setEnabled(false);
            LEdDeltaY->setEnabled(false);
            m_BStart->setEnabled(true);

            Plot->SetPlotsXScales(x[0], x[1]);
            Plot->SetPlotsYScales(y[0], y[1]);
            for (int i = 0; i < 4; i++)
                Plot->ThreeDPlots[i]->ResizeQPlot();
        }
        if (LEdDeltaX->isEnabled() && LEdDeltaY->isEnabled())
        {
            LEdDeltaX->setValue(1e6 * ((plot::TSelectEvent *) Event)->SelectedRect.width());
            LEdDeltaY->setValue(1e6 * ((plot::TSelectEvent *) Event)->SelectedRect.height());
            if (BXYLock->on_off && (Plot->ThreeDPlots.at(Plot->ActivePlot)->SelectAction == plot::TPlotSelectionData::MoveLeft ||
                                    Plot->ThreeDPlots.at(Plot->ActivePlot)->SelectAction == plot::TPlotSelectionData::MoveRight)) {
                LEdDeltaY->setValue(LEdDeltaX->value());
                Plot->SetSelectHeight(LEdDeltaY->value() / 1e6);
            }
            if (BXYLock->on_off && (Plot->ThreeDPlots.at(Plot->ActivePlot)->SelectAction == plot::TPlotSelectionData::MoveTop ||
                                    Plot->ThreeDPlots.at(Plot->ActivePlot)->SelectAction == plot::TPlotSelectionData::MoveBottom)) {
                LEdDeltaX->setValue(LEdDeltaY->value());
                Plot->SetSelectWidth(LEdDeltaX->value() / 1e6);
            }
        }
    }
    else if (Event->type() == plot::TCrossEvent::id)
    {
        LEdX->setValue(1e6 * ((plot::TCrossEvent *) Event)->Coord.x() );
        LEdY->setValue(1e6 * ((plot::TCrossEvent *) Event)->Coord.y() );
        sendXYZPos();
    }
    else if (Event->type() == plot::TScalesChangedEvent::id)
    {
        top_lef_point   = QPointF(1e6 * ((plot::TScalesChangedEvent *) Event)->XLimits[0], 1e6 *((plot::TScalesChangedEvent *) Event)->YLimits[0]);
        bot_right_point = QPointF(1e6 * ((plot::TScalesChangedEvent *) Event)->XLimits[1], 1e6 * ((plot::TScalesChangedEvent *) Event)->YLimits[1]);

        xLeft   = top_lef_point.x();
        xRight  = bot_right_point.x();
        yTop    = bot_right_point.y();
        yBottom = top_lef_point.y();

        LEdDeltaX->setValue(1e6 * (((plot::TScalesChangedEvent *) Event)->XLimits[1] - ((plot::TScalesChangedEvent *) Event)->XLimits[0]));
        LEdDeltaY->setValue(1e6 * (((plot::TScalesChangedEvent *) Event)->YLimits[1] - ((plot::TScalesChangedEvent *) Event)->YLimits[0]));

        if (((plot::TScalesChangedEvent *)Event)->Reason == plot::TScalesChangedEvent::R_BApply_clicked ||
                ((plot::TScalesChangedEvent *)Event)->Reason == plot::TScalesChangedEvent::R_BRestoreScale_clicked)
        {
            for (int i = 0; i < 4; i++)
                Plot->ThreeDPlots[i]->ResizeQPlot();

            LEdDeltaX->setEnabled(false);
            LEdDeltaY->setEnabled(false);
            m_BStart->setEnabled(true);
        }
        check_speed();
    }
    if (Event->type() == QEvent::Close) {
        closeEvent((QCloseEvent *)Event);
        return true;
    }
    return QWidget::eventFilter(Object, Event);
}


//----------------------------------------------------------

int ch_frac(bool on) { return on ? 1 : 0; }
void ScanPlot::scan_start_stop(bool value)
{
    for (int i = 0; i < 4; i++) {
        dataScan->clear();
        plot::ResizeOwner(*dataScan, 4);
        dataScan->at(i)->clear();
        dataScan_src->at(i)->clear();
    }

    dataScan_X->clear();
    dataScan_Y->clear();
    dataScan_Xb->clear();
    dataScan_Yb->clear();

    lsbData.clear();

    int idx[4];
    for (int i = 0; i < 4; i++)
        idx[i] = CBChannel[i]->itemData(CBChannel[i]->currentIndex()).toInt();

    channelsActive = qMax<int>(1, 4 - std::count(idx, idx+4, srcNone));
    if (value && std::count(idx, idx+4, srcNone) != 4)
    {
        scanFailed = false;

        Plot->Split3d->setSizes(QList<int>() << ch_frac(idx[0] != srcNone || idx[1] != srcNone) << ch_frac(idx[2] != srcNone || idx[3] != srcNone));
        Plot->Containers.at(0)->setSizes(QList<int>() <<
                                         ch_frac(idx[0] != srcNone || (idx[1] != srcNone && (idx[2] != srcNone || idx[3] != srcNone))) <<
                                         ch_frac(idx[1] != srcNone || (idx[0] != srcNone && (idx[2] != srcNone || idx[3] != srcNone))));
        Plot->Containers.at(1)->setSizes(QList<int>() <<
                                         ch_frac(idx[2] != srcNone || (idx[3] != srcNone && (idx[0] != srcNone || idx[1] != srcNone))) <<
                                         ch_frac(idx[3] != srcNone || (idx[2] != srcNone && (idx[0] != srcNone || idx[1] != srcNone))));

        if (idx[Ch_activated] == srcNone)
            for (int i = 0; i < 4; i++)
                if (idx[i] != srcNone) {
                    Plot->SetActivePlot(i);
                    break;
                }

        FillDataAttributes();

        SetSectionUnits();

        onBApply_clicked();

        sendXYsync(stPointX, stPointY, CBScanner->currentIndex());

        for (int i = 0; i < 4; i++) {
            if (!BContinuousScan->on_off) { //continuous scan
                Plot->ThreeDPlots[i]->MinSliderVal = 0;
                Plot->ThreeDPlots[i]->MaxSliderVal = 1;
                Plot->ThreeDPlots[i]->AutoZScaleOn = true;
                Plot->ThreeDPlots[i]->ReSetAxisScales();
            }

            scanLineInd[i]->setXValue(stPointX);
            scanLineInd[i]->setYValue(stPointY);
            scanLineInd[i]->setLineStyle(scan_direction ? QwtPlotMarker::VLine : QwtPlotMarker::HLine);
        }
        if (!BContinuousScan->on_off) { //continuous scan
            Plot->Slider->Sliders->SetMinMaxRelSlidersValues(0, 1);
            Plot->Slider->Sliders->setAutoMode(true);
            Plot->Refresh();
        }

        try {
            int sc[4];
            for (int i = 0; i < 4; i++) {
                sc[i] = (idx[i] != srcNone) && (idx[i] != srcCCD);
                if (sc[i] || fast_scan_correction) {
                    dataScan->at(i)->XLimits = Plot->ThreeDPlots.at(0)->PlotsXLimits;
                    dataScan->at(i)->YLimits = Plot->ThreeDPlots.at(0)->PlotsYLimits;
                    dataScan->at(i)->setSize(Nx_points, Ny_points);
                    dataScan->at(i)->LimsForMathX[1] = dataScan->at(i)->LimsForMathY[1] = 0;
                }
                if (idx[i] != srcNone || fast_scan_correction)
                    scanData[CBScanner->currentIndex()][i].prepend(dataScan->at(i));
            }

            if (fast_scan_correction && (sc[0] || sc[1])) {
                *dataScan_X = dataScan->at(0)->Array;
                *dataScan_Y = dataScan->at(1)->Array;
            }
            if (fast_scan_correction && (sc[2] || sc[3])) {
                *dataScan_Xb = dataScan->at(2)->Array;
                *dataScan_Yb = dataScan->at(3)->Array;
            }

            if (fast_scan_correction) {
                if (dataScan_src == dataScan) {
                    dataScan_src = new plot::T3dDataOwner;
                    plot::ResizeOwner(*dataScan_src, 4);
                }
                for (int i = 0; i < 4; i++)
                    *dataScan_src->at(i) = *dataScan->at(i);
            } else if (dataScan_src != dataScan) {
                delete dataScan_src;
                dataScan_src = dataScan;
            }
        } catch (std::bad_alloc) {
            for (int i = 0; i < 4; i++) {
                dataScan->at(i)->clear();
                dataScan_src->at(i)->clear();
            }
            dataScan_X->clear();
            dataScan_Y->clear();
            dataScan_Xb->clear();
            dataScan_Yb->clear();

            new Notification(tr("Scan") + ": " + tr("Out of memory"), Notification::Bad, 60);
            doBeep();

            scanFailed = true;
            m_BStart->change_state_man_with_sig(false);
            return;
        }

        Plot->PlotControl->BCrossection->setChecked(false);
        Plot->PlotControl->BMarkers->setChecked(false);

        if (sectionEnabled) {
            Plot->TwoDPlot->Pure2dPlot->SourceData.clear();
            Plot->TwoDPlot->Pure2dPlot->SourceData.resize(4);
        }

        foreach (QWidget *wid, scanControlWidgets)
            wid->setEnabled(false); // !m_BStart->on_off

        GrBZPos->setEnabled((!FB_status || (ZFB_status != CBScanner->currentIndex())) && !m_BStart->on_off);

        for (int i = 0; i < 4; i++) {
            refresh_3d_limit_timer[i].invalidate();
            line_number[i] = -1;
            if (idx[i] != srcNone) {
                ready[i] = false;
                nlcupd[i] = 0;
            }
        }

        if (!scan_bar) {
            scan_bar = new StatusBar(tr("Scan"), scan_direction ? Nx_points : Ny_points);
            connect(scan_bar, SIGNAL(canceled()), this, SLOT(s_stopScan()));
            scan_bar->add();
        }
        scan_bar->setValue(0);
        scan_bar->cancel->setEnabled(m_BStart->isEnabled());

        if (CBExternalMode->currentIndex() != 0) {
            bool ok = currExtScan->init(CBPointsOXext->currentText().toInt(), CBPointsOYext->currentText().toInt(), scanDirStPoint,
                                        Plot->ThreeDPlots.at(0)->PlotsXLimits, Plot->ThreeDPlots.at(0)->PlotsYLimits);

            if (ok) { //check for canceled in fast scan preparation
                if (!ccd_scan_bar) {
                    ccd_scan_bar = new StatusBar(tr("External scan"), CBPointsOXext->currentText().toInt()*CBPointsOYext->currentText().toInt());
                    connect(ccd_scan_bar, SIGNAL(canceled()), this, SLOT(s_stopScan()));
                    ccd_scan_bar->add();
                }
                ccd_scan_bar->setValue(0);

                emit sig_ccd_scan_start();
            } else {
                //TODO silent if empty?
                new Notification(tr("External scan") + ": " + tr("Scan init error:") + " " + currExtScan->lastError, Notification::Bad, 10);
                doBeep();

                destructStatus(ccd_scan_bar);
                destructStatus(scan_bar);

                scanFailed = true;
                m_BStart->change_state_man_with_sig(false);
            }
        } else
            emit scan_start();
    }
    else
    {
        //TODO crop new data on cancel, remove crop from plot Save()
        for (int i = 0; i < 4; i++) {
            int j = plot::FindDataWithPlotsLimits(scanData[CBScanner->currentIndex()][i].mid(1),
                    Plot->ThreeDPlots.at(0)->PlotsXLimits, Plot->ThreeDPlots.at(0)->PlotsYLimits);
            if (j != -1) {
                scanData[CBScanner->currentIndex()][i].remove(j+1);
                Plot->ThreeDPlots[i]->SourceData = plot::getUnLock(scanData[CBScanner->currentIndex()][i]);
            }
        }
        Plot->Refresh();

        if (std::count(idx, idx+4, srcNone) == 4) {
            m_BStart->change_state_man(false);
            MobileServer::sendData(DataID::ScanOnOff, false);
        } else
        {
            if (CBExternalMode->currentIndex() != 0) {
                if (!scanFailed)
                    emit sig_ccd_scan_stop();

                currExtScan->finish();
            } else if (!scanFailed)
                emit scan_stop();

            for (int i = 0; i < 4; i++)
                Plot->ThreeDPlots[i]->ReSetAxisScales();
            Plot->Refresh();

            destructStatus(ccd_scan_bar);
            destructStatus(scan_bar);

            foreach (QWidget *wid, scanControlWidgets)
                wid->setEnabled(true); // !m_BStart->on_off

            GrBZPos->setEnabled((!FB_status || (ZFB_status != CBScanner->currentIndex())) && !m_BStart->on_off);

            sendXYsync((xLeft + xRight) / 2, (yTop + yBottom) / 2, CBScanner->currentIndex());

            if (DACcurrentNum > 0) {
                DACcurrentNum = EDACcount->value()-1;//properly reset dacscan
                sl_dacscan_advance();
            }
            if (GrBDACscan->isChecked())
                GrBDACscan->setChecked(false); //do not want autosave triggering advance

            FillDataAttributes2();
            if (Autosend && !scanFailed)
                AutosaveTimer->start(100);
        }

        DACcurrentNum = 0;
        for (int i = 0; i < 4; i++) {
            ready[i] = true;
            nlcupd[i] = 2;
        }
    }
    scanFailed = false;
}
void ScanPlot::s_stopScan()
{
    if (m_BStart->on_off)
        m_BStart->change_state_man_with_sig(false);
}

void ScanPlot::sl_stop_scan()
{
    if (!std::accumulate(ready, ready+4, true, std::logical_and<bool>()))
        return;

    m_BStart->change_state_man(false);
    MobileServer::sendData(DataID::ScanOn, false);

    if (CBExternalMode->currentIndex() != 0)
        currExtScan->finish();

    if (BContinuousScan->on_off && !is_dacscan()) { //continuous scan
        int idx = CBStartPos->currentIndex();
        switch (idx) {
        case 0:
            idx = 4;
            break;
        case 1:
            idx = 3;
            break;
        case 2:
            idx = 6;
            break;
        case 3:
            idx = 1;
            break;
        case 4:
            idx = 0;
            break;
        case 5:
            idx = 7;
            break;
        case 6:
            idx = 2;
            break;
        case 7:
            idx = 5;
            break;
        }

        CBStartPos->setCurrentIndex(idx);
        m_BStart->change_state_man_with_sig(true);
        return;
    }

    if (!is_dacscan())
        for (int i = 0; i < 4; i++)
            Plot->ThreeDPlots[i]->ReSetAxisScales();
    Plot->Refresh();

    //TODO deduplicate gui enable/disable code with scan_start_stop
    destructStatus(ccd_scan_bar);
    destructStatus(scan_bar);

    foreach (QWidget *wid, scanControlWidgets)
        wid->setEnabled(true); // !m_BStart->on_off

    GrBZPos->setEnabled((!FB_status || (ZFB_status != CBScanner->currentIndex())) && !m_BStart->on_off);

    sendXYsync((xLeft + xRight) / 2, (yTop + yBottom) / 2, CBScanner->currentIndex());

    FillDataAttributes2();
    if (Autosend)
        AutosaveTimer->start(100);
    else
        sl_dacscan_advance();

    doBeep();
}
void ScanPlot::slAutosaveData()
{
    I_MODULE *datam = ModuleManager::instance->find1("data");
    if (!Autosend || !datam)
        return;

    DataPlot *data = (DataPlot *)datam->basecast();

    if (currExtScan)
        currExtScan->save();

    QMessageBox mb(QMessageBox::NoIcon, tr("Autosend to data"), "0/4 " + tr("Please wait."));
    mb.setStandardButtons(0);
    mb.show();
    mb.repaint();

    for (int i = 0; i < 4; i++) {
        int idx = CBChannel[i]->itemData(CBChannel[i]->currentIndex()).toInt();
        if (idx != srcNone) {
            TContainer *c = data->Container;

            Plot->AutoSave(i);

            while (c->CurrentDataList && c->CurrentDataList->ClickedItem &&
                   c->CurrentDataList->ClickedItem->UpdatePreview) {
                qApp->processEvents();
                SleepExport::msleep2(3);
            }
        }

        mb.setText(QString::number(i+1) + "/4 " + tr("Please wait."));
        mb.repaint();
    }

    mb.hide();
}


void ScanPlot::check_nx_ny()
{
    unsigned short NX = CBPointsOX->currentText().toUShort();
    unsigned short NY = CBPointsOY->currentText().toUShort();
    if (Nx_points == NX && Ny_points == NY)
        return;

    if (BXYLock->on_off) {
        if (sender() == CBPointsOX) {
            CBPointsOY->setCurrentIndex(CBPointsOX->currentIndex());
            NY = NX;
        }
        if (sender() == CBPointsOY) {
            CBPointsOX->setCurrentIndex(CBPointsOY->currentIndex());
            NX = NY;
        }
    }

    Nx_points = NX;
    Ny_points = NY;
    MobileServer::sendData(DataID::ScanNxNy, qSqrt(NX*NY));

    if (CBExternalMode->currentIndex() == 0 || BCCDXYLock->on_off) {
        CBPointsOXext->setCurrentIndex(CBPointsOX->currentIndex());
        CBPointsOYext->setCurrentIndex(CBPointsOY->currentIndex());
    }

    check_speed();
}
void ScanPlot::check_speed()
{
    QSettings settings("Param/controller.conf", QSettings::IniFormat);
    QList<I_MODULE *> l = ModuleManager::instance->find("oscilloscope");
    Oscilloscope *osc = l.isEmpty() ? 0 : (Oscilloscope *)l.last()->basecast();

    scan_direction = (CBStartPos->currentIndex() % 2 != 0);
    double scan_points = !scan_direction ? Nx_points : Ny_points; //double to skip conversion for division


    //hw limits
    int scan_rate = 20000;
    int jm_rate = 20000;
    int csb_rate = !osc ? 10000 : osc->CalibWidget->CBSpeedCSB->itemData(osc->CalibWidget->CBSpeedCSB->currentIndex()).toInt();
    if (settings.value("double_csb_rate", false).toBool())
        csb_rate *= 2;

    int scan_cyc_min = 0; //minimum
    int scan_cyc_perpoint = 2; //latency
    int pid_cyc_perpoint = 2; //target >1
    int pid_dv_perpoint = 150; //fw-dependent
    if (l2k_controller) {
        jm_rate = 312500;
        scan_rate = 60000;
        pid_dv_perpoint = 16;
    } else
#ifndef _NO_FTDI
    if (settings.value("use_eg4000", false).toBool())
#endif
    {
        scan_cyc_min = 1;
        scan_cyc_perpoint = 3;
        pid_dv_perpoint = 150 / 4; //150 in 18b lsb, = 37.5
    }

    double rate_limit_scan_cycles = scan_rate / scan_points / (scan_cyc_min + scan_cyc_perpoint);
    int limit_pid_cycles = qMax(pid_cyc_perpoint,
                                qCeil((65535 * 1e-6 * (!scan_direction ?
                                                           (xRight - xLeft) / (CBScanner->currentIndex() == 0 ? Xmax : Xmax2) / Nx_points :
                                                           (yTop - yBottom) / (CBScanner->currentIndex() == 0 ? Ymax : Ymax2) / Ny_points)) / pid_dv_perpoint));
    double rate_limit_pid_dv = csb_rate / scan_points / limit_pid_cycles;

    double limit = rate_limit_scan_cycles;
    bool cl_on = !osc ? false : (CBScanner->currentIndex() == 0 ? osc->CalibWidget->CL_status_xy : osc->CalibWidget->CL2_status_xy);
    if (cl_on)// && !fast_scan_correction) //not interested in bad scan
        limit = qMin(limit, rate_limit_pid_dv);

    double max_scan_rate = settings_double(settings, "limit_scan_rate", 10);
    DCScanRate->setMaximum(qMin(limit, max_scan_rate));


    //closest available rates
    double srate = qMax(0.01, DCScanRate->value());
    int rate = cl_on ? csb_rate : scan_rate;
    double raw_cycles = rate / scan_points / srate;
    int cycles = qMax(cl_on ? limit_pid_cycles : scan_cyc_min + scan_cyc_perpoint, qRound(raw_cycles));
    double real_rate = rate / scan_points / cycles;
    requested_scan_speed = rate / scan_points / (0.9 + cycles - (cl_on ? 0 : scan_cyc_perpoint));
    requested_scan_speed = qMax(0.01, requested_scan_speed);


    //jm
    I_MODULE *spmm = ModuleManager::instance->find1("spm");
    SPMTunning *spm = !spmm ? 0 : (SPMTunning *)spmm->basecast();
    double max_jm_rate = 10000;
    if (spm && spm->BFeedbackOnOffMode->on_off)
        max_jm_rate = jm_rate / 2 / (spm->DCfbHeight->value() / (1e-3 * qMax(spm->DCfbPrec->value(), l2k_controller ? 0.01 : 0.3)) )
                / scan_points;


    bool jm_limited = real_rate / max_jm_rate > 1.07;
    bool mv_limited = srate / real_rate > 1.07 || srate / real_rate < 0.93;

    Qt::GlobalColor clrs[3] = {Qt::darkGray, Qt::green, Qt::darkGray};
    DCScanRate->slider->marks.clear();
    if (mv_limited && LScanRateWarn->isVisibleTo(LScanRateWarn->parentWidget()))
        for (int i = -1; i <= 1; i++)
            DCScanRate->slider->marks.append(QPair<double, QColor>(
                                                 rate / scan_points / (cycles + i)
                                                 , clrs[i+1]));

    double mins = (!scan_direction ? Ny_points : Nx_points) / qMin(real_rate, max_jm_rate) / 60;

    LScanRateWarn->setEnabled(mv_limited || jm_limited);
    LScanRateWarn->setToolTip(tr("Effective scan rate: ") + QString::number(qMin(real_rate, max_jm_rate)) +
                              (jm_limited ? "\n" + tr("JM speed limitation") : QString()) +
                              (mv_limited ? (jm_limited ? "\n" + tr("Movement rate: ") + QString::number(real_rate) : QString())
                                            + tr(" - green") + "\n" + tr("next/prev available - grey") : QString()) +
                              (mins > 5 ? "\n" + tr("Scan time: ") + QString::number(mins, 'f', 1) + tr("min") : QString()));

    if (sender() == DCScanRate)
        emit scan_speed(requested_scan_speed);
}

void ScanPlot::onBApply_clicked()
{
    int idx[4];
    for (int i = 0; i < 4; i++)
        idx[i] = CBChannel[i]->itemData(CBChannel[i]->currentIndex()).toInt();

    unsigned short int NX = CBPointsOX->currentText().toUShort();
    unsigned short int NY = CBPointsOY->currentText().toUShort();
    Nx_points = NX;
    Ny_points = NY;

    scanDirStPoint = CBStartPos->currentIndex();
    scan_direction = (CBStartPos->currentIndex() % 2 != 0);
    switch(CBStartPos->currentIndex()) {
    case 0:
    case 1:
        stPointX=xLeft;
        stPointY=yTop;
        break;
    case 2:
    case 3:
        stPointX=xRight;
        stPointY=yTop;
        break;
    case 4:
    case 5:
        stPointX=xLeft;
        stPointY=yBottom;
        break;
    case 6:
    case 7:
        stPointX=xRight;
        stPointY=yBottom;
        break;
    }

    double stepX = fabs((xRight - xLeft) / ((float)NX - 1));
    double stepY = fabs((yTop - yBottom) / ((float)NY - 1));

    emit sig_sendScanParam(NX, NY, requested_scan_speed, stepX, stepY, stPointX, stPointY, CBStartPos->currentIndex(),
                           idx[0], idx[1], idx[2], idx[3],
                           LiftMode->on_off, LmSwitch[0]->isChecked(), LmSwitch[1]->isChecked(),
                           LmSwitch[2]->isChecked(), LmSwitch[3]->isChecked(), (LEd_LFmodeHeight->value())/1000.);

    if (CBExternalMode->currentIndex() != 0)
        emit ccdScanParam(CBScanner->currentIndex(), (BCCDXYLock->on_off ? 1 : 2),
                          CBPointsOXext->currentText().toInt(), CBPointsOYext->currentText().toInt(),
                          CBStartPos->currentIndex(), stPointX, stPointY, stepX, stepY, false/*BFastScan->on_off*/);
}

//----------------------------------------------------------

void ScanPlot::Fill_Data_Array(plot::T2dArray& Array, EG_ScanData *Scan_Data, EG_ScanData *lsb_Scan_Data, plot::TCurveData *Section, bool raw)
{
    bool backward_line = false; //dirFW
    switch(scanDirStPoint) {
    case spTopLeftAxisX:
    case spTopRightAxisX:
    case spTopRightAxisY:
    case spBottomRightAxisY:
        backward_line = true; //dirBW
        break;
    case spBottomRightAxisX:
    case spBottomLeftAxisX:
    case spTopLeftAxisY:
    case spBottomLeftAxisY:
        break;
    }

    bool directionY = (Scan_Data->Axis == AxisY);

    int ny = Array.size();
    int nx = ny > 0 ? Array[ny-1].size() : 0;
    int size = Scan_Data->ScanSize;
    size = directionY ? qMin(size, nx) : qMin(size, ny);
    int size2 = Scan_Data->Size;
    size2 = directionY ? qMin(size2, ny) : qMin(size2, nx);

    int line = Scan_Data->Line;
    line = qMin(line, (size-1));
    if (backward_line) line = (size - 1) - line;

    if (Section) {
        double stepX = fabs((xRight - xLeft) / ((float)Nx_points - 1));
        double stepY = fabs((yTop - yBottom) / ((float)Ny_points - 1));

        double start = directionY ? yBottom : xLeft;
        double step = directionY ? stepY : stepX;

        Section->setXCalib(start * 1e-6, step * 1e-6);
        Section->YSeq.resize(size2);
    }

    for (int i = 0; i < size2; i++) {
        double val = Scan_Data->Data[i] + (lsb_Scan_Data ? ((double)lsb_Scan_Data->Data[i]) / 0x10000 : 0);
        if (!raw) //no point optimizing input_calib - max 2x speedup
            val = input_calib(Scan_Data->Source, val, (ZFB_status ? Zmax2 : Zmax));

        if (directionY)
            Array[i][line] = val;
        else
            Array[line][i] = val;

        if (Section)
            Section->YSeq[i] = val;
    }
}
void ScanPlot::sl_update_progress(QByteArray data)
{
    if (!m_BStart->on_off)
        return;

    EG_ScanData *sdata = (EG_ScanData *)data.constData();
    bool dir = sdata->Direction == dirBW;
    int ch = sdata->Channel + (dir ? 2 : 0);

    if (sdata->Scanner & 0x80) {
        lsbData = data;
        return;
    }

    EG_ScanData *sdata2 = (EG_ScanData *)lsbData.constData();
    if (lsbData.isEmpty() || sdata2->Channel % 4 != sdata->Channel)
        sdata2 = 0;

    bool refresh = false;
    if (fast_scan_correction) {
        if (sdata->Source == (CBScanner->currentIndex() ? srcCX2 : srcCX)) {
            Fill_Data_Array(dir ? *dataScan_Xb : *dataScan_X, sdata, sdata2, 0, true);
            refresh = true;
        }
        if (sdata->Source == (CBScanner->currentIndex() ? srcCY2 : srcCY)) {
            Fill_Data_Array(dir ? *dataScan_Yb : *dataScan_Y, sdata, sdata2, 0, true);
            refresh = true;
        }

        if (refresh) {
            refresh = false;
            int chh = 0 + (dir ? 2 : 0);
            for (int i = 0; i < 2; i++)
                if (CBChannel[chh+i]->currentIndex() != 0)
                    if ((++nlcupd[chh+i]) >= 2 && //inc
                            (!_wThread->testMode && sdata->Channel >= 2)) { //this data only for xy buffer, need refresh
                        if (ready[chh+i])
                            refresh_3d_limit_timer[chh+i].invalidate();
                        refresh = true;
                        sl_scan_refresh_ch(chh+i);
                    }
            if (refresh)
                //if (std::accumulate(ready, ready+4, true, std::logical_and<bool>()))
                    sl_stop_scan();
        }
        refresh = false;
    }

    if ((!_wThread->testMode && sdata->Channel >= 2) || ch >= 4) {
        lsbData.clear();
        return;
    }

    line_number[ch]++;
    if (scan_bar != 0)
        scan_bar->setValue(line_number[ch] + 1);

    int points = Ny_points;
    if (scan_direction)
        points = Nx_points;

    if (line_number[ch] >= points-1) {
        ready[ch] = true;
        refresh_3d_limit_timer[ch].invalidate();
    }

    bool is_ext = (ch == 1 && CBExternalMode->currentIndex() != 0);
    if (!is_ext) { //discard fake data
        Fill_Data_Array(dataScan_src->at(ch)->Array, sdata, sdata2, Section_data[ch].data());
        if (!fast_scan_correction || nlcupd[ch] >= 2) {
            refresh = true;
            sl_scan_refresh_ch(ch);
        }
    }

    lsbData.clear();

    if (!fast_scan_correction || is_ext || refresh) //we need that last nlc line displayed before end
    //if (std::accumulate(ready, ready+4, true, std::logical_and<bool>()))
        sl_stop_scan();
}

//----------------------------------------------------------

void get_scan_lim(plot::TMin3dData *data, int dir, int line, int &x0, int &x1, int &y0, int &y1)
{
    x0 = y0 = x1 = y1 = 0;
    if (dir == spBottomLeftAxisX || dir == spBottomRightAxisX)
    {
        x1 = data->NXElems;
        y1 = line;
    }
    if (dir == spTopLeftAxisX || dir == spTopRightAxisX)
    {
        x1 = data->NXElems;
        y0 = data->NYElems - line;
        y1 = data->NYElems;
    }
    if (dir == spTopLeftAxisY || dir == spBottomLeftAxisY)
    {
        x1 = line;
        y1 = data->NYElems;
    }
    if (dir == spTopRightAxisY || dir == spBottomRightAxisY)
    {
        x0 = data->NXElems - line;
        x1 = data->NXElems;
        y1 = data->NYElems;
    }
}

void ScanPlot::sl_scan_refresh_ch(int n)
{
    bool is_ext = (n == 1 && CBExternalMode->currentIndex() != 0);
    if (!m_BStart->on_off && !is_ext)
        return;

    if (!is_ext && refresh_3d_limit_timer[n].isValid() && refresh_3d_limit_timer[n].elapsed() <= 100 + refresh_limit_last_elapsed[n])
        return; //~10 refreshes per second max
    refresh_3d_limit_timer[n].start();
    refresh_limit_need_math_update[n] = true;

    if (sectionEnabled && Plot->TwoDPlot->Pure2dPlot->SourceData.size() == 4) {
        if (CBSec[n]->isChecked()) {
            Plot->TwoDPlot->Pure2dPlot->SourceData[n] = (is_ext && currExtScan) ? currExtScan->Section.at(0) : Section_data.at(n);

            Plot->Set2dPlotVisible(true);
            Plot->TwoDPlot->Pure2dPlot->Refresh();
        } else if (!Plot->TwoDPlot->Pure2dPlot->SourceData[n].isNull()) {
            Plot->TwoDPlot->Pure2dPlot->SourceData[n].clear();
            Plot->TwoDPlot->Pure2dPlot->Refresh();
        }
    }

    if (fast_scan_correction) {
        nlcupd[n] = 0;
        sl_fast_scan_correction(n);
    }

    int x0, x1, y0, y1;
    if (!is_ext) {
        get_scan_lim(dataScan->at(n).data(), scanDirStPoint, line_number[n] + 1, x0, x1, y0, y1);
        dataScan->at(n)->LimsForMathX[0] = x0;
        dataScan->at(n)->LimsForMathX[1] = x1;
        dataScan->at(n)->LimsForMathY[0] = y0;
        dataScan->at(n)->LimsForMathY[1] = y1;
    } else if (currExtScan)
        scanData[CBScanner->currentIndex()][n][0] = currExtScan->Output3d.at(0);

    const plot::TMin3dData *Min3dData = scanData[CBScanner->currentIndex()][n].at(0).data();
    if (scan_direction) {
        Range r = Min3dData->LimsForMathX;
        if (is_ext) r[1]--;
        double d = Min3dData->XLimits[0] + (r[0] != 0 ? r[0] : r[1]) * Min3dData->dx;
        scanLineInd[n]->setXValue(d);
    } else {
        Range r = Min3dData->LimsForMathY;
        if (is_ext) r[1]--;
        double d = Min3dData->YLimits[0] + (r[0] != 0 ? r[0] : r[1]) * Min3dData->dy;
        scanLineInd[n]->setYValue(d);
    }

    if (ready[n] && dataScan_src != dataScan)
        dataScan_src->at(n)->clear();
    if (ready[0] && ready[1]) {
        dataScan_X->clear();
        dataScan_Y->clear();
    }
    if (ready[2] && ready[3]) {
        dataScan_Xb->clear();
        dataScan_Yb->clear();
    }

    if (ready[n]) {
        int j = plot::FindDataWithPlotsLimits(scanData[CBScanner->currentIndex()][n].mid(1),
                Plot->ThreeDPlots.at(0)->PlotsXLimits, Plot->ThreeDPlots.at(0)->PlotsYLimits);
        if (j != -1)
            scanData[CBScanner->currentIndex()][n].remove(j+1);
    }

    Plot->ThreeDPlots[n]->SourceData = plot::getUnLock(scanData[CBScanner->currentIndex()][n]);
    Plot->ThreeDPlots[n]->Refresh();
    if (Plot->ActivePlot == n)
        Plot->UpdateLayers();

    //also included in plot refreshed signal handler, to accounts for math\replot etc.
    refresh_limit_last_elapsed[n] = refresh_3d_limit_timer[n].elapsed() / 0.8 * channelsActive; //time budget, %
}


//----------------------------------------------------------

void
data_field_area_fit_plane(double *data,
                          int col, int row,
                          int width, int height,
                          int xres, int /*yres*/,
                          double *pa, double *pbx, double *pby)
{
    double sumx, sumy, sumz, sumxx, sumyy, sumxy, sumxz, sumyz;
    double alpha1, alpha2, alpha3, beta1, beta2, gamma1;
    double det;
    double a, b, c;
    double x, y, z;
    double n;
    double *datapos = 0;
    double *drow = 0;
    int i, j;

    n = 0;

    sumx = sumy = sumz = sumxx = sumyy = sumxy = sumxz = sumyz = 0;
    datapos = data + row*xres + col;
    for (i = 0; i < height; i++) {
        drow = datapos + i*xres;
        for (j = 0; j < width; j++) {
            x = j;
            y = i;
            z = drow[j];

            sumx += x;
            sumy += y;
            sumz += z;
            sumxx += x*x;
            sumyy += y*y;
            sumxy += x*y;
            sumxz += x*z;
            sumyz += y*z;
            n++;
        }
    }

    det = (n*sumxx*sumyy) + (2*sumx*sumxy*sumy) - (sumx*sumx*sumyy)
            -(sumy*sumy*sumxx) - (n*sumxy*sumxy);

    /* try to return something reasonable in case of singularity */
    if (det == 0.0)
        a = b = c = 0.0;
    else
    {
        det = 1.0/det;

        alpha1 = (n*sumyy) - (sumy*sumy);
        alpha2 = (n*sumxx) - (sumx*sumx);
        alpha3 = (sumxx*sumyy) - (sumxy*sumxy);
        beta1 = (sumx*sumy) - (n*sumxy);
        beta2 = (sumx*sumxy) - (sumxx*sumy);
        gamma1 = (sumxy*sumy) - (sumx*sumyy);

        a = det*(alpha1*sumxz + beta1*sumyz + gamma1*sumz);
        b = det*(beta1*sumxz + alpha2*sumyz + beta2*sumz);
        c = det*(gamma1*sumxz + beta2*sumyz + alpha3*sumz);
    }

    if (pbx)
        *pbx = a;
    if (pby)
        *pby = b;
    if (pa)
        *pa = c;
}
void ScanPlot::sl_motor_level()
{
    I_MODULE *spmm = ModuleManager::instance->find1("spm");
    I_MODULE *motorsm = ModuleManager::instance->find1("motors");
    if (!motorsm)
        return;

    Motors *motors = (Motors *)motorsm->basecast();
    SPMTunning *spm = 0;
    if (spmm)
        spm = (SPMTunning *)spmm->basecast();

    int idx = -1;
    bool is_inverted = true;
    for (int i = 0; i < 4; i++) {
        int idx_ = CBChannel[i]->itemData(CBChannel[i]->currentIndex()).toInt();
        if (idx_ == srcHeight) {
            idx = i;
            break;
        }
    }
    if (idx == -1)
    for (int i = 0; i < 4; i++) {
        int idx_ = CBChannel[i]->itemData(CBChannel[i]->currentIndex()).toInt();
        if (idx_ == (ZFB_status ? srcCZ2 : srcCZ) && !qFuzzyCompare(input_gain(idx_), 1)) {
            idx = i;
            is_inverted = false;
            break;
        }
    }

    if (idx == -1 || !scanData[CBScanner->currentIndex()][idx].size() || !scanData[CBScanner->currentIndex()][idx].at(0)->Array.size()) {
        new Notification(tr("Motor level") + ": " + tr("Please acquire some height data before."), Notification::Warn, 10);
        doBeep();

        return;
    }
    if (CBScanner->currentIndex() != 0) {
        new Notification(tr("Motor level") + ": " + tr("Stage axes orientation is unknown."), Notification::Warn, 10);
    }

    const plot::TMin3dData *Min3dData = scanData[CBScanner->currentIndex()][idx].at(0).data();

    QVector<double> Points;
    for (int i = 0; i < Min3dData->Array.size(); i++)
        for (int j = 0; j < Min3dData->Array.at(i).size(); j++)
             Points.push_back(Min3dData->Array.at(i).at(j));

    unsigned int MinX = 0, MaxX = 0, MinY = 0, MaxY = 0;
    plot::math::TMathProcessing::SetupMinsMaxs(MinX, MaxX, MinY, MaxY, *Min3dData);

    double a, b, c;
    data_field_area_fit_plane(&*Points.begin(), MinX, MinY, MaxX - MinX, MaxY - MinY, Min3dData->Array.at(0).size(), 0, &c, &a, &b);
    qDebug() << a << b;

    a /= Min3dData->dx; b /= Min3dData->dy; //tg(a) = dz/dx

    a *= 136e3; b *= 104.5e3; // * head dx = motors dz, mkm

    if (is_inverted) { //inverted, as height = inverted dz
        a *= -1;
        b *= -1;
    }

    double m1 = 0, m2 = 0, m3 = 0;
    //x
    if (a > 0)
        m1 += fabsl(a);
    else
        m2 += fabsl(a);
    m3 += fabsl(a) / 2; //preserve y angle

    //y
    //inverted in new head
    if ((CBScanner->currentIndex() == 0 && is_optical_scanner[CBScanner->currentIndex()]) ? b > 0 : b < 0) {
        m3 += fabsl(b);
    } else {
        m1 += fabsl(b);
        m2 += fabsl(b);
    }
    qDebug() << m1 << m2 << m3;

    double speed = spm ? spm->OptionsForMotor->up_speed : 500;
    if (m1 > 0.001)
        motors->goUp(1, speed, m1);
    if (m2 > 0.001)
        motors->goUp(2, speed, m2);
    if (m3 > 0.001)
        motors->goUp(3, speed, m3);

    if (spm && /*spm->BFeedbackOnOff->on_off &&*/ m1+m2+m3 > 0.001)
        new Notification(tr("You need to perform landing again after motors stop."), Notification::Warn, 30);
}


//----------------------------------------------------------

void ScanPlot::sl_ScanCorrection()
{
    int st = FindCheckedItemIn(FastScanCorrection->menu()->actions()[0]->actionGroup());
    if (st < 0) st = 0;
    FastScanCorrection->setIcon(makeHugeIcon(st > 0 ? ":/ScanPlot/fscanOn.png" : ":/ScanPlot/fscanOf.png"));
    fast_scan_correction = st;
    emit sig_ScanCorrection(st);

    QList<I_MODULE *> l = ModuleManager::instance->find("oscilloscope");
    Oscilloscope *osc = l.isEmpty() ? 0 : (Oscilloscope *)l.last()->basecast();

    if (st > 0) {
       osc->CalibWidget->Scanner_type->setCurrentIndex(CBScanner->currentIndex());
        if (osc->CalibWidget->fb_on_off_xy->on_off) {
            osc->CalibWidget->fb_on_off_xy->change_state_man_with_sig(false);
            fast_scan_CLZ_disabled = true;
        }
    } else if (fast_scan_CLZ_disabled) {
        fast_scan_CLZ_disabled = false;
        osc->CalibWidget->Scanner_type->setCurrentIndex(CBScanner->currentIndex());
        if (!osc->CalibWidget->fb_on_off_xy->on_off)
            osc->CalibWidget->fb_on_off_xy->change_state_man_with_sig(true);
    }
}

inline float ToPercent(float val, float min, float max)
{
    if (max == min) return 0;
    return (val - min)/(max - min);
}
inline float FromPercent(float val, float min, float max)
{
    return min + val*(max - min);
}

template<class T>
double NewtonResult(T arr[5], double x0, double value, double ac, DWORD max = 100)
{
    double x_new = x0;
    WORD N = 0;

    do {
        x0 = x_new;
        x_new = x0 -
                (((((arr[0] * x0 + arr[1]) * x0 + arr[2]) * x0 + arr[3]) * x0 + arr[4]) - value) /
                (((4*arr[0] * x0 + 3*arr[1]) * x0 + 2*arr[2]) * x0 + arr[3]);
        N++;
    } while ((fabs(x0 - x_new) > ac) && (N < max));

    return x_new;
}

static float CalibRangeCL[3];
static float ShiftCL[3];
extern inline void init_reverse_nlc_params(float *Scanner_data_pid_cal_range)
{
    ShiftCL[0] = NewtonResult(Scanner_data_pid_cal_range + 27, 10, 0, 0.001); //27-31.32
    CalibRangeCL[0] = NewtonResult(Scanner_data_pid_cal_range + 27, 50, 65535, 0.001) - ShiftCL[0];

    ShiftCL[1] = NewtonResult(Scanner_data_pid_cal_range + 33, 10, 0, 0.001); //33-37.38
    CalibRangeCL[1] = NewtonResult(Scanner_data_pid_cal_range + 33, 50, 65535, 0.001) - ShiftCL[1];

    ShiftCL[2] = NewtonResult(Scanner_data_pid_cal_range + 39, 10, 0, 0.001); //39-43.44
    CalibRangeCL[2] = NewtonResult(Scanner_data_pid_cal_range + 39, 50, 65535, 0.001) - ShiftCL[2];
}
//c = raw [0..65535]
inline float nonlinear_c_to_mkm_x(float c, float *Scanner_data_pid_cal_range)
{
    float C = NewtonResult(Scanner_data_pid_cal_range + 27, 10, c, 0.001); //27-31.32

    return 1e-6 * Scanner_data_pid_cal_range[45] * (C - ShiftCL[0]) / CalibRangeCL[0];
}
inline float nonlinear_c_to_mkm_y(float c, float *Scanner_data_pid_cal_range)
{
    float C = NewtonResult(Scanner_data_pid_cal_range + 33, 10, c, 0.001); //33-37.38

    return 1e-6 * Scanner_data_pid_cal_range[46] * (C - ShiftCL[1]) / CalibRangeCL[1];
}
//inline //not in gcc
extern float nonlinear_c_to_mkm_z(float c, float *Scanner_data_pid_cal_range)
{
    float C = NewtonResult(Scanner_data_pid_cal_range + 39, 10, c, 0.001); //39-43.44

    return 1e-6 * Scanner_data_pid_cal_range[47] * (C - ShiftCL[2]) / CalibRangeCL[2];
}

void ScanPlot::sl_fast_scan_correction(int ch)
{
#define DISTRIBUTE_1PX //extend to unused 1px range
    int idx = CBChannel[ch]->itemData(CBChannel[ch]->currentIndex()).toInt();

    if ((idx == srcNone) ||
            (idx == srcCCD) ||
            !dataScan_src->at(ch) ||
            !dataScan_src->at(ch)->Array.size())
        return;

    plot::T2dArray dataScan_new;
    dataScan_new.resize(Ny_points);

    plot::T2dArray dataScan_fill;
    dataScan_fill.resize(Ny_points);

    QVector<double> Row;
    Row.resize(Nx_points);
    for (int Xind = 0; Xind < Nx_points; Xind++)
        Row[Xind] = 0;
    for (int Yind = 0; Yind < Ny_points; Yind++) {
        dataScan_new[Yind] = Row;
        dataScan_fill[Yind] = Row;
    }

    bool nlc_on = *_wThread->Nlc_status_cl_xy;
    if (CBScanner->currentIndex() == scStage)
        nlc_on = *_wThread->Nlc_status_cl_xy2;

    float *Scanner_data_pid_cal_range = _wThread->Scanner_data_pid_cal_range;
    if (CBScanner->currentIndex() == scStage)
        Scanner_data_pid_cal_range = _wThread->Scanner2_data_pid_cal_range;

    float xmin = FromPercent(ToPercent(Plot->ThreeDPlots.at(0)->PlotsXLimits[0],
                                       Plot->ThreeDPlots.at(0)->DefaultXLimits[0],
                                       Plot->ThreeDPlots.at(0)->DefaultXLimits[1]),
                             0, 65535);
    float xmax = FromPercent(ToPercent(Plot->ThreeDPlots.at(0)->PlotsXLimits[1],
                                       Plot->ThreeDPlots.at(0)->DefaultXLimits[0],
                                       Plot->ThreeDPlots.at(0)->DefaultXLimits[1]),
                             0, 65535);
    float ymin = FromPercent(ToPercent(Plot->ThreeDPlots.at(0)->PlotsYLimits[0],
                                       Plot->ThreeDPlots.at(0)->DefaultYLimits[0],
                                       Plot->ThreeDPlots.at(0)->DefaultYLimits[1]),
                             0, 65535);
    float ymax = FromPercent(ToPercent(Plot->ThreeDPlots.at(0)->PlotsYLimits[1],
                                       Plot->ThreeDPlots.at(0)->DefaultYLimits[0],
                                       Plot->ThreeDPlots.at(0)->DefaultYLimits[1]),
                             0, 65535);


    int Xmin = 0, Xmax = 0, Ymin = 0, Ymax = 0;

    if (!scanData[CBScanner->currentIndex()][ch].size())
        return;
    Xmin = scanData[CBScanner->currentIndex()][ch].at(0)->LimsForMathX[0];
    Xmax = scanData[CBScanner->currentIndex()][ch].at(0)->LimsForMathX[1];
    Ymin = scanData[CBScanner->currentIndex()][ch].at(0)->LimsForMathY[0];
    Ymax = scanData[CBScanner->currentIndex()][ch].at(0)->LimsForMathY[1];

    Ymin = qMin(Ymin, dataScan_src->at(ch)->Array.size()-1);
    Ymax = qMin(Ymax, dataScan_src->at(ch)->Array.size()-1);
    Xmin = qMin(Xmin, dataScan_src->at(ch)->Array[Ymin].size()-1);
    Xmax = qMin(Xmax, dataScan_src->at(ch)->Array[Ymax].size()-1);

    plot::T2dArray *xdataptr = ch < 2 ? dataScan_X : dataScan_Xb;
    plot::T2dArray *ydataptr = ch < 2 ? dataScan_Y : dataScan_Yb;

    Ymin = qMin(Ymin, qMin((*xdataptr).size()-1, (*ydataptr).size()-1));
    Ymax = qMin(Ymax, qMin((*xdataptr).size()-1, (*ydataptr).size()-1));
    Xmin = qMin(Xmin, qMin((*xdataptr)[Ymin].size()-1, (*ydataptr)[Ymin].size()-1));
    Xmax = qMin(Xmax, qMin((*xdataptr)[Ymax].size()-1, (*ydataptr)[Ymax].size()-1));

    Ymin = qBound(0, Ymin, (int)Ny_points-1);
    Ymax = qBound(0, Ymax, (int)Ny_points-1);
    Xmin = qBound(0, Xmin, (int)Nx_points-1);
    Xmax = qBound(0, Xmax, (int)Nx_points-1);

    double min_value = std::numeric_limits<double>::max();


    if (nlc_on) {
        init_reverse_nlc_params(Scanner_data_pid_cal_range);

        for (int Yind = Ymin; Yind < Ymax; Yind++) {
            for (int Xind = Xmin; Xind < Xmax; Xind++) {
                int xind = qRound(
                            ToPercent(
                                nonlinear_c_to_mkm_x((*xdataptr)[Yind][Xind], Scanner_data_pid_cal_range),
                                Plot->ThreeDPlots.at(0)->PlotsXLimits[0], Plot->ThreeDPlots.at(0)->PlotsXLimits[1])
                            *(Nx_points-1));
                int yind = qRound(
                            ToPercent(
                                nonlinear_c_to_mkm_y((*ydataptr)[Yind][Xind], Scanner_data_pid_cal_range),
                                Plot->ThreeDPlots.at(0)->PlotsYLimits[0], Plot->ThreeDPlots.at(0)->PlotsYLimits[1])
                            *(Ny_points-1));

                if (xind >= 0 && xind < Nx_points && yind >= 0 && yind < Ny_points) {
                    if (min_value > dataScan_src->at(ch)->Array[Yind][Xind])
                        min_value = dataScan_src->at(ch)->Array[Yind][Xind];
                    dataScan_new[yind][xind] += dataScan_src->at(ch)->Array[Yind][Xind];
                    dataScan_fill[yind][xind]++;
                }
            }
        }
    } else {
        for (int Yind = Ymin; Yind < Ymax; Yind++) {
            for (int Xind = Xmin; Xind < Xmax; Xind++) {
                int xind = qRound(ToPercent((*xdataptr)[Yind][Xind], xmin, xmax)*(Nx_points-1));
                int yind = qRound(ToPercent((*ydataptr)[Yind][Xind], ymin, ymax)*(Ny_points-1));

                if (xind >= 0 && xind < Nx_points && yind >= 0 && yind < Ny_points) {
                    if (min_value > dataScan_src->at(ch)->Array[Yind][Xind])
                        min_value = dataScan_src->at(ch)->Array[Yind][Xind];
                    dataScan_new[yind][xind] += dataScan_src->at(ch)->Array[Yind][Xind];
                    dataScan_fill[yind][xind]++;
                }
            }
        }
    }

    if (min_value == std::numeric_limits<double>::max())
        min_value = 0;

    //nearest pixels relative coordinates
    const int xseq[8] = {0, 0, 1, -1, 1, 1, -1, -1};
    const int yseq[8] = {1, -1, 0, 0, 1, -1, 1, -1};

    //fill empty with min_value, normalize averages, [interpolate gaps]
    for (int Yind=0; Yind<Ny_points; Yind++) {
        for (int Xind=0; Xind<Nx_points; Xind++) {
            if (dataScan_fill[Yind][Xind] == 0) {
#ifdef DISTRIBUTE_1PX
                int count = 0;

                for (int i=0; i<8; i++) {
                    int _x = Xind + xseq[i];
                    int _y = Yind + yseq[i];
                    if (_x < 0 || _x >= Nx_points || _y < 0 || _y >= Ny_points)
                        continue;

                    if (dataScan_fill[_y][_x] != 0) {
                        dataScan_new[Yind][Xind] += dataScan_new[_y][_x] / dataScan_fill[_y][_x];
                        count++;
                    }
                }
                if (count != 0)
                    dataScan_new[Yind][Xind] /= count;
                else
#endif
                    dataScan_new[Yind][Xind] = min_value;
            } else {
                dataScan_new[Yind][Xind] /= dataScan_fill[Yind][Xind];
                dataScan_fill[Yind][Xind] = 1;
            }
        }
    }

    dataScan->at(ch)->Array = dataScan_new;
}
