#include "shuttle.h"
#include "controls.h"
#include "EG3000.h"
#include <QSettings>
#include <QLabel>
#include <QDial>
#include <QApplication>
#include <QVBoxLayout>
#include <QSignalMapper>
#include <QDir>
#include <QDebug>

const QString __b_off[15] = {
    ":/Shuttle/MObj.png",
    ":/Shuttle/PObj.png",
    ":/Shuttle/FBOn.png",
    ":/Shuttle/FBOff.png",
    ":/Shuttle/M123.png",
    ":/Shuttle/M1.png",
    ":/Shuttle/M2.png",
    ":/Shuttle/M3.png",
    ":/Shuttle/L.png",
    ":/CCD/onn.png",
    ":/CCD/onn.png",
    ":/CCD/onn.png",
    ":/CCD/onn.png",
    ":/CCD/onn.png",
    ":/CCD/onn.png",
};
const QString __b_on[15] = {
    ":/Shuttle/MObjOn.png",
    ":/Shuttle/PObjOn.png",
    ":/Shuttle/FBOn_Active.png",
    ":/Shuttle/FBActive.png",
    ":/Shuttle/M123On.png",
    ":/Shuttle/M1On.png",
    ":/Shuttle/M2On.png",
    ":/Shuttle/M3On.png",
    ":/Shuttle/LOn.png",
    ":/Osc_window/onn.png",
    ":/Osc_window/onn.png",
    ":/Osc_window/onn.png",
    ":/Osc_window/onn.png",
    ":/Osc_window/onn.png",
    ":/Osc_window/onn.png",
};

void ShuttleGuiControl::button_press_slot(int num)
{
    for (int i = 0; i < 15; i++)
        b[i]->setIcon(makeHugeIcon(__b_off[i]));

    b[num]->setIcon(makeHugeIcon(__b_on[num]));

    emit button_press(num);

    if (!deviceStatus)
        return;

    QSettings settings("Param/motor.conf", QSettings::IniFormat);

    int idx = -1;
    QString name;
    switch (num) {
    case 0:
        idx = settings_double(settings, "z_motor", 4);
        name = "Z-M";
        break;
    case 1:
        name = "Z-P";
        break;
    case 4:
        idx = settings.value("land_mode", 2).toInt()-1 != 0 ? -1 : 3;
        name = settings.value("land_mode", 2).toInt()-1 != 0 ? "123" : QString();
        break;
    case 5:
        idx = 1;
        break;
    case 6:
        idx = 2;
        break;
    case 7:
        idx = 3;
        break;
    case 9:
        idx = settings_double(settings, "x_motor", 7);
        name = "X2-M";
        break;
    case 10:
        name = "X-P";
        break;
    case 11:
        name = "X2-P";
        break;
    case 12:
        name = "Y2-P";
        break;
    case 13:
        name = "Y-P";
        break;
    case 14:
        idx = settings_double(settings, "y_motor", 8);
        name = "Y2-M";
        break;
    }

    QString n1 = tr("Shuttle: ");
    if (idx > 0) {
        QString stridx = QString::number(idx);
        QString name1 = settings.value("name/" + stridx).toString();
        if (!name1.isEmpty()) name = name1;

        n1 += stridx;
        if (!name.isEmpty()) n1 += " - ";
    }
    n1 += name;
    if (idx > 0 || !name.isEmpty())
        deviceStatus->setText(n1);
}
void ShuttleGuiControl::shuttle_position_slot(int num)
{
    dial_shuttle->setValue(num);
}

void ShuttleGuiControl::wheel_event_slot(int dir)
{
    int pos = dial_wheel->value() + dir;
    if (pos == -4)
        pos = 2;
    if (pos == 4)
        pos = -2;

    dial_wheel->blockSignals(true); //setVal duplicates signal, wheel_gui makes it wrong on -3<->3 border
    dial_wheel->setValue(pos);
    dial_wheel->blockSignals(false);

    emit wheel_event(dir);
}

void ShuttleGuiControl::wheel_gui(int num)
{
    static int prev = 0;

    if (abs(prev) == abs(num) && abs(num) == 3) {
        prev = num;
        return;
    }

    if (prev < num) {
        prev = num;
        emit wheel_event(1);
    } else {
        prev = num;
        emit wheel_event(-1);
    }
}

void ShuttleGuiControl::shuttle_press()
{
    if (!qApp->keyboardModifiers())
        snap_to_back = true;
}
void ShuttleGuiControl::shuttle_release()
{
    if (snap_to_back)
        dial_shuttle->setValue(0);
    snap_to_back = false;
}
void doBeep();
void ShuttleGuiControl::shuttle_gui(int num)
{
    if (snap_to_back && qAbs(num - prev_shuttle_val) > 7) {
        if (!wrap_warn)
            doBeep();
        wrap_warn = true;
        dial_shuttle->blockSignals(true);
        dial_shuttle->setValue(prev_shuttle_val);
        dial_shuttle->blockSignals(false);
        return;
    }
    wrap_warn = false;
    prev_shuttle_val = num;

    emit shuttle_position(num);
}

ShuttleGuiControl::ShuttleGuiControl()
{
    snap_to_back = false;
    prev_shuttle_val = 0;
    wrap_warn = false;

    setWindowTitle(tr("Shuttle Gui"));

    deviceStatus = new QLabel(tr("Shuttle: ") + "123");

    active_motor = 250; //all motors

    dial_shuttle = new QDial;
    dial_shuttle->setRange(-7, 7);
    dial_shuttle->setPageStep(7);
    dial_shuttle->setNotchesVisible(true);
    dial_shuttle->setFixedSize(160, 160);

    dial_wheel = new QDial;
    dial_wheel->setRange(-3, 3);
    dial_wheel->setNotchesVisible(true);
    dial_wheel->setWrapping(true);
    dial_wheel->setFixedSize(50, 50);

    QWidget *dww = new QWidget(dial_shuttle);
    QVBoxLayout *dwl = new QVBoxLayout();
    dwl->addWidget(dial_wheel, 0, Qt::AlignCenter);
    dww->setLayout(dwl);
    dww->setFixedSize(160, 160);

    for (int i = 0; i < 15; i++) {
        b[i] = new QPushButton(QIcon(__b_off[i]), QString());
        b[i]->setFixedSize(b[i]->sizeHint() + QSize(10, 10));
        b[i]->setFlat(true);
    }

    QHBoxLayout *row1 = new QHBoxLayout;
    row1->addWidget(b[0]);
    row1->addWidget(b[1]);
    row1->addWidget(b[2]);
    row1->addWidget(b[3]);

    QHBoxLayout *row2 = new QHBoxLayout;
    row2->addWidget(b[4]);
    row2->addWidget(b[5]);
    row2->addWidget(b[6]);
    row2->addWidget(b[7]);
    row2->addWidget(b[8]);

    QHBoxLayout *row3 = new QHBoxLayout;
    row3->addWidget(b[9]);
    row3->addWidget(dial_shuttle);
    row3->addWidget(b[14]);

    QGridLayout *row4 = new QGridLayout;
    row4->addWidget(b[10], 0, 0);
    row4->addWidget(b[11], 1, 0);
    row4->addWidget(b[12], 1, 1);
    row4->addWidget(b[13], 0, 1);

    QVBoxLayout *mainb = new QVBoxLayout(this);
    mainb->setSizeConstraint(QLayout::SetFixedSize);

    mainb->addLayout(row1);
    mainb->addLayout(row2);
    mainb->addLayout(row3);
    mainb->addLayout(row4);

#ifdef _ADROID
    QPushButton *cwC = new QPushButton(tr("Close"));
    connect(cwC, SIGNAL(clicked()), this, SLOT(hide()));
    mainb->insertWidget(0, cwC);
#endif

    connect(dial_shuttle, SIGNAL(valueChanged(int)), SLOT(shuttle_gui(int)));
    connect(dial_shuttle, SIGNAL(sliderPressed()), SLOT(shuttle_press()));
    connect(dial_shuttle, SIGNAL(sliderReleased()), SLOT(shuttle_release()));
    connect(dial_wheel, SIGNAL(valueChanged(int)), SLOT(wheel_gui(int)));

    QSignalMapper *BsignalMapper = new QSignalMapper(this);
    for (int i = 0; i < 15; i++) {
        BsignalMapper->setMapping(b[i], i);
        connect(b[i], SIGNAL(clicked()), BsignalMapper, SLOT(map()));
    }

    connect(BsignalMapper, SIGNAL(mapped(int)), SLOT(button_press_slot(int)));
}


ShuttleControllerNative *scninst = 0;
#ifdef _WIN32
#define WIN32_LEAN_AND_MEAN
#define NOMINMAX
#include <windows.h>
bool myEventFilter(void *message, long *)
{
    MSG *p = (MSG *)message;
    if (p->message == WM_INPUT) {
        UINT dataSize = 256;
        BYTE buf[256];
        GetRawInputData((HRAWINPUT)p->lParam, RID_INPUT, buf, &dataSize, sizeof(RAWINPUTHEADER));
        const RAWINPUT *raw = (const RAWINPUT*)buf;
        if (raw->header.dwType == RIM_TYPEHID) {
            if (raw->data.hid.dwCount == 1 && raw->data.hid.dwSizeHid == 6) {
                if (scninst)
                    scninst->set_hid((signed char)raw->data.hid.bRawData[1], raw->data.hid.bRawData[2], raw->data.hid.bRawData[4] + (raw->data.hid.bRawData[5] << 8));
                return true;
            }
        }
    }
    return false;
}
#if (QT_VERSION >= QT_VERSION_CHECK(5, 0, 0))
#include <QAbstractNativeEventFilter>
class efilt : public QAbstractNativeEventFilter
{
public:
     bool nativeEventFilter(const QByteArray &, void * message, long * result)
     {
         return myEventFilter(message, result);
     }
};
efilt nativefilter;
#endif
#endif

#if defined(Q_OS_LINUX)
#include <linux/input.h>
#include <fcntl.h>
#include <errno.h>
static volatile bool evrun;
class EvReader : public QThread
{
    void run() {
        while (evrun) {
            QStringList l = QDir("/dev/input/by-id", "*Contour_Design_Shuttle*").entryList(QDir::Files);
            if (!l.size()) {
                for (int i = 0; i < 10 && evrun; i++)
                    msleep(50);
                continue;
            }

            QFile f("/dev/input/by-id/" + l.first());
            if (!f.open(QIODevice::ReadWrite | QIODevice::Unbuffered)) {
                qDebug() << l.first() << "open fail";
                for (int i = 0; i < 10 && evrun; i++)
                    msleep(50);
                continue;
            }
            int flags = fcntl(f.handle(), F_GETFL, 0);
            fcntl(f.handle(), F_SETFL, flags | O_NONBLOCK);
            struct {timeval t; WORD type; WORD code; uint val;} ev;
            int wheel_abs = 0;
            int shuttle = 0;
            long buttons = 0;
            do {
                int read = f.read((char *)&ev, sizeof(ev));
                if (read != sizeof(ev)) {
                    if (errno != EWOULDBLOCK)
                        break;
                    msleep(10);
                    continue;
                }
                //qDebug() << ev.type << ev.code << ev.val;
                if (ev.type == EV_KEY) {
                    if (ev.val)
                        buttons |= 1 << (ev.code - 256);
                    else
                        buttons &= ~(1 << (ev.code - 256));
                }
                if (ev.type == EV_REL && ev.code == 7)
                    wheel_abs = ev.val;
                if (ev.type == EV_REL && ev.code == 8)
                    shuttle = (signed char)ev.val;
                if (ev.type == EV_SYN) {
                    if (scninst)
                        scninst->set_hid(shuttle, wheel_abs, buttons);

                    wheel_abs = 0;
                    shuttle = 0;

                    //fake press, so we always get sync report even with rel wheel and shuttle == 0
                    //button still works as this changes kernel button state too
                    ev.type = EV_KEY; ev.code = BTN_0; ev.val = 1;
                    ev.t.tv_usec++;
                    f.write((char *)&ev, sizeof(ev));
                    ev.type = EV_KEY; ev.code = BTN_0; ev.val = 0;
                    ev.t.tv_usec++;
                    f.write((char *)&ev, sizeof(ev));
                }
            } while (evrun);
            f.close();
        }
    }
};
EvReader evreader;
#endif

ShuttleControllerNative::ShuttleControllerNative(QObject *parent) : QObject(parent)
{
    if (scninst) throw scninst;
    scninst = this;

    QSettings settings("Param/shuttle.conf", QSettings::IniFormat);
    small_remap = settings.value("shuttle_xpress", false).toBool();
    first_event = true;
    prev_wheel = 127;
    prev_shuttle = 0;
    for (int i = 0; i < 16; i++)
        prev_buttons[i] = 0;

#if defined(Q_OS_LINUX)
    QStringList l = QDir("/dev/input/by-id", "*Contour_Design_Shuttle*").entryList(QDir::Files);
    if (l.size())
        small_remap = !l.first().contains("PRO_v2");

    evrun = true;
    evreader.start(QThread::LowPriority);
#endif

#ifdef _WIN32
    RAWINPUTDEVICE device;
    device.usUsagePage = 12;
    device.usUsage = 1;
    device.dwFlags = RIDEV_INPUTSINK;
    device.hwndTarget = (HWND)qApp->topLevelWidgets().first()->winId();
    RegisterRawInputDevices(&device, 1, sizeof device);

#if (QT_VERSION >= QT_VERSION_CHECK(5, 0, 0))
    qApp->installNativeEventFilter(&nativefilter);
#else
    qApp->setEventFilter(&myEventFilter);
#endif
#endif
}
#include "mainwindow.h"
ShuttleControllerNative::~ShuttleControllerNative()
{
    scninst = 0;
#if defined(Q_OS_LINUX)
    evrun = false;
    while (evreader.isRunning()) {
        qApp->processEvents();
        SleepExport::msleep2(3);
    }
#endif
}
void ShuttleControllerNative::set_hid(int shuttle, int wheel_abs, long rawbuttons)
{
    if (shuttle != prev_shuttle)
        emit shuttle_position(prev_shuttle = shuttle);

    int delta = (signed char)((wheel_abs - prev_wheel + 256)%256);
    if (qAbs(delta) == 1)
        emit wheel_event(delta);
    prev_wheel = wheel_abs;

    for (int i = 0; i < 16; i++) {
        bool ebuttons = rawbuttons & (1 << i);
        int idx = i;
        if (small_remap) idx += 4; //xpress remap
        if (i == 9) idx = 10; if (i == 13) idx = 9; if (i == 10) idx = 13; //pro v2 remap
        if (ebuttons && ebuttons != prev_buttons[i])
            emit button_press(idx);
        prev_buttons[i] = ebuttons;
    }

    if (first_event) {
        first_event = false;
        QObject *ptr = qApp->findChild<ShuttleController *>(); //needs qApp parent
        if (ptr) ptr->blockSignals(true); //we don't need duplicate events from shuttle driver
    }
}


void ShuttleController::key_event(char key)
{
    switch(key) {
    case 'Y':
        emit button_press(0);
        break;
    case 'U':
        emit button_press(1);
        break;
    case 'I':
        emit button_press(2);
        break;
    case 'O':
        emit button_press(3);
        break;

    case 'G':
        emit button_press(4);
        break;
    case 'H':
        emit button_press(5);
        break;
    case 'J':
        emit button_press(6);
        break;
    case 'K':
        emit button_press(7);
        break;
    case 'L':
        emit button_press(8);
        break;

    case 'V':
        emit button_press(9);
        break;
    case 'B':
        emit button_press(10);
        break;
    case 'N':
        emit button_press(11);
        break;
    case 'M':
        emit button_press(12);
        break;
    case ',':
        emit button_press(13);
        break;
    case '.':
        emit button_press(14);
        break;

    case 'Z':
        emit wheel_event(-1);
        break;
    case 'X':
        emit wheel_event(1);
        break;

    case '1':
        emit shuttle_position(-1);
        break;
    case '2':
        emit shuttle_position(-2);
        break;
    case '3':
        emit shuttle_position(-3);
        break;
    case '4':
        emit shuttle_position(-4);
        break;
    case '5':
        emit shuttle_position(-5);
        break;
    case '6':
        emit shuttle_position(-6);
        break;
    case '7':
        emit shuttle_position(-7);
        break;

    case 'D':
        emit shuttle_position(0);
        break;

    case 'Q':
        emit shuttle_position(1);
        break;
    case 'W':
        emit shuttle_position(2);
        break;
    case 'E':
        emit shuttle_position(3);
        break;
    case 'R':
        emit shuttle_position(4);
        break;
    case 'T':
        emit shuttle_position(5);
        break;
    case 'A':
        emit shuttle_position(6);
        break;
    case 'S':
        emit shuttle_position(7);
        break;
    }
}
