#ifndef SHUTTLE_H
#define SHUTTLE_H

#include <QWidget>
#include <QPointer>

class QPushButton;
class QDial;
class QLabel;
class ShuttleGuiControl: public QWidget
{
    Q_OBJECT

signals:
    void button_press(int num);// 0..15
    void shuttle_position(int num);// -7..7 current position
    void wheel_event(int dir);// -1 left 1 right

public slots:
    void button_press_slot(int num);// 0..15
    void shuttle_position_slot(int num);// -7..7 current position
    void wheel_event_slot(int dir);// -1 left 1 right

    void shuttle_gui(int num);
    void shuttle_press();
    void shuttle_release();
    void wheel_gui(int num);

public:
    ShuttleGuiControl();

    QPushButton *b[15];
    QDial *dial_shuttle;
    QDial *dial_wheel;

    int active_motor;
    bool snap_to_back;
    int prev_shuttle_val;
    bool wrap_warn;

    QPointer<QLabel> deviceStatus;
};

class ShuttleControllerNative: public QObject
{
    Q_OBJECT

signals:
    void button_press(int num);// 0..15
    void shuttle_position(int num);// -7..7 current position
    void wheel_event(int dir);// -1 left 1 right

public:
    ShuttleControllerNative(QObject *parent = 0);
    ~ShuttleControllerNative();
    void set_hid(int shuttle, int wheel_abs, long rawbuttons);

    int prev_wheel, prev_shuttle;
    bool prev_buttons[16];
    bool first_event;
    bool small_remap;
};

//handler is installed in mainwindow::eventFilter
class ShuttleController: public QObject
{
    Q_OBJECT

public:
    ShuttleController(QObject *parent = 0) : QObject(parent) { }

signals:
    void button_press(int num);// 0..15
    void shuttle_position(int num);// -7..7 current position
    void wheel_event(int dir);// -1 left 1 right

public slots:
    void key_event(char key);
};

#endif //SHUTTLE_H
