#include "spmtunning.h"
#include "spm_motors.hpp"
#include "edit.hpp"

#include <QTimer>
#include <QApplication>
#include <QCheckBox>
#include <QGroupBox>
#include <QComboBox>
#include <QVBoxLayout>
#include <QSettings>
#include <QProgressBar>
#include <QFormLayout>
#include "flowlayout.hpp"

#include "EG3000.h"
#include "mainwindow.h"
#include "shuttle.h"
#include "motors.hpp"


typedef QPair<int, int> FBinfo;
Q_DECLARE_METATYPE(FBinfo)


TOptionsForMotor::TOptionsForMotor()
{
    LEdUpSpeed = new Edit(0, 10000, 3);
    LEdUpDist = new Edit(0, 1000, 3);
    LEd_Level_speed = new Edit(0, 1000, 3);
    connect(LEdUpSpeed, SIGNAL(newValue()), this, SLOT(sl_update()));
    connect(LEdUpDist, SIGNAL(newValue()), this, SLOT(sl_update()));
    connect(LEd_Level_speed, SIGNAL(newValue()), this, SLOT(sl_update()));

    LEdLandingSpeed = new Edit(0, 1000, 3);
    LEdLandingSpeedClose = new Edit(0, 1000, 3);
    LEdLandingSpeedCloseC = new Edit(0, 1, 3);
    connect(LEdLandingSpeed, SIGNAL(newValue()), this, SLOT(sl_update()));
    connect(LEdLandingSpeedClose, SIGNAL(newValue()), this, SLOT(sl_update()));
    connect(LEdLandingSpeedCloseC, SIGNAL(newValue()), this, SLOT(sl_update()));


    CB_landing_Mode = new QComboBox;
    CB_landing_Mode->addItem(tr("3"));
    CB_landing_Mode->addItem(tr("1,2,3"));
    CB_landing_Mode->addItem(tr("1,2,3, slow(3 upward)"));
    CB_landing_Mode->addItem(tr("1,2,3, slow(1,2,3)"));
    CB_landing_Mode->setToolTip(tr("mode 1: motor 3") + "\n" +
                                tr("modes 2,3,4: motors 1,2,3") + "\n" +
                                tr("modes 3,4 will slow down when close.") + "\n" +
                                tr("mode 3 will switch to single motor 3 in reverse direction for super-slow landing"));
    connect(CB_landing_Mode, SIGNAL(currentIndexChanged(int)), this, SLOT(sl_update()));

    CB_Device = new QCheckBox(tr("Snotra/Tuning fork"));
    connect(CB_Device, SIGNAL(toggled(bool)), this, SIGNAL(sig_Device_ID(bool)));
    connect(CB_Device, SIGNAL(toggled(bool)), this, SLOT(sl_save_to_file()));

    QVBoxLayout *HBLParams = new QVBoxLayout(this);
    HBLParams->setSizeConstraint(QLayout::SetFixedSize);
    HBLParams->setSpacing(3);
    HBLParams->setContentsMargins(6, 3, 6, 6);

    QGroupBox *GrB_DeviceID = new QGroupBox(tr("Device Type"), this);
    QFormLayout *VBL_Device_ID = makeFormLayout(GrB_DeviceID);

    QGroupBox *GrBUp = new QGroupBox(tr("Movement"), this);
    QFormLayout *VBLUpSpeed = makeFormLayout(GrBUp);

    QGroupBox *GrBLanding = new QGroupBox(tr("Landing"), this);
    QFormLayout *VBLLanding = makeFormLayout(GrBLanding);

    QGroupBox *GrB_Level_speed = new QGroupBox(tr("Motors Level"), this);
    QFormLayout *VBL_Level_speed = makeFormLayout(GrB_Level_speed);

    HBLParams->addWidget(GrB_DeviceID);
    HBLParams->addWidget(GrBLanding);
    HBLParams->addWidget(GrBUp);
    HBLParams->addWidget(GrB_Level_speed);

    VBL_Device_ID->addRow(CB_Device);
    VBL_Device_ID->setContentsMargins(6, 3, 6, 6);

    VBLUpSpeed->addRow(tr("Speed (")+tr("micro")+tr("m/s)"), LEdUpSpeed);
    VBLUpSpeed->addRow(tr("Distance (mm)"), LEdUpDist);
    VBLUpSpeed->setContentsMargins(6, 3, 6, 6);

    VBLLanding->addRow(tr("Motors"), CB_landing_Mode);
    VBLLanding->addRow(tr("Speed (")+tr("micro")+tr("m/s)"), LEdLandingSpeed);
    VBLLanding->addRow(tr("Slow Speed (")+tr("micro")+tr("m/s)"), LEdLandingSpeedClose);
    VBLLanding->addRow(tr("Slow SP %[0..1]"), LEdLandingSpeedCloseC);
    VBLLanding->setContentsMargins(6, 3, 6, 6);

    VBL_Level_speed->addRow(tr("Speed (")+tr("micro")+tr("m/s)"), LEd_Level_speed);
    VBL_Level_speed->setContentsMargins(6, 3, 6, 6);

    setWindowTitle(tr("Motor Options - NSpec"));
}
void TOptionsForMotor::sl_save_to_file()
{
    QSettings settings("Param/motor.conf", QSettings::IniFormat);
    settings.setValue("device_id", CB_Device->isChecked() ? 4 : 0);

    settings.setValue("up_speed", up_speed);
    settings.setValue("up_dist", up_dist / 1e3); //from mkm
    settings.setValue("level_speed", level_speed);

    settings.setValue("land_speed", land_speed);
    settings.setValue("land_speed_close", land_speed_close);
    settings.setValue("land_close_perc", land_close_perc);
    settings.setValue("land_mode", Landing_Mode);
}
void TOptionsForMotor::sl_load_from_file()
{
    QSettings settings("Param/motor.conf", QSettings::IniFormat);

    up_speed = settings.value("up_speed", 500).toDouble();
    up_dist = settings.value("up_dist", 1).toDouble() * 1e3; //to mkm
    level_speed = settings.value("level_speed", 100).toDouble();

    land_speed = settings.value("land_speed", 50).toDouble();
    land_speed_close = settings.value("land_speed_close", 5).toDouble();
    land_close_perc = settings.value("land_close_perc", 0.7).toDouble();
    Landing_Mode = settings.value("land_mode", 2).toInt();

    int Device = settings.value("device_id", 0).toInt();
    CB_Device->setChecked(Device == 4); //snotra
    emit sig_Device_ID(Device == 4);

    LEdUpSpeed->setValue(up_speed);
    LEdUpDist->setValue(up_dist / 1e3); //from mkm
    LEd_Level_speed->setValue(level_speed);

    LEdLandingSpeed->setValue(land_speed);
    LEdLandingSpeedClose->setValue(land_speed_close);
    LEdLandingSpeedCloseC->setValue(land_close_perc);
    CB_landing_Mode->setCurrentIndex(Landing_Mode-1);
}
void TOptionsForMotor::sl_update()
{
    up_speed = LEdUpSpeed->value();
    up_dist = LEdUpDist->value() * 1e3; //to mkm
    level_speed = LEd_Level_speed->value();

    land_speed = LEdLandingSpeed->value();
    land_speed_close = LEdLandingSpeedClose->value();
    land_close_perc = LEdLandingSpeedCloseC->value();
    Landing_Mode = CB_landing_Mode->currentIndex() + 1;

    sl_save_to_file();
}



void SPMTunning::slot_m123stopped()
{
    last_command = 0;
}

void SPMTunning::motors_stop()
{
    motors->goStop(250); //123 motors
    motors->goStop(4); //4 motor
}

void SPMTunning::slot_DoMoveUp()
{
    BLanding->change_state_man(false);
    blanding_status = false;
    MobileServer::sendData(DataID::LandingOn, blanding_status);

    BYTE CurrentMotor = OptionsForMotor->CB_landing_Mode->currentIndex() != 0 ? 250 : 3;
    if (last_command == 1) { last_command = 0; return motors->goStop(CurrentMotor); }
    last_command = 1;
    QSettings settings("Param/motor.conf", QSettings::IniFormat);
    double k = settings_double(settings, "divider/123", 1);
    motors->goUp(CurrentMotor, OptionsForMotor->up_speed * k, OptionsForMotor->up_dist * k);
}

void SPMTunning::slot_DoFastMoveUp()
{
    BLanding->change_state_man(false);
    blanding_status = false;
    MobileServer::sendData(DataID::LandingOn, blanding_status);

    BYTE CurrentMotor = OptionsForMotor->CB_landing_Mode->currentIndex() != 0 ? 250 : 3;
    if (last_command == 2) { last_command = 0; return motors->goStop(CurrentMotor); }
    last_command = 2;
    QSettings settings("Param/motor.conf", QSettings::IniFormat);
    double k = settings_double(settings, "divider/123", 1);
    motors->goUpE(CurrentMotor, OptionsForMotor->up_speed * k);
}

void SPMTunning::slot_DoMoveDown()
{
    BLanding->change_state_man(false);
    blanding_status = false;
    MobileServer::sendData(DataID::LandingOn, blanding_status);

    BYTE CurrentMotor = OptionsForMotor->CB_landing_Mode->currentIndex() != 0 ? 250 : 3;
    if (last_command == -1) { last_command = 0; return motors->goStop(CurrentMotor); }
    last_command = -1;
    QSettings settings("Param/motor.conf", QSettings::IniFormat);
    double k = settings_double(settings, "divider/123", 1);
    motors->goDown(CurrentMotor, OptionsForMotor->up_speed * k, OptionsForMotor->up_dist * k);
}

void SPMTunning::slot_DoFastMoveDown()
{
    BLanding->change_state_man(false);
    blanding_status = false;
    MobileServer::sendData(DataID::LandingOn, blanding_status);

    BYTE CurrentMotor = OptionsForMotor->CB_landing_Mode->currentIndex() != 0 ? 250 : 3;
    if (last_command == -2) { last_command = 0; return motors->goStop(CurrentMotor); }
    last_command = -2;
    QSettings settings("Param/motor.conf", QSettings::IniFormat);
    double k = settings_double(settings, "divider/123", 1);
    motors->goDownE(CurrentMotor, OptionsForMotor->up_speed * k);
}


void SPMTunning::slot_DoLanding(bool st)
{
    QString text = tr("Landing Status: Waiting for response....");
    MobileServer::sendData(DataID::Land_status, text);

    last_command = 0;
    if (st) {
        QSettings settings("Param/motor.conf", QSettings::IniFormat);

        BYTE direction = (motors->orient123 != settings.value("landing_upward", false).toBool()) ? 0 : 1;
        double k = motors->Estepsrev->value() / 16 / motors->Epitch->value() / 1e3;
        k *= settings_double(settings, "divider/123", 1);
        emit Landing_Data(OptionsForMotor->Landing_Mode, direction,
                          OptionsForMotor->land_speed * k, OptionsForMotor->land_speed_close * k,
                          OptionsForMotor->land_close_perc);


        switch (fbs->itemData(fbs->currentIndex()).value<QPair<int, int> >().second) {
        case srcDFL: {
            double cur = 1.1*nDFL;
            double sp = 2.2*DCfbSetPoint->value() * spm_sp_units_gain;
            double pad = qAbs(input_gain(srcDFL)) * (B_PID_sign->on_off || cur < sp ? -1 : 1);
            OscYrange[0] = qMin(qMin(sp, pad), cur);
            OscYrange[1] = qMax(qMax(sp, cur), pad);
            OscYrange[0] /= input_gain(srcDFL);
            OscYrange[1] /= input_gain(srcDFL);
        }
            break;
        case srcLFL: {
            double cur = 1.1*nLFL;
            double sp = 2.2*DCfbSetPoint->value() * spm_sp_units_gain;
            double pad = qAbs(input_gain(srcLFL)) * (B_PID_sign->on_off || cur < sp ? -1 : 1);
            OscYrange[0] = qMin(qMin(sp, pad), cur);
            OscYrange[1] = qMax(qMax(sp, cur), pad);
            OscYrange[0] /= input_gain(srcLFL);
            OscYrange[1] /= input_gain(srcLFL);
        }
            break;
        case srcMag1:
            OscYrange[0] = 0;
            OscYrange[1] = qMax(1.1*nMAG, 2.2*DCfbSetPoint->value() * spm_sp_units_gain);
            OscYrange[0] /= input_gain(srcMag1);
            OscYrange[1] /= input_gain(srcMag1);
            break;
        }

        PlotMode = FbSignalOscMod;
        sl_setPlotMode();


        if (!landing_bar) {
            landing_bar = new StatusBar(text, 0);
            connect(landing_bar, SIGNAL(canceled()), BLanding, SLOT(click()));
            landing_bar->add();
        } else
            landing_bar->statusLabel->setText(text);
    } else {
        emit Landing_Data(lmOff, 0, 0, 0, 0);
        destructStatus(landing_bar);
    }
    destructStatus(landing_notification);
}

void SPMTunning::blanding_sync()
{
    if (BLanding->on_off != blanding_status)
        doBeep();

    BLanding->change_state_man(blanding_status);
    if (!blanding_status)
        BLandingM4->change_state_man_with_sig(false);
    MobileServer::sendData(DataID::LandingOn, blanding_status);
}

void SPMTunning::slot_set_Landing_status(int status)
{
    destructStatus(landing_notification);

    QString text;
    switch (status)
    {
    case lndOff:
        text = tr("Landing Status: Canceled...");
        destructStatus(landing_bar);
        break;
    case lndLanding:
        text = tr("Landing Status: In progress...");
        if (landing_bar) {
            landing_bar->bar->setMaximum(0);
            Progress_timer->stop();
        }
        blanding_status = true;
        blanding_syncT->start(100);
        break;
    case lndDone:
        text = tr("Landing Status: Done...");
        destructStatus(landing_bar);
        landing_notification = new Notification(text, Notification::Good, 10);
        blanding_status = false;
        blanding_syncT->start(100);
        break;
    case lndError:
        text = tr("Landing Status: Error...");
        if (!BFeedbackOnOff->on_off) text += " " + tr("Feedback is disabled!");
        destructStatus(landing_bar);
        landing_notification = new Notification(text, Notification::Bad, 60);
        blanding_status = false;
        blanding_syncT->start(100);
        break;
    case lndWaitDrift:
        text = tr("Landing Status: Checking stability");
        if (landing_bar) {
            landing_bar->bar->setMaximum(5*1000);
            landing_bar->bar->setValue(0);
            landing_bar->bar->setFormat("%p%");
            Progress_timer->start(100);
        }
        blanding_status = true;
        blanding_syncT->start(100);
        break;
    }

    if (landing_bar) landing_bar->statusLabel->setText(text);
    else Progress_timer->stop();
    MobileServer::sendData(DataID::Land_status, text);
}

void SPMTunning::s_updLandProgBar()
{
    if (!landing_bar)
        return;

    if (landing_bar->bar->maximum() && landing_bar->bar->value() < 5*1000)
        landing_bar->setValue(landing_bar->bar->value() + 100);
    else {
        Progress_timer->stop();
        landing_bar->bar->setMaximum(0);
    }
}

void SPMTunning::s_landing_m4(bool val)
{
    int landing_m4_mode = 0;
    if (val)
        landing_m4_mode = (motors->orient123 == motors->orient[3]) ? 1 : 2;
    emit m4landing(landing_m4_mode);
}

void SPMTunning::InitMotorControlSpace()
{
    last_command = 0;

    QPushButton *BMoveDown = new QPushButton(makeHugeIcon(":/SPMTuning_window/Down.png"), QString());
    BMoveDown->setToolTip(tr("Move Tip Down"));

    QPushButton *BMoveUp = new QPushButton(makeHugeIcon(":/SPMTuning_window/Up.png"), QString());
    BMoveUp->setToolTip(tr("Move Tip Up"));

    QPushButton *BFastMoveDown = new QPushButton(makeHugeIcon(":/SPMTuning_window/FastDown.png"), QString());
    BFastMoveDown->setToolTip(tr("Fast Tip Down till end"));

    QPushButton *BFastMoveUp = new QPushButton(makeHugeIcon(":/SPMTuning_window/FastUp.png"), QString());
    BFastMoveUp->setToolTip(tr("Fast Tip Up till end"));

    BLanding = new TwoStateButton(":start.png",":stop.png");
    BLanding->setProperty("id", DataID::LandingOn);
    BLanding->setToolTip(tr("Tip Landing"));

    BLandingM4 = new TwoStateButton(":/SPMTuning_window/CanOb_Of.png",":/SPMTuning_window/CanOb_On.png");
    BLandingM4->setToolTip(tr("Sync Focus with Landing") + "\n" + tr("Enable before landing"));
    connect(BLandingM4, SIGNAL(state_changed(bool)), this, SLOT(s_landing_m4(bool)));

    BMotors_leveling = new QPushButton();
    BMotors_leveling->setIcon(makeHugeIcon(":/SPMTuning_window/Motors_leveling.png"));
    BMotors_leveling->setToolTip(tr("Motors control"));

    QPushButton *BOptions = new QPushButton();
    BOptions->setIcon(makeHugeIcon(":/SPMTuning_window/tools.png"));
    BOptions->setToolTip(tr("SPM/Motor Options"));

    QPushButton *BShuttleControl = new QPushButton();
    BShuttleControl->setIcon(makeHugeIcon(":/Shuttle/ShWind.png"));
    BShuttleControl->setToolTip(tr("Shuttle Gui"));
    BShuttleControl->setCheckable(true);

    QPushButton *BMotorsStop = new QPushButton();
    BMotorsStop->setIcon(makeHugeIcon(":/SPMTuning_window/approach/landing_done.png"));
    BMotorsStop->setToolTip(tr("Stop all motors"));

    connect(BMoveDown,     SIGNAL(clicked()), this, SLOT(slot_DoMoveDown()));
    connect(BMoveUp,       SIGNAL(clicked()), this, SLOT(slot_DoMoveUp()));
    connect(BFastMoveDown, SIGNAL(clicked()), this, SLOT(slot_DoFastMoveDown()));
    connect(BFastMoveUp,   SIGNAL(clicked()), this, SLOT(slot_DoFastMoveUp()));
    connect(BLanding,      SIGNAL(state_changed(bool)), this, SLOT(slot_DoLanding(bool)));
    connect(BMotorsStop, SIGNAL(clicked()), SLOT(motors_stop()));

    connect(BOptions,      SIGNAL(clicked()), OptionsForMotor, SLOT(show()));
    connect(BOptions,      SIGNAL(clicked()), OptionsForMotor, SLOT(raise()));
    connect(BShuttleControl, SIGNAL(clicked()), shuttle_gui, SLOT(show()));
    connect(BShuttleControl, SIGNAL(clicked()), shuttle_gui, SLOT(raise()));

    if (!show_additional_gui())
        BLandingM4->hide();
    if (!show_additional_gui() || !show_advanced_gui())
        BShuttleControl->hide();


    GrBMotorControl = new QGroupBox(this);
    QVBoxLayout *Motor_Layout = new QVBoxLayout(GrBMotorControl);
    Motor_Layout->setContentsMargins(3, 3, 3, 3);
    Motor_Layout->setSpacing(6);
    Motor_Layout->addWidget(BLanding);
    Motor_Layout->addWidget(BMoveUp);
    Motor_Layout->addWidget(BMoveDown);
    Motor_Layout->addWidget(BFastMoveUp);
    Motor_Layout->addWidget(BFastMoveDown);
    Motor_Layout->addWidget(BMotorsStop);

    addDelimeterH(Motor_Layout, BMoveUp->sizeHint().width());
    Motor_Layout->addWidget(BMotors_leveling);
    Motor_Layout->addWidget(BOptions);
    Motor_Layout->addWidget(BShuttleControl);
    Motor_Layout->addWidget(BLandingM4);
}



void SPMTunning::shuttle_button(int num)
{
    //first row
    if (num == 0) {
        QSettings settings("Param/motor.conf", QSettings::IniFormat);
        shuttle_gui->active_motor = settings_double(settings, "z_motor", 4);
        piezo_shuttle = 0;
    }
    if (num == 1)
        shuttle_gui->active_motor = 251;//piezo Z

    if (num == 2) {
        BFeedbackOnOff->change_state_man_with_sig(true);
    }
    if (num == 3) {
        BFeedbackOnOff->change_state_man_with_sig(false);
    }

    //second row
    if (num == 4) {
        shuttle_gui->active_motor = (OptionsForMotor->CB_landing_Mode->currentIndex() != 0 ? 250 : 3); // all motors
        piezo_shuttle = 0;
    }
    if (num == 5) {
        shuttle_gui->active_motor = 1;
        piezo_shuttle = 0;
    }
    if (num == 6) {
        shuttle_gui->active_motor = 2;
        piezo_shuttle = 0;
    }
    if (num == 7) {
        shuttle_gui->active_motor = 3;
        piezo_shuttle = 0;
    }

    if (num == 8) {
        BLanding->click();
        blanding_status = BLanding->on_off;
    }

    if (num == 9) {
        QSettings settings("Param/motor.conf", QSettings::IniFormat);
        shuttle_gui->active_motor = settings_double(settings, "x_motor", 7);
        piezo_shuttle = 0;
    }

    if (num == 10)
        shuttle_gui->active_motor = 252;//piezo X
    if (num == 11)
        shuttle_gui->active_motor = 254;//piezo X2
    if (num == 13)
        shuttle_gui->active_motor = 253;//piezo Y
    if (num == 12)
        shuttle_gui->active_motor = 255;//piezo Y2

    if (num == 14) {
        QSettings settings("Param/motor.conf", QSettings::IniFormat);
        shuttle_gui->active_motor = settings_double(settings, "y_motor", 8);
        piezo_shuttle = 0;
    }

}

void SPMTunning::shuttle_wheel(int num)
{
    num *= -1;
    if (shuttle_gui->active_motor < 251) {
        motors->goShuttleStep(num, shuttle_gui->active_motor);
    } else {
        double sign = num > 0 ? 1 : -1;
        double speed = piezo_speed;
        if (abs(num) != 1)
            speed = speeds[abs(num)-2];

        if (shuttle_gui->active_motor == 251)
            go_relative_objective_z(sign*speed);

        if (shuttle_gui->active_motor == 252)
            go_relative_x(sign*speed);

        if (shuttle_gui->active_motor == 253)
            go_relative_y(sign*speed);

        if (shuttle_gui->active_motor == 254)
            go_relative_x2(sign*speed);

        if (shuttle_gui->active_motor == 255)
            go_relative_y2(sign*speed);
    }
}

void SPMTunning::shuttle_position_piezo_timeout()
{
    if (shuttle_gui->active_motor >= 251 && piezo_shuttle != 0)
        shuttle_wheel(piezo_shuttle);
}

void SPMTunning::shuttle_position(int num)
{
    num *= -1;
    if (shuttle_gui->active_motor < 251) {
        motors->goShuttle(num, shuttle_gui->active_motor);
    } else {
        piezo_shuttle = num;
        double sign = piezo_shuttle > 0 ? 1 : -1;
        if (num != 0)
            piezo_shuttle = sign*(abs(piezo_shuttle) + 1);
    }
}

void SPMTunning::InitShuttleGuiLink()
{
    Shuttle = new ShuttleController(qApp);

    shuttle_gui = new ShuttleGuiControl;
    shuttle_gui->active_motor = (OptionsForMotor->CB_landing_Mode->currentIndex() != 0 ? 250 : 3); // all motors
    shuttle_gui->hide();
    piezo_shuttle = 0;

    QSettings settings("Param/shuttle.conf", QSettings::IniFormat);

    piezo_speed = settings_double(settings, "piezo/step", 1e-1);
    speeds[0] = speeds[1] = speeds[2] = 1e-1;
    speeds[3] = 1; speeds[4] = 2; speeds[5] = 5; speeds[6] = 10;
    for (int i = 0; i < 7; i++)
        speeds[i] = settings_double(settings, "piezo/scale"
                                   + QString::number(i), speeds[i]) /
                (1000.0/settings_double(settings, "piezo/delay", 50));

    QTimer *piezo_shuttle_timer = new QTimer(this);
    connect(piezo_shuttle_timer, SIGNAL(timeout()), this, SLOT(shuttle_position_piezo_timeout()));
    piezo_shuttle_timer->start(settings_double(settings, "piezo/delay", 50));

    connect(Shuttle, SIGNAL(button_press(int)), shuttle_gui, SLOT(button_press_slot(int)));
    connect(Shuttle, SIGNAL(wheel_event(int)), shuttle_gui, SLOT(wheel_event_slot(int)));
    connect(Shuttle, SIGNAL(shuttle_position(int)), shuttle_gui, SLOT(shuttle_position_slot(int)));

    ShuttleControllerNative *hidinput = new ShuttleControllerNative(this);
    connect(hidinput, SIGNAL(button_press(int)), shuttle_gui, SLOT(button_press_slot(int)));
    connect(hidinput, SIGNAL(wheel_event(int)), shuttle_gui, SLOT(wheel_event_slot(int)));
    connect(hidinput, SIGNAL(shuttle_position(int)), shuttle_gui, SLOT(shuttle_position_slot(int)));

    connect(shuttle_gui, SIGNAL(button_press(int)), this, SLOT(shuttle_button(int)));
    connect(shuttle_gui, SIGNAL(wheel_event(int)), this, SLOT(shuttle_wheel(int)));
    connect(shuttle_gui, SIGNAL(shuttle_position(int)), this, SLOT(shuttle_position(int)));
}
