#include "spmtunning.h"
#include "defs.h"

#include "mainwindow.h"
#include "nstslider.h"
#include "EG3000.h"
#include "ccd/ccdplot.h"

#include <QCheckBox>
#include <QFormLayout>
#include <QSettings>
#include <QTimer>
#include <QApplication>
#include <QDialog>
#include <QProgressBar>


extern USB_WRITE_THREAD *_wThread; //from modules.cpp


typedef QPair<int, int> FBinfo;
Q_DECLARE_METATYPE(FBinfo)


//scanplot
void init_reverse_nlc_params(float *Scanner_data_pid_cal_range);
float nonlinear_c_to_mkm_z(float c, float *Scanner_data_pid_cal_range);

void SPMTunning::sl_get_data(int ch, long val)
{
    if (cz_updated)
        return;

    if ((CBFBScanner->currentIndex() == 0) ? ch == srcCZ : ch == srcCZ2) {
        double zsp = val / 65535.0 * Zrange;

        bool nlc_on = *_wThread->Nlc_status_cl_z;
        if (ch == srcCZ2)
            nlc_on = *_wThread->Nlc_status_cl_z2;
        if (nlc_on) {
            float *Scanner_data_pid_cal_range = _wThread->Scanner_data_pid_cal_range;
            if (ch == srcCZ2)
                Scanner_data_pid_cal_range = _wThread->Scanner2_data_pid_cal_range;

            init_reverse_nlc_params(Scanner_data_pid_cal_range);
            zsp = nonlinear_c_to_mkm_z(val, Scanner_data_pid_cal_range) * 1e6;
        }
        NS_Z_spectr->setValue(zsp);
        cz_updated = true;
    }
}

void SPMTunning::s_spectr_start(bool st)
{
    Spectr_Scan_On = st;
    bool jm_on = st && BFeedbackOnOff->on_off && BFeedbackOnOffMode->on_off && BSpectrJM->on_off;

    destructStatus(Spect_progBar);
    if (st) {
        Spect_progBar = new StatusBar(tr("SPM Z scan"), jm_on ? 0 : 10000);
        Spect_progBar->bar->setFormat("%p%");
        connect(Spect_progBar, SIGNAL(canceled()), AB_Spectr_start_stop, SLOT(click()));
        Spect_progBar->add();
    }

    bool cl_fix = false;
    if (CBFBScanner->currentIndex() == 0){
        cl_fix = *_wThread->CL_status_z && BFeedbackOnOff->on_off;
    } else {
        cl_fix = *_wThread->CL_status_z2 && BFeedbackOnOff->on_off;
    }
    if (jm_on) cl_fix = false;
    if (st && cl_fix) {
        cz_updated = false;
        _wThread->sl_get_curr_cz();

        QElapsedTimer t;
        t.start();
        while (!cz_updated && t.elapsed() < 2000) {
            qApp->processEvents();
            SleepExport::msleep2(10);
        }

        cz_updated = true;
        if (t.elapsed() >= 2000) {
            new Notification(tr("SPM Z scan") + ": " + tr("Can't get Cz"), Notification::Bad, 10);
            doBeep();
            AB_Spectr_start_stop->change_state_man(false);
            destructStatus(Spect_progBar);
            return;
        }
    }

    double z = (BFeedbackOnOff->on_off && !cl_fix) ? Z_scanner : NS_Z_spectr->value();
    int Nx = CB_Points_spectr->currentText().toInt();

    Z1spectr = z - Z1;
    Z2spectr = z + Z2;
    if (st) {
        emit sig_spectroscopy(jm_on ? 2 : 1, Z1spectr, Z2spectr, ev_Speed_spectr->value()/1000, CB_Channel->itemData(CB_Channel->currentIndex()).toInt(), Nx);
        int div = (Z1 * 2 + Z2) * 2;
        if (div <= 0) div = 1;
        progDelta = ev_Speed_spectr->value() / div;
        Progress_timer->start(100);
    } else {
        Progress_timer->stop();
        emit sig_spectroscopy(0, Z1spectr, Z2spectr, ev_Speed_spectr->value()/1000, CB_Channel->itemData(CB_Channel->currentIndex()).toInt(), Nx);
    }
}
void SPMTunning::s_spectr_jm(bool st)
{
    ev_Speed_spectr->setEnabled(!st);
    NS_Z1_spectr->setEnabled(!st);NS_Z1_spectr->update();
    ev_Z1_spectr->setEnabled(!st);
    NS_Z2_spectr->setEnabled(!st);
}

void SPMTunning::s_updSpectrProgBar()
{
    if (!Spect_progBar) //|| !Spectr_Scan_On || !AB_Spectr_start_stop->on_off
        return;

    if (Spect_progBar->bar->maximum() && Spect_progBar->bar->value() < Spect_progBar->bar->maximum())
        Spect_progBar->setValue(qMin(Spect_progBar->bar->value() + progDelta, Spect_progBar->bar->maximum()));
    else {
        Progress_timer->stop();
        Spect_progBar->bar->setMaximum(0);
    }
}

void SPMTunning::s_setForceCurveData(QByteArray data)
{
    curveData_force[0]->clear();
    curveData_force[1]->clear();

    EG_ForceCurveData *sdata = (EG_ForceCurveData *)data.constData();
    int size = qMin<int>(FORCE_CURVE_LINE_SIZE, sdata->Size);
    bool jm_on = BFeedbackOnOff->on_off && BFeedbackOnOffMode->on_off && BSpectrJM->on_off;
    if (!Spectr_Scan_On || jm_on) { //jm
        Z1spectr = 0;
        Z2spectr = qMax(0.1, DCfbPrec->value() * JMdiv->value()) * size * 1e-3;
    }

    curveData_force[0]->setXCalib(Z1spectr * 1e-6, (Z2spectr - Z1spectr) / double(size - 1) * 1e-6);
    curveData_force[1]->setXCalib(Z1spectr * 1e-6, (Z2spectr - Z1spectr) / double(size - 1) * 1e-6);
    curveData_force[0]->YSeq.resize(0);
    curveData_force[0]->YSeq.reserve(size);
    curveData_force[1]->YSeq.resize(0);
    curveData_force[1]->YSeq.reserve(size);
    for (int i = 0; i < size; i++) {
        curveData_force[0]->AddY(input_calib(sdata->Source, sdata->Data[i], (CBFBScanner->currentIndex() ? Zrange2 : Zrange1)));
        curveData_force[1]->AddY(input_calib(sdata->Source, sdata->DataBack[i], (CBFBScanner->currentIndex() ? Zrange2 : Zrange1)));
    }

    Progress_timer->stop();
    AB_Spectr_start_stop->change_state_man(false);
    destructStatus(Spect_progBar);

    if (PlotMode == SpmSpectroscopy) {
        MyPl->Pure2dPlot->YAxisUnit.SetDescAndSIUnit(CB_Channel->currentText(), input_units(CB_Channel->itemData(CB_Channel->currentIndex()).toInt()));
        MyPl->Refresh();
        if (MyPl->Pure2dPlot->QwtCurves.size() >= 2) {
            MyPl->Pure2dPlot->QwtCurves[0]->Curve.setTitle(tr("Approach"));
            MyPl->Pure2dPlot->QwtCurves[1]->Curve.setTitle(tr("Retract"));
        }

        if (Spectr_Scan_On && AutoSend) {
            MyPl->on_SyncTimer_timeout();
            MyPl->PlotControl->BSave->animateClick();
        }

        if (Spectr_Scan_On)
            doBeep();
    }

    Spectr_Scan_On = false;
}


void SPMTunning::s_FreqScanStartAuto()
{
    double mino = EFreqScanMin->value();
    double maxo = EFreqScanMax->value();
    bool set = freqScanSet->isChecked();
    freqScanSet->setChecked(true);

    double minf0 = lowPassFilter->value() * 1.1 + lowPassFilter->edit->minimum() * 1.5;
    double minf = minf0;
    double maxf = curFrequency->edit->maximum() * 0.9;
    double prev_freq;
    int check_low_freq = 0;
    bool last = false;

redo:
    EFreqScanMin->setValue(minf);
    EFreqScanMax->setValue(maxf);

    //measure resonance
    PBFreqStart->change_state_man_with_sig(true);
    QElapsedTimer t; t.start();
    bool progbar_cancel = false;
    while (PBFreqStart->on_off && ModuleManager::instance->isRunning && t.elapsed() < 10000 && !(progbar_cancel = !progBar)) {
        qApp->processEvents();
        SleepExport::msleep2(10);
    }

    if (!ModuleManager::instance->isRunning || t.elapsed() >= 10000 || progbar_cancel) {
exit:
        EFreqScanMin->setValue(mino);
        EFreqScanMax->setValue(maxo);
        freqScanSet->setChecked(set);

        new Notification(tr("Frequency scan") + ": " + tr("Can't find peak"), Notification::Warn, 10);
        doBeep();
        return;
    }


    if (((curFrequency->value() - minf) < 0.1 * (maxf - minf) && !check_low_freq) ||
            (check_low_freq && qAbs(prev_freq - curFrequency->value()) > 0.01 * curFrequency->value())) {
        prev_freq = curFrequency->value();
        check_low_freq++;
        if (check_low_freq > 3)
            goto exit;
        goto redo;
    }
    check_low_freq = 0;


    double resfreq = curFrequency->value() * 1e3; //todo fit?
    int residx = 0;
    do {
        double x = curveData_freq.at(0)->XSeq.at(0) + residx * curveData_freq.at(0)->XSeq.at(1);
        if (x > resfreq)
            break;
        residx++;
    } while (residx < curveData_freq.at(0)->size());
    residx = qMax(0, residx-1);

    double Vres = curveData_freq.at(0)->YSeq.at(residx); //todo better fit
    int peakrange = qMax<int>(1, resfreq / 2000 / curveData_freq.at(0)->XSeq.at(1));
    for (int i = qBound(0, residx - peakrange, curveData_freq.at(0)->size()-1);
         i <= qBound(0, residx + peakrange, curveData_freq.at(0)->size()-1); i++)
        Vres = qMax(Vres, curveData_freq.at(0)->YSeq.at(i));
    double Vnoise = min(curveData_freq.at(0)->YSeq);
    if (Vres / Vnoise < 2)
        goto exit;
    if (Vnoise < Vres / 3) Vnoise = 0; //ignore low background noise variation - more stable q factor

    QVector<bool> dir; QVector<int> pos;
    ccd::GetIntersections(curveData_freq.at(0)->YSeq, 0, curveData_freq.at(0)->YSeq.size(), 0.7*(Vres - Vnoise) + Vnoise, dir, pos);
    if (pos.size() < 2)
        goto exit;
    int resintidx = 0;
    while (pos.at(resintidx) < residx && resintidx < pos.size()-1) resintidx++;
    int resintidx0 = resintidx;
    while (!dir.at(resintidx0) && resintidx0 > 0) resintidx0--;
    while (dir.at(resintidx) && resintidx < pos.size()-1) resintidx++;
    if (!dir.at(resintidx0) || dir.at(resintidx))
        goto exit;
    double deltafreq = (pos.at(resintidx) - pos.at(resintidx0)) * curveData_freq.at(0)->XSeq.at(1);
    double qfactor = resfreq / deltafreq;
    if (qfactor < 10)
        goto exit;

    double basefreq = deltafreq * 6.5; //gaussian at 0.1A base width

    if (!last) {
        const double width_percent = 0.2;
        double width = (maxf - minf) * 1e3;
        width /= 2.7;
        if (basefreq > width_percent * width)
            last = true;
        double minfn = (resfreq - width / 2) / 1e3;
        double maxfn = (resfreq + width / 2) / 1e3;
        if (minfn < minf0) {
            maxfn += minf0 - minfn;
            minfn = minf0;
        }
        minf = minfn;
        maxf = maxfn;
        goto redo;
    }

    doBeep();
}

void SPMTunning::s_FreqScanStart(bool st)
{
    PBFreqStart->setEnabled(!st);
    PBFreqStartAuto->setEnabled(!st);
    destructStatus(progBar);
    if (st) {
        emit sig_sendFrequencyRange(EFreqScanMin->value()*1000.0, EFreqScanMax->value()*1000.0, CB_fr_Channel->itemData(CB_fr_Channel->currentIndex()).toInt());

        progBar = new StatusBar(tr("Frequency scan"), 10000);
        progBar->bar->setFormat("%p%");
        //connect(progBar, SIGNAL(canceled()), PBFreqStart, SLOT(click()));
        progBar->add();
        progDelta = 200;
        if (l2k_controller)
            progDelta *= 3;
        Progress_timer->start(100);
        sl_save_to_file();
    } else
        Progress_timer->stop();
}
void SPMTunning::s_updFreqProgBar()
{
    if (!PBFreqStart->on_off || !progBar)
        return;

    if (progBar->bar->maximum() && progBar->bar->value() < progBar->bar->maximum())
        progBar->setValue(qMin(progBar->bar->value() + progDelta, progBar->bar->maximum()));
    else {
        Progress_timer->stop();
        progBar->bar->setMaximum(0);
    }
}

void SPMTunning::s_setFrequencyScanData(QByteArray data)
{
    curveData_freq[0]->clear();

    EG_LockInScanData *sdata = (EG_LockInScanData *)data.constData();
    int size = qMin<int>(LOCKIN_LINE_SIZE, sdata->Size);

    curveData_freq[0]->setXCalib(EFreqScanMin->value() * 1e3, (EFreqScanMax->value() - EFreqScanMin->value()) / double(size - 1) * 1e3);
    curveData_freq[0]->YSeq.resize(0);
    curveData_freq[0]->YSeq.reserve(size);
    for (int i = 0; i < size; i++)
        curveData_freq[0]->AddY(input_calib(sdata->Source, sdata->Data[i], (CBFBScanner->currentIndex() ? Zrange2 : Zrange1)));

    if (freqScanSet->isChecked()) {
        double value = ((float)((unsigned long)(sdata->res_point * 1000))) / 1000000.;

        curFrequency->setValue(value);
        MobileServer::sendData(DataID::SPM_freq, value);

        MyPl->Pure2dPlot->MoveVertLine(value * 1000, 0);
    } else {
        curFrequency->setNewValue();
    }

    if (PlotMode == ResCurveMode) {
        MyPl->Pure2dPlot->YAxisUnit.SetDescAndSIUnit(CB_fr_Channel->currentText(), input_units(CB_fr_Channel->itemData(CB_fr_Channel->currentIndex()).toInt()));
        MyPl->Refresh();
    }

    PBFreqStart->change_state_man_with_sig(false);
}


/*
void SPMTunning::doVoltageScan()
{
    setEnabled(false);
    QVector<double> y;
    double drive1 = driveInt->value();
    for (int i = 0; i < 100; i++) {
        driveInt->setValue(drive1/100*i);
        driveInt->setNewValue();
        QElapsedTimer t;t.start();
        while (t.elapsed() < 10) {
            qApp->processEvents();
            SleepExport::msleep2(2);
        }
        double v;
        _mainw->osc->acp->read_osc(&v, srcMag1);
        y << v;
    }

    curveData_freq[0]->clear();

    curveData_freq[0]->setXCalib(0, drive1/100);
    curveData_freq[0]->YSeq.resize(0);
    curveData_freq[0]->YSeq.reserve(y.size());
    for (int i = 0; i < y.size(); i++)
        curveData_freq[0]->AddY(input_calib(srcMag1, y.at(i)));

    MyPl->Refresh();

    setEnabled(true);
    return;
}
*/


void SPMTunning::doForceCalib()
{
    int trygain = gain1->count(); //todo config?
redo:
    trygain--;
    if (trygain < 0) {
        new Notification(tr("Frequency scan") + ": " + tr("Can't find peak"), Notification::Warn, 10);
        doBeep();
        return;
    }

    setEnabled(false);

    int previdx = fbs->currentIndex();
    for (int i = 0; i < fbs->count(); i++)
        if (srcMag1 == fbs->itemData(i).value<QPair<int, int> >().second) {
            fbs->setCurrentIndex(i);
            break;
        }

    double drive = driveInt->value();
    driveInt->setValue(0);
    driveInt->setNewValue();
    int gainidx = gain1->currentIndex();
    gain1->setCurrentIndex(trygain);
    CBcalibDFL_F->setChecked(false);
    freqScanSet->setChecked(false);
    double qc = EQControlGain->value();
    EQControlGain->setValue(0);
    EQControlGain->setNewValue();


    //measure thermal noise spectrum
    PBFreqStart->change_state_man_with_sig(true);
    QElapsedTimer t; t.start();
    bool progbar_cancel = false;
    while (PBFreqStart->on_off && ModuleManager::instance->isRunning && t.elapsed() < 10000 && !(progbar_cancel = !progBar)) {
        qApp->processEvents();
        SleepExport::msleep2(10);
    }

    double gain = 1e-9 / gain1->itemData(gain1->currentIndex()).toDouble();
    gain1->setCurrentIndex(gainidx);
    driveInt->setValue(drive);
    driveInt->setNewValue();
    fbs->setCurrentIndex(previdx);
    freqScanSet->setChecked(true);
    EQControlGain->setValue(qc);
    EQControlGain->setNewValue();
    setEnabled(true);

    if (!ModuleManager::instance->isRunning || t.elapsed() >= 10000 || progbar_cancel)
        return;


    //fit/set model params
    QSettings settings("Param/gui.conf", QSettings::IniFormat);
    double width = 1e-6 * settings_double(settings, "spm/force_calib/tip_width", 30);
    double length = 1e-6 * settings_double(settings, "spm/force_calib/tip_length", 125);

    double resfreq = curFrequency->value() * 1e3; //todo fit?
    int residx = 0;
    do {
        double x = curveData_freq.at(0)->XSeq.at(0) + residx * curveData_freq.at(0)->XSeq.at(1);
        if (x > resfreq)
            break;
        residx++;
    } while (residx < curveData_freq.at(0)->size());
    residx = qMax(0, residx-1);

    double Vres = curveData_freq.at(0)->YSeq.at(residx); //todo better fit
    int peakrange = qMax<int>(1, resfreq / 2000 / curveData_freq.at(0)->XSeq.at(1));
    for (int i = qBound(0, residx - peakrange, curveData_freq.at(0)->size()-1);
         i <= qBound(0, residx + peakrange, curveData_freq.at(0)->size()-1); i++)
        Vres = qMax(Vres, curveData_freq.at(0)->YSeq.at(i));
    double Vnoise = min(curveData_freq.at(0)->YSeq);
    if (Vres / Vnoise < 2)
        goto redo;
    if (Vnoise < Vres / 3) Vnoise = 0; //ignore low background noise variation - more stable q factor
    double Pres = pow(Vres, 2) - pow(Vnoise, 2); //need to normalize by qfactor^2 later
    Pres /= resfreq; //passband normalization?
    if (l2k_controller) //l2k different gain for low voltage?
        Pres *= 6.2*6.2;

    QVector<bool> dir; QVector<int> pos;
    ccd::GetIntersections(curveData_freq.at(0)->YSeq, 0, curveData_freq.at(0)->YSeq.size(), 0.7*(Vres - Vnoise) + Vnoise, dir, pos);
    if (pos.size() < 2)
        goto redo;
    int resintidx = 0;
    while (pos.at(resintidx) < residx && resintidx < pos.size()-1) resintidx++;
    int resintidx0 = qMax(0, resintidx-1);
    while (!dir.at(resintidx0) && resintidx0 > 0) resintidx0--;
    while (dir.at(resintidx) && resintidx < pos.size()-1) resintidx++;
    if (!dir.at(resintidx0) || dir.at(resintidx))
        goto redo;
    double deltafreq = (pos.at(resintidx) - pos.at(resintidx0)) * curveData_freq.at(0)->XSeq.at(1);
    double qfactor = resfreq / deltafreq;
    if (qfactor < 10)
        goto redo;

    Pres = Pres / qfactor / qfactor; //normalize


    //some calculus
    const double air_density = 1.18;
    const double air_viscosity = 1.86e-5;
    const double water_density = 997.7;
    const double water_viscosity = 9.98e-4;
    bool in_liquid = settings.value("spm/force_calib/in_liquid", false).toBool();
    double calib_density = in_liquid ? water_density : air_density;
    double calib_viscosity = in_liquid ? water_viscosity : air_viscosity;
    double reynolds = calib_density * width * width * (2 * M_PI * resfreq) / 4 / calib_viscosity;
    double a0 = 0.9;//0.8-1.0
    bool triangle_tip = settings.value("spm/force_calib/tip_triangular", false).toBool();
    if (triangle_tip)
        a0 = 1.3; //1.2-1.5
    double a1 = -0.7;
    double a2 = 0.035 * log(reynolds);
    double somefluffybullshit = a0 * pow(reynolds, a1 + a2); //0.1906* imaginary part of hydrodynamic function at res freq...
    double k = calib_density * width * width * length * qfactor * (2 * M_PI * resfreq) * (2 * M_PI * resfreq) * somefluffybullshit; //generalised sader 2012
    qDebug() << "k Q Re";
    qDebug() << k << qfactor << reynolds;

    const double kbolz = 1.380648813e-23;
    const double Temp = 273.16 + 22;
    double invols = sqrt(2 * kbolz * Temp / M_PI / k / resfreq / Pres / qfactor); //equipartition theorem - afm ols 2006
    qDebug() << invols;


    //apply
    double voltagetoforce = invols * k / gain;
    qDebug() << voltagetoforce;
    if (voltagetoforce > 0.1) {
        EcalibDFL_F->setValue(voltagetoforce);
        EcalibDFL_F->setNewValue();
        CBcalibDFL_F->setChecked(true);
    }

    doBeep();
}
void SPMTunning::doForceCalibConfig()
{
    QSettings settings("Param/gui.conf", QSettings::IniFormat);

    QDialog *d = new QDialog;
    d->setWindowTitle(tr("Force calib settings"));
    QFormLayout *lay = makeFormLayout(d);
    Edit *width;
    lay->addRow(tr("Tip width, ")+tr("micro")+tr("m"), width = new Edit(0, 1000, 1));
    Edit *length;
    lay->addRow(tr("Tip length, ")+tr("micro")+tr("m"), length = new Edit(0, 1000, 1));
    QCheckBox *triangular;
    lay->addRow(triangular = new QCheckBox(tr("Triangular tip")));
    QCheckBox *liquid;
    lay->addRow(liquid = new QCheckBox(tr("In liquid")));

    if (show_advanced_gui()) {
        lay->addItem(new QSpacerItem(0, 0));
        lay->addRow(new QLabel("References: Sader et al."));
        QLabel *l;
        lay->addRow(l = new QLabel("<a href='http://dx.doi.org/10.1063/1.2162455'>Determination of optical lever sensitivity...</a>"));
        l->setOpenExternalLinks(true);
        lay->addRow(l = new QLabel("<a href='http://dx.doi.org/10.1063/1.4757398'>Spring constant calibration...</a>"));
        l->setOpenExternalLinks(true);
    }

    double widthv = settings_double(settings, "spm/force_calib/tip_width", 30);
    double lengthv =settings_double(settings, "spm/force_calib/tip_length", 125);
    bool in_liquid = settings.value("spm/force_calib/in_liquid", false).toBool();
    bool triangle_tip = settings.value("spm/force_calib/tip_triangular", false).toBool();
    width->setValue(widthv);
    length->setValue(lengthv);
    liquid->setChecked(in_liquid);
    triangular->setChecked(triangle_tip);

    d->exec();

    if (widthv != width->value())
        settings.setValue("spm/force_calib/tip_width", width->value());
    if (lengthv != length->value())
        settings.setValue("spm/force_calib/tip_length", length->value());
    if (triangle_tip != triangular->isChecked())
        settings.setValue("spm/force_calib/tip_triangular", triangular->isChecked());
    if (in_liquid != liquid->isChecked())
        settings.setValue("spm/force_calib/in_liquid", liquid->isChecked());

    d->deleteLater();
}


void SPMTunning::createSpectroscopyWorkSpace()
{
    AB_Spectr_start_stop = new TwoStateButton(":start.png",":stop.png");
    connect(AB_Spectr_start_stop, SIGNAL(state_changed(bool)), this, SLOT(s_spectr_start(bool)));

    BSpectrJM = new TwoStateButton(":/SPMTuning_window/JmOf.png", ":/SPMTuning_window/JmOn.png");
    BSpectrJM->setToolTip(tr("Get data from JM feedback, when both are enabled"));
    connect(BSpectrJM, SIGNAL(state_changed(bool)), this, SLOT(s_spectr_jm(bool)));

    JMdiv = new Edit(1, 255);

    NS_Z1_spectr = new NSTSlider(Zrange, 0, 4);
    connect(NS_Z1_spectr, SIGNAL(valueChanged(double)), this, SLOT(s_set_Z1_spectr(double)));
    connect(NS_Z1_spectr, SIGNAL(sliderReleased()), this, SLOT(sl_up_lines_on()));
    connect(NS_Z1_spectr, SIGNAL(sliderPressed()), this, SLOT(sl_up_lines_of()));

    ev_Z1_spectr = new Edit(0, 500, 4);
    connect(ev_Z1_spectr, SIGNAL(newValue(double)),NS_Z1_spectr, SLOT(setValue(double)));
    connect(ev_Z1_spectr, SIGNAL(editingFinished()), this, SLOT(sl_up_lines_on()));
    connect(ev_Z1_spectr, SIGNAL(editingStarted()),    this, SLOT(sl_up_lines_of()));

    NS_Z2_spectr = new DoubleChanger(0, Zrange, 4);
    connect(NS_Z2_spectr->slider, SIGNAL(valueChanged(double)), this, SLOT(s_set_Z2_spectr(double)));
    connect(NS_Z2_spectr, SIGNAL(newValue(double)), this, SLOT(s_set_Z2_spectr(double)));
    connect(NS_Z2_spectr, SIGNAL(editingFinished()), this, SLOT(sl_up_lines_on()));
    connect(NS_Z2_spectr, SIGNAL(editingStarted()), this, SLOT(sl_up_lines_of()));

    NS_Z_spectr = new DoubleChanger(0, Zrange, 4);
    connect(NS_Z_spectr->slider, SIGNAL(valueChanged(double)), this, SLOT(s_set_Z_spectr(double)));
    connect(NS_Z_spectr, SIGNAL(newValue(double)), this, SLOT(s_set_Z_spectr(double)));
    connect(NS_Z_spectr, SIGNAL(newValue()), this, SLOT(s_send_Zpos()));
    connect(NS_Z_spectr, SIGNAL(editingFinished()), this, SLOT(sl_up_lines_on()));
    connect(NS_Z_spectr, SIGNAL(editingStarted()), this, SLOT(sl_up_lines_of()));

    CB_Points_spectr = new QComboBox;
    CB_Points_spectr->addItem("50");  CB_Points_spectr->addItem("100");
    CB_Points_spectr->addItem("200"); CB_Points_spectr->addItem("300");
    CB_Points_spectr->addItem("400"); CB_Points_spectr->addItem("500");
    CB_Points_spectr->addItem("600"); CB_Points_spectr->addItem("700");
    CB_Points_spectr->addItem("800"); CB_Points_spectr->addItem("900");
    CB_Points_spectr->addItem("1000");

    CB_Channel = new QComboBox;
    fillCBchannels_(CB_Channel, false);

    ev_Speed_spectr = new Edit(100, 10000, -2);
    ev_Speed_spectr->setImmediateMode();
    ev_Speed_spectr->setValue(100);
    NS_Z1_spectr->setValue(Z1);
    NS_Z2_spectr->setValue(Z2);
}

void SPMTunning::createFreqWorkSpace()
{
    CB_fr_Channel = new QComboBox;
    fillCBchannels_(CB_fr_Channel, false);
    CB_fr_Channel->setCurrentIndex(CB_fr_Channel->findData(srcMag1));

    EFreqScanMin = new Edit(0, 20000, 1);
    EFreqScanMin->setImmediateMode();
    EFreqScanMin->setValue(50);

    EFreqScanMax = new Edit(0, 20000, 1);
    EFreqScanMax->setImmediateMode();
    EFreqScanMax->setValue(500);

    PBFreqStart = new TwoStateButton(":start.png",":stop.png");
    PBFreqStart->setProperty("id", DataID::SPM_freq_scan);
    connect(PBFreqStart, SIGNAL(state_changed(bool)), this, SLOT(s_FreqScanStart(bool)));

    PBFreqStartAuto = new QPushButton(makeHugeIcon(":AutoStart.png"), "");
    PBFreqStartAuto->setToolTip(tr("Auto freq scan"));
    connect(PBFreqStartAuto, SIGNAL(clicked()), this, SLOT(s_FreqScanStartAuto()));

    PBForceCalib = new QPushButton(tr("Calibrate Force"));
    connect(PBForceCalib, SIGNAL(clicked()), this, SLOT(doForceCalib()));
    if (show_advanced_gui())
        PBForceCalib->setToolTip(tr("See settings window for reference papers information."));

    PBForceCalibConfig = new QPushButton(makeHugeIcon(":/MainToolbar/tools.png"), "");
    connect(PBForceCalibConfig, SIGNAL(clicked()), this, SLOT(doForceCalibConfig()));
}
