#ifndef SPMTUNNING_H
#define SPMTUNNING_H

#include "defs.h"
#include <QWidget>
#include <QPointer>
class QCheckBox;
#include <plot.h>
#include "controls.h"
#include "edit.hpp"

class StatusBar;
class Notification;
class TOptionsForMotor;
class DeflectWid;
class ShuttleGuiControl;
class ShuttleController;
class Motors;
class temp_control;
class NSTSlider;
class StatusWidget1;
class TTabWidget;
class SPMTunning: public QWidget
{
    Q_OBJECT

    enum plot_mode{ResCurveMode, FbSignalOscMod, SpmSpectroscopy, TempMode};

public slots:
    void capBits(int, long);
    void reset();


    void sl_save_to_file();
    void sl_load_from_file();
    void setFBchannel();
    void s_setFBOnOff(bool state);
    void s_setFBMode(bool state);
    void sl_laser_on_of(bool);
    void s_setFBScanner(int idx);

    void setSPgui();
    void s_sendFBP();
    void s_sendFBI();
    void s_sendSP();
    void s_jm_prec();
    void s_jm_height();

    void s_set_Kelvin(bool);
    void s_set_efmplus(bool);
    void s_setCalibLFI();
    void s_setCalibDFL();
    void s_Imode();

    void s_sendGains();
    void s_sendGains2();
    void s_sendDriveInt();

    void s_FreqScanStartAuto();
    void s_FreqScanStart(bool st);
    void s_setFrequencyScanData(QByteArray data);
    void s_setCurFrequency(double value);
    void s_updFreqProgBar();

    void sl_get_data(int ch, long val);
    void s_spectr_start(bool);
    void s_spectr_jm(bool);
    void s_setForceCurveData(QByteArray data);
    void s_updSpectrProgBar();

    void set_spot_data(double zpos, double dfl, double lfl, double mag, double int_);
    void set_spot_data_handler();
    void doForceCalib();
    void doForceCalibConfig();

    void s_set_Z1_spectr(double value);
    void s_set_Z2_spectr(double value);
    void s_set_Z_spectr(double value);
    void s_send_Zpos();
    void Zpos_on_off(bool);
    void sl_change_z_slider();
    void sl_set_XYZ_range(double x, double y, double z);
    void sl_set_XYZ_range2(double x, double y, double z);
    void sl_set_XYZ_range_sync();
    void sl_up_lines_on();
    void sl_up_lines_of();

    void shuttle_button(int num);
    void shuttle_wheel(int num);
    void shuttle_position(int num);
    void shuttle_position_piezo_timeout();

    void motors_stop();
    void slot_m123stopped();
    void slot_DoMoveUp();
    void slot_DoFastMoveUp();
    void slot_DoMoveDown();
    void slot_DoFastMoveDown();
    void slot_DoLanding(bool);
    void slot_set_Landing_status(int);
    void blanding_sync();
    void s_updLandProgBar();
    void s_landing_m4(bool);

    void sl_setPlotMode();
    void switchMode(int idx);

    void osc_refresh();
    void sl_set_osc_T(int);
    void sl_HVtoZero();

    void AddDataToContainer_slot(plot::TCurvesDataOwner, plot::TMetaStorage);
    void appendAttrib(plot::TMetaStorage &);

signals:
    void AddDataToContainer(plot::TCurvesDataOwner, plot::TMetaStorage);

    void sig_osc_channel(int);
    void sig_osc_start();
    void sig_osc_stop();
    void sig_setFBOnOff(unsigned char code);
    void sig_FbChannel(int);
    void sig_sendFBP(double value);
    void sig_sendFBI(double value);
    void sig_sendSP(double value);
    void jm_prec(WORD msb, WORD lsb);
    void jm_height(DWORD val);
    void sig_sendData(double *data, double xMin, double yMin, double xMax, double yMax, double min, double max,
                      QString author, QString name, QString date, QString id, QPixmap &pixmap,
                      QString xAxis, QString yAxis, QString zAxis, QString units,
                      unsigned int numOfX, unsigned int numOfY);


    void sig_sendGains(int gain1, int gain2, int gain3);
    void sig_sendGains2(int gain1, int gain2, int gain3);
    void sig_sendDriveInt(double value);
    void sig_sendFrequencyRange(double from, double to, int channel);
    void sig_Zpos(double);
    void sig_Z2pos(double);

    void Landing_Data(BYTE, BYTE, double, double, double);
    void m4landing(int);

    void sig_osc_mode(bool);
    void sig_spectroscopy(int mode_on, double z1, double z2, double speed, int channel, int points);
    void sig_Head_data(int, int, int);

    void input_calib_updated();

public:
    temp_control *tcontrol;
    QWidget *RW;
    QTimer *Progress_timer;
    QTimer *laserSpotSync;

    QGroupBox *GrBMotorControl;

    TwoStateButton *BLanding;
    TwoStateButton *BLandingM4;
    TwoStateButton *BLaserOnOff;
    QPushButton *BMotors_leveling;
    QTabBar *modeBar;

    Edit *Kelvin_P;
    QCheckBox *Kelvin_on;
    Edit *Edfl_xtalk;
    Edit *Emag_xtalk;
    QGroupBox *box_qcontrol;
    Edit *EQControlGain;
    bool l2k_controller;

    QCheckBox *CBcalibLF_I;
    Edit *EcalibLF_I;
    QCheckBox *CBcalibDFL_I;
    Edit *EcalibDFL_I;
    QCheckBox *CBcalibDFL_F;
    Edit *EcalibDFL_F;

    TTabWidget *spmTabs;
    TTabWidget *spmTabs2;

    DoubleChanger *curFrequency2;
    DoubleChanger *driveInt2;
    DoubleChanger *Vtip;
    DoubleChanger *Vtip2;

    QCheckBox *piezoTip;
    QCheckBox *capMode;
    QCheckBox *piezoTip2;
    QCheckBox *capMode2;
    QCheckBox *efmSinglePass;

    QCheckBox *lflCurrent;
    QCheckBox *extCurrent;

    QPointer<StatusBar> landing_bar;
    QPointer<Notification> landing_notification;
    bool blanding_status;
    QTimer *blanding_syncT;

    QComboBox *gain1;
    QComboBox *gain2;
    QComboBox *gain3;

    QComboBox *gain12;
    QComboBox *gain22;
    QComboBox *gain32;

    QComboBox *CB_fr_Channel;

    QGroupBox *grb_Av_rms;
    QGroupBox *box_scan;

    Edit *SBT;
    Edit *ed_Aver;
    Edit *ed_RMS;
    QLabel *Lrms;
    QLabel *Laver;

    //=======Spectroscopy======================

    Edit *ev_Z1_spectr;
    Edit *ev_Speed_spectr;

    QComboBox *CB_Points_spectr;
    QComboBox *CB_Channel;
    TwoStateButton *AB_Spectr_start_stop;
    TwoStateButton *BSpectrJM;
    Edit *JMdiv;
    QLabel *LJMdiv;

    NSTSlider *NS_Z1_spectr;
    DoubleChanger *NS_Z2_spectr;

    QGroupBox *box_Spect_start;
    QGroupBox *box_Z2_spectr;
    QGroupBox *box_Z1_spectr;
    QGroupBox *box_Z_spectr;

    QPointer<StatusBar> Spect_progBar;

    //==============================================

    Motors *motors;
    ShuttleGuiControl *shuttle_gui;
    ShuttleController *Shuttle;
    int last_command;

    double piezo_speed;
    int piezo_shuttle;
    double speeds[7];

    void InitShuttleGuiLink();

    //==================================================

    DoubleChanger *curFrequency;
    DoubleChanger *lowPassFilter;
    DoubleChanger *phaseShift;
    DoubleChanger *driveInt;

    QPushButton *PBForceCalib;
    QPushButton *PBForceCalibConfig;
    TwoStateButton *PBFreqStart;
    QPushButton *PBFreqStartAuto;
    Edit *EFreqScanMin;
    Edit *EFreqScanMax;
    QCheckBox *freqScanSet;

    unsigned char Spectr_Scan_On;
    bool saveblock;
    bool AutoSend;

    void createFeedBackWorkSpace();
    void createLockInWorkSpace();
    void createFreqWorkSpace();
    void createSpectroscopyWorkSpace();
    void create_plot_mode_control();
    void InitMotorControlSpace();
    void createLayout();
    void closeEvent(QCloseEvent *);

public:
    DoubleChanger *DCfbP;
    DoubleChanger *DCfbI;
    DoubleChanger *DCfbSetPoint;
    QLabel *DCfbSetPointUnits;
    double spm_sp_units_gain, spm_sp_min, spm_sp_max;
    DoubleChanger *DCfbPrec;
    DoubleChanger *DCfbHeight;

    DoubleChanger *NS_Z_spectr;

    TwoStateButton *PBToNewWindow;
    TwoStateButton *B_PID_sign;

    DeflectWid *LaserSpotWindow;
    DeflectWid *LaserSpot;
    TOptionsForMotor *OptionsForMotor;
    QComboBox *fbs;
    QComboBox *CBFBScanner;

    double nDFLr, nLFLr;
    double nDFL, nLFL, nMAG, nInt, nZ;
    double xMin, xMax, yMin, yMax;

    double Mode_data[10][4];//0-setpoint; 1-fbp; 2-fbi; 3-drive;
    int    Gain_data[10][3];//0-g1; 1-g2; 2-g3;

    Range OscYrange;
    double osc_y_postscale;
    int PlotMode;

    TwoStateButton *BFeedbackOnOff;
    TwoStateButton *BFeedbackOnOffMode;
    int FB_sinal_num;
    QPointer<StatusWidget1> SWspmfb;
    int prev_spmfb_state;

    int LaserOnOf, AC_DC, Preamp;
    double Z1, Z2, Z1spectr, Z2spectr, Z_scanner, Zrange, Zrange1, Zrange2;
    bool cz_updated;
    bool Up_lines_spectr;
    int progDelta;
    QPointer<StatusBar> progBar;

    plot::TCurvesDataOwner curveData_freq;
    plot::TCurvesDataOwner curveData_force;

    plot::T2dPlot *MyPl;
    SPMTunning(QWidget *parent = 0);
    ~SPMTunning();

};


#endif //SPMTUNNING_H
