#ifndef XYZCALIBRATIONS_HPP
#define XYZCALIBRATIONS_HPP

#include <QWidget>
#include <QPicture>
#include <QPainter>
class QCheckBox;
class QComboBox;
class QTabWidget;
class QLabel;
#include "controls.h"
#include "edit.hpp"
#include "defs.h"

class NLCWidget;
class Oscilloscope;
class XYZCalibrations : public QWidget
{
    Q_OBJECT
public:
    XYZCalibrations(QWidget *parent = 0, Oscilloscope *osc = 0, bool second3030 = false);

signals:
    void program_controller(QString);

    void sig_XYZ_range(double,double,double);
    void sig_XYZ_range2(double,double,double);
    void sig_XYZ_range3(double,double,double);

    void sig_SentDataToEG3000();
    void sigUpdExCalibData();
    void sig_csb_speed(int);

    void input_calib_updated();

public slots:
    void nlcPIDLoad();
    void nlcPIDSave();
    void nlcPIDSend();

    void nlcEdit();
    void nlcEditRst();
    void nlcEditUpd();
    void nlcEditApply();

    void z3Save();
    void z3Load();
    void sl_Load_Extra_calib_from_File();
    void AutoDataLoad();
    void sl_Load_from_File();
    void read_calib_par(QString file, bool autoload = false);
    void save_To_File();
    void write_calib_par(QString file, bool need_old = true);
    void apply_calib_data();
    void sl_XYZ_range();
    void sl_setCalibVC();

    void s_sendData();
    void eeprom_received();
    void s_updEEPData();

    void lvmClicked();
    void s_setCLStatus(bool status);
    void s_setCLStatusZ(bool status);
    void set_nlc_status_cl(bool);
    void set_nlc_status_ol(bool);
    void CL_NLS_Lamps(int scanner);

    void sl_csb_speed(int idx);
    void capBits(int, long);
    void reset();
    void program_controller();

public:
    bool second_3030;
    Oscilloscope *osc;

    //Calibrations
    bool eeprom_first_load;
    bool skip_autocalib;
    QString defaultFileName;
    QString lastFileName;

    QCheckBox *CBlvm[2];
    QTabWidget *Scanner_type;
    QComboBox *CBSpeedCSB;

    QPushButton *PrintDebug;
    QPushButton *Save_to_Contr;
    QPushButton *Load_from_Contr;
    TwoStateButton *fb_on_off_xy;
    TwoStateButton *fb_on_off_z;
    QCheckBox *nl_on_off_cl;
    QCheckBox *nl_on_off_ol;
    QCheckBox *nl_on_off_cl2;
    QCheckBox *nl_on_off_ol2;

    //Extra calibration
    double adcCalib[16];
    double dacCalib[8];

    //-----------------------------------------------------------------
    bool Nlc_status_cl_xy;
    bool Nlc2_status_cl_xy;
    bool Nlc_status_cl_z;
    bool Nlc2_status_cl_z;

    bool Nlc_status_ol_xy;
    bool Nlc2_status_ol_xy;
    bool Nlc_status_ol_z;
    bool Nlc2_status_ol_z;

    bool CL_status_xy;
    bool CL2_status_xy;
    bool CL_status_z;
    bool CL2_status_z;
    bool CL3_status_z;

    float free_move_speed;
    float free_move_speed2;
    float free_move_speed3;

    //nlcPID
    QCheckBox *CBnlcPID[2];
    QCheckBox *CBnlcPIDz[2];
    double nlcPID[2][3][6]; //scanner, axis, calib
    BYTE nlcPIDon[2][3];
    bool nlcpid_loaded;

    //nlc edit
    DoubleEdit *lineNLCEdit[6];
    NLCWidget *viewNLCEdit;
    QWidget *NLCEditWid;
    QLabel *NLCEditValid;


    //xyz1
    DoubleEdit *lineFreeSpeed;
    DoubleEdit *linePID[3][3];

    DoubleEdit *lineScannerRangeOL[3];
    DoubleEdit *lineScannerRangeCL[3];
    DoubleEdit *lineSensorScale[3];
    DoubleEdit *lineSensorScale0[3]; //shift

    double linePiezoCalibration[3][6];
    double lineSensorCalibration[3][6];
    NLCWidget *viewPiezoCalibration[3];
    NLCWidget *viewSensorCalibration[3];

    float Scanner_data_pid_cal_range[48]; //don't change order of fields!! it's same as in EG_CalData
    DWORD Scanner_data_scale[3];
    long Scanner_data_Shift[3];

    //xyz2
    DoubleEdit *lineFreeSpeed2;
    DoubleEdit *linePID2[3][3];

    DoubleEdit *lineScannerRangeOL2[3];
    DoubleEdit *lineScannerRangeCL2[3];
    DoubleEdit *lineSensorScale2[3];
    DoubleEdit *lineSensorScale02[3]; //shift

    double linePiezoCalibration2[3][6];
    double lineSensorCalibration2[3][6];
    NLCWidget *viewPiezoCalibration2[3];
    NLCWidget *viewSensorCalibration2[3];

    float Scanner2_data_pid_cal_range[48]; //don't change order of fields!! it's same as in EG_CalData
    DWORD Scanner2_data_scale[3];
    long Scanner2_data_Shift[3];

    //Z3
    DoubleEdit *lineFreeSpeed3;
    DoubleEdit *linePIDProportionalZ3;
    DoubleEdit *linePIDIntegralZ3;

    DoubleEdit *lineScannerRangeOLZ3;
    DoubleEdit *lineScannerRangeCLZ3;
    DoubleEdit *lineSensorScaleZ3;
    DoubleEdit *lineSensorScaleZ03;

    float Scanner3_data_pid_cal_range[48]; //don't change order of fields!! it's same as in EG_CalData
    DWORD Scanner3_data_scale[3];
    long Scanner3_data_Shift[3];
};


QPicture buildNlcPic(double k[6]);

class NLCWidget: public QFrame
{
    Q_OBJECT
signals:
    void clicked();
public:
    QPicture m_pic;
    void SetPic(const QPicture &c) { m_pic = c; update(); }

    int heightForWidth(int w) const { return w*0.66; }
    void paintEvent(QPaintEvent *)
    {
        QPainter p(this);
        p.translate(contentsRect().bottomLeft());
        p.scale(contentsRect().width(), -contentsRect().height());
        m_pic.play(&p);
    }
    NLCWidget() {
        setStyleSheet("background-color: rgb(230, 255, 230); border: 1px solid #0000DF; border-radius: 3px;");

        setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Minimum);
        QSizePolicy p = sizePolicy();
        p.setHeightForWidth(true);
        setSizePolicy(p);
        setMinimumSize(10, 13);
    }
    //void mouseDoubleClickEvent(QMouseEvent *) { emit clicked(); }
    void mousePressEvent(QMouseEvent *) { emit clicked(); }
};

#endif // XYZCALIBRATIONS_HPP
